//
// Type definitions shared by the terrain format reader modules
//

#ifndef TFORMATTYPESH
#define TFORMATTYPESH

#include <math.h>

#ifndef PI
#define PI		3.141592653589793238
#endif
#ifndef PI2
#define PIf 	3.141592653589793238f
#define PI2 	6.283185307179586476
#define PI2f 	6.283185307179586476f
#define PID2 	1.570796326794896619
#define PID2f 	1.570796326794896619f
#endif

#ifndef NULL
#define NULL    0
#endif

class FPoint3 {
public:
	FPoint3() { x = y = z = 0.0f; }
	FPoint3(float fx, float fy, float fz) { x=fx; y=fy; z=fz; }

	float Length() const { return sqrtf(x*x+y*y+z*z); }
	void Normalize() { float s = 1.0f/Length(); x*=s; y*=s; z*=s; }
	void Set(float fx, float fy, float fz) { x=fx; y=fy; z=fz; }
	FPoint3 &operator=(const FPoint3 &v) { x = v.x; y = v.y; z = v.z; return *this; }

	FPoint3 operator +(const FPoint3 &v) const { return FPoint3(x+v.x, y+v.y, z+v.z); }
	FPoint3 operator -(const FPoint3 &v) const { return FPoint3(x-v.x, y-v.y, z-v.z); }
	FPoint3 operator *(float s) const { return FPoint3(x*s, y*s, z*s); }

	void operator +=(const FPoint3 &v) { x+=v.x; y+=v.y; z+=v.z; }
	void operator -=(const FPoint3 &v) { x-=v.x; y-=v.y; z-=v.z; }
	void operator *=(float s) { x*=s; y*=s; z*=s; }

	float x, y, z;
};

class FPoint2 {
public:
	FPoint2() { x = y = 0.0f; }
	FPoint2(float fx, float fy) { x=fx; y=fy; }

	float Length() const { return sqrtf(x*x+y*y); }
	void Normalize() { float s = 1.0f/Length(); x*=s; y*=s; }
	void Set(float fx, float fy) { x=fx; y=fy; }
	FPoint2 &operator=(const FPoint2 &v) { x = v.x; y = v.y; return *this; }

	FPoint2 operator +(const FPoint2 &v) const { return FPoint2(x+v.x, y+v.y); }
	FPoint2 operator -(const FPoint2 &v) const { return FPoint2(x-v.x, y-v.y); }
	FPoint2 operator *(float s) const { return FPoint2(x*s, y*s); }

	void operator +=(const FPoint2 &v) { x+=v.x; y+=v.y; }
	void operator -=(const FPoint2 &v) { x-=v.x; y-=v.y; }
	void operator *=(float s) { x*=s; y*=s; }

	float x, y;
};

typedef struct DPoint2 { double x, y; } DPoint2;
//typedef struct IPoint2 { int x, y; } IPoint2;

class IPoint2 {
public:
	IPoint2() {}
	IPoint2(int ix, int iy) { x=ix; y=iy; }

	float Length() const { return sqrtf((float)x*x + (float)y*y); }
	void Set(int ix, int iy) { x=ix; y=iy; }
	IPoint2 &operator=(const IPoint2 &v) { x = v.x; y = v.y; return *this; }

	IPoint2 operator +(const IPoint2 &v) const { return IPoint2(x+v.x, y+v.y); }
	IPoint2 operator -(const IPoint2 &v) const { return IPoint2(x-v.x, y-v.y); }
	IPoint2 operator *(int s) const { return IPoint2(x*s, y*s); }

	void operator +=(const IPoint2 &v) { x+=v.x; y+=v.y; }
	void operator -=(const IPoint2 &v) { x-=v.x; y-=v.y; }
	void operator *=(int s) { x*=s; y*=s; }

	int x, y;
};

#endif
