//
// This code was extracted from GCTPC, a public-domain Cartographic
//	Transformation Package which came from the USGS
//

#include "dem/utm_cproj.h"

#define MAX_VAL 4
#define MAXLONG 2147483647.
#define DBLLONG 4.61168601e18

void sincos(double val, double *sin_val, double *cos_val)
{
	*sin_val = sin(val);
	*cos_val = cos(val);
}

/* Function computes the value of M which is the distance along a meridian
   from the Equator to latitude phi.
------------------------------------------------*/
double mlfn(e0,e1,e2,e3,phi)
 double e0,e1,e2,e3,phi;
{
	return(e0*phi-e1*sin(2.0*phi)+e2*sin(4.0*phi)-e3*sin(6.0*phi));
}

/* Functions to compute the constants e0, e1, e2, and e3 which are used
   in a series for calculating the distance along a meridian.  The
   input x represents the eccentricity squared.
----------------------------------------------------------------*/
double e0fn(x) double x;{return(1.0-0.25*x*(1.0+x/16.0*(3.0+1.25*x)));}
double e1fn(x) double x;{return(0.375*x*(1.0+0.25*x*(1.0+0.46875*x)));}
double e2fn(x) double x;{return(0.05859375*x*x*(1.0+0.75*x));}
double e3fn(x) double x;{return(x*x*x*(35.0/3072.0));}

/* Function to return the sign of an argument
  ------------------------------------------*/
int sign(double x) { if (x < 0.0) return(-1); else return(1);}

/* Function to adjust a longitude angle to range from -180 to 180 radians
   added if statments 
  -----------------------------------------------------------------------*/
double adjust_lon(double x) /* Angle in radians			*/
{
	long count = 0;
	for(;;)
	{
		if (fabs(x)<=PI)
			 break;
		else
		if (((long) fabs(x / PI)) < 2)
			x = x-(sign(x) *TWO_PI);
		else
		if (((long) fabs(x / TWO_PI)) < MAXLONG)
		{
			x = x-(((long)(x / TWO_PI))*TWO_PI);
		}
		else
		if (((long) fabs(x / (MAXLONG * TWO_PI))) < MAXLONG)
		{
			x = x-(((long)(x / (MAXLONG * TWO_PI))) * (TWO_PI * MAXLONG));
		}
		else
		if (((long) fabs(x / (DBLLONG * TWO_PI))) < MAXLONG)
		{
			x = x-(((long)(x / (DBLLONG * TWO_PI))) * (TWO_PI * DBLLONG));
		}
		else
			x = x-(sign(x) *TWO_PI);
		count++;
		if (count > MAX_VAL)
			break;
	}

	return(x);
}

/* Function to eliminate roundoff errors in asin
----------------------------------------------*/
double asinz (double con)
{
	if (fabs(con) > 1.0)
	{
		if (con > 1.0)
			con = 1.0;
		else
			con = -1.0;
	}
	return(asin(con));
}

//
// Spheroids
//
//	 0: Clarke 1866 (default)	 1: Clarke 1880
//	 2: Bessel					 3: International 1967
//	 4: International 1909		 5: WGS 72
//	 6: Everest					 7: WGS 66
//	 8: GRS 1980				 9: Airy
//	10: Modified Everest		11: Modified Airy
//	12: WGS 84					13: Southeast Asia
//	14: Australian National		15: Krassovsky
//	16: Hough					17: Mercury 1960
//	18: Modified Mercury 1968	19: Sphere of Radius 6370997 meters

double g_major[20] = {6378206.4, 6378249.145, 6377397.155, 6378157.5,
                    6378388.0, 6378135.0, 6377276.3452, 6378145.0,
                    6378137.0, 6377563.396, 6377304.063, 6377340.189,
                    6378137.0, 6378155.0, 6378160.0, 6378245.0,
                    6378270.0, 6378166.0, 6378150.0, 6370997.0};

double g_minor[20] = {6356583.8, 6356514.86955, 6356078.96284, 6356772.2,
                    6356911.94613, 6356750.519915, 6356075.4133,
                    6356759.769356, 6356752.31414, 6356256.91,
                    6356103.039, 6356034.448, 6356752.314245,
                    6356773.3205, 6356774.719, 6356863.0188,
                    6356794.343479, 6356784.283666, 6356768.337303,
                    6370997.0};
