///////////////////////////////////////////////////////////////////////////////
// FFT.cpp
//-----------------------------------------------------------------------------
//
// Fast forward/inverse Fourier transform over n dimensions.
//-----------------------------------------------------------------------------
//
// Project....: Terrain Generator Demo
// Compiler...: Microsoft Visual C++ 6.0
// Platform...: Microsoft Windows 95/98/2000
///////////////////////////////////////////////////////////////////////////////
#include "util/ddgutil.h"
#include "ext/ddgsptrr.h"

void FFT(float * p_complex, int * p_extents, int dimensions, bool inverse)
{
	int i, n, nrem, ip1, ip2, ip3, i2rev, i3rev, i1, i2, i3, ibit, ifp1, ifp2, k1, k2, nprev, ntotal;
	int isign = inverse ? -1 : +1;
	float theta, wtemp, wpr, wpi, tempr, tempi, wr, wi;

	ntotal = 1;
	for (i = 0; i < dimensions; i++) ntotal *= p_extents[i];

	nprev = 1;
	for (i = dimensions - 1; i >= 0; i--)
	{
		n     = p_extents[i];
		nrem  = ntotal / (n * nprev);
		ip1   = nprev << 1;
		ip2   = ip1 * n;
		ip3   = ip2 * nrem;
		i2rev = 1;

		for (i2 = 1; i2 <= ip2; i2 += ip1)
		{
			if (i2 < i2rev)
			{
				for (i1 = i2; i1 <= i2 + ip1 - 2; i1 += 2)
				{
					for (i3 = i1; i3 <= ip3; i3 += ip2) 
					{
						i3rev = i2rev + i3 - i2;
						float t = p_complex[i3];
						p_complex[i3] = p_complex[i3rev];
						p_complex[i3rev] = t;
						t = p_complex[i3 + 1];
						p_complex[i3 + 1] = p_complex[i3rev + 1];
						p_complex[i3rev + 1] = t;
					}
				}
			}

			ibit = ip2 >> 1;
			while (ibit >= ip1 && i2rev > ibit)
			{
				i2rev -= ibit;
				ibit >>= 1;
			}
			i2rev += ibit;
		}

		ifp1 = ip1;
		while (ifp1 < ip2)
		{
			ifp2  = ifp1 << 1;
			theta = isign * (float(2.0) * float(M_PI)) / (ifp2 / ip1);
			wtemp = float(sin(float(0.5) * theta));
			wpr   = float(-2.0) * wtemp * wtemp;
			wpi   = float(sin(theta));
			wr	  = float(1.0);
			wi    = float(0.0);
			
			for (i3 = 1; i3 <= ifp1; i3 += ip1)
			{
				for (i1 = i3; i1 <= i3 + ip1 - 2; i1 += 2)
				{
					for (i2 = i1; i2 <= ip3; i2 += ifp2)
					{
						k1 = i2;
						k2 = k1 + ifp1;
						tempr = wr * p_complex[k2]     - wi * p_complex[k2 + 1];
						tempi = wr * p_complex[k2 + 1] + wi * p_complex[k2];
						p_complex[k2]      = p_complex[k1]     - tempr;
						p_complex[k2 + 1]  = p_complex[k1 + 1] - tempi;
						p_complex[k1]     += tempr;
						p_complex[k1 + 1] += tempi;
					}
				}
				wtemp = wr;
				wr = wtemp * wpr -    wi * wpi + wr;
				wi = wi    * wpr + wtemp * wpi + wi;
			}
			ifp1 = ifp2;
		}

		nprev *= n;
	}
}