///////////////////////////////////////////////////////////////////////////////
// HeightField.h
//-----------------------------------------------------------------------------
//
// This file contains the implementation of the height field class.
//-----------------------------------------------------------------------------
//
// Project....: Terrain Generator Demo
// Compiler...: Microsoft Visual C++ 6.0
// Platform...: Microsoft Windows 95/98/2000
///////////////////////////////////////////////////////////////////////////////
#ifndef __HEIGHTFIELD_H__
#define __HEIGHTFIELD_H__



class CHeightField
{
    // Construction & Destruction
    public:
        CHeightField();
        virtual ~CHeightField();

    // Height Field Generators
    public:
        bool Destroy();

        bool Create(int width, int height, float initialHeight);

        bool CreateRidged(
            int width,
            int height,
            float H,
            float lacunarity,
            float octaves,
            float offset,
            float gain);

        bool CreateSpectral(int level, float smoothness);

        bool CreateMidpoint(int level, float sigma, float H, bool randomAdditions, bool island);

    private:
        bool CreateMDPMountain(int level, float sigma, float H, bool randomAdditions);
        bool CreateMDPIsland(int level, float sigma, float H, bool randomAdditions);

    // Transformations
    public:
        void Scale(float scaleFactor);
        void PowerScale(float exponent);

    // Miscellaneous
    public:
        void FindMinMaxHeights();

        float GetMinHeight()
        { return m_minHeight; }

        float GetMaxHeight()
        { return m_maxHeight; }

        int GetFieldWidth()
        { return m_fieldWidth; }

        int GetFieldHeight()
        { return m_fieldHeight; }

        float * GetField()
        { return mp_field; }

    // Private Member Data
    private:
        float *     mp_field;

        int         m_fieldSize;
        int         m_fieldWidth;
        int         m_fieldHeight;

        float       m_minHeight;
        float       m_maxHeight;

};

#endif