///////////////////////////////////////////////////////////////////////////////
// Noise.h
//-----------------------------------------------------------------------------
//
// This file contains source code for White Gaussian Noise, and Perlin Noise
// over 1, 2, and 3 dimensions.
//-----------------------------------------------------------------------------
//
// Project....: Terrain Generator Demo
// Compiler...: Microsoft Visual C++ 6.0
// Platform...: Microsoft Windows 95/98/2000
///////////////////////////////////////////////////////////////////////////////
#ifndef __NOISE_H__
#define __NOISE_H__



///////////////////////////////////////////////////////////////////////////////
// White Gaussian Noise
///////////////////////////////////////////////////////////////////////////////

class CWhiteGaussianNoise
{
    public:
        CWhiteGaussianNoise(); 
        CWhiteGaussianNoise(unsigned int numSamples);

    public:
        float Noise();

    unsigned int        m_numSamples;
    float       m_gaussAdd;
    float       m_gaussFactor;
};

///////////////////////////////////////////////////////////////////////////////
// Perlin Solid Noise over 1, 2, and 3 dimensions
///////////////////////////////////////////////////////////////////////////////

class CPerlinNoise
{
    public:
        CPerlinNoise();
        virtual ~CPerlinNoise();

    public:
        float Noise1(float x);
        float Noise2(float x, float y);
        float Noise3(float x, float y, float z);

    private:
        bool Initialize();
        void Normalize2D(float * p_v);
        void Normalize3D(float * p_v);
        void Setup(float v, int& b0, int& b1, float& r0, float& r1);

        float Lerp(float t, float t0, float t1)
        { return t0 + t * (t1 - t0); }

        float Spline(float t)
        { return t * t * (float(3) - float(2) * t); }

    private:
        typedef float    OiDim1;
        struct OiDim2 { float v[2]; };
        struct OiDim3 { float v[3]; };

        int *       mp_indices;
        OiDim1 *    mp_dim1;
        OiDim2 *    mp_dim2;
        OiDim3 *    mp_dim3;
};

#endif