/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe, Klaus Hartmann
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

// ---------------------------------------------------------------------
//
// DDG definitions.
//
#include "ext/ddgeco.h"
#include "struct/ddgimage.h"
#include "ext/ddgtxtsyn.h"

// Make a texture that is TEXTURE_SCALE_X by TEXTURE_SCALE_Y times larger than
// the height field, and begin synthesizing process.

// This function invokes the actual texture synthesizer.
bool ddgTextureSynthesizer::synthesizeSurfaceTexture(ddgHeightMap * hfl, ddgImage * p_image, ddgVector2 vSize, ddgImage *ecoMap, ddgStr *paramFile)
{
    ddgEcoParams    ecoParams;

	// $TODO: use paramFile to override default ecoSystem data.

	p_image->allocate(1+(hfl->rows()-1) * vSize[0], 1+(hfl->cols()-1) * vSize[1], 3);

    // Pointer to the ddgHeightMap object that contains the
    // elevation data.
    ecoParams.mp_heightMap = hfl;

    // Pointer to the ddgImage object that receives the surface
    // texture. The image needs to be pre-allocated as a 24 bpp
    // RGB image. The size of the image can be arbitrarily chosen.
    // Hence, this synthesis is also good to generate mip-map
    // levels.
    ecoParams.mp_baseTexture = p_image;

    // A pointer to the list of ecosystems, and the number of
    // ecosystems in this list
    ecoParams.mp_ecosystems   = NULL;	// Use defaults.
    ecoParams.m_numEcosystems = 0;

    // A pointer to the list of directional light sources, and
    // the number of light sources in this list.
    ecoParams.mp_lights   = NULL;		// Use defaults.
    ecoParams.m_numLights = 0;

    // An optional pointer to a ddgImage. If specified (not NULL),
    // this image will receive a map of ecosystems. It is an 8 bpp
    // image and has the same size as the height field. The information
    // stored at each pixel is equal to the "m_user" parameter
    // in the corresponding ddgEcosystem structure. The value 0xFF is
    // stored in this map, if no matching ecosystem was found.
    // The texture generator will allocate this buffer for you.
    ecoParams.mp_ecoMap = ecoMap ? ecoMap : NULL;

    // Apply a simple box-filter. This is only a temporary solution.
    ecoParams.m_filter = false;

    // Random additions. Adds some fractal Brownian motion to each elevation.
    // This is only a temporary solution.
    ecoParams.m_randomAdditions = false;

    // Now synthesize the texture
    ddgEco  eco;
    eco.Synthesize(ecoParams);

//  p_image->writeTGA("surface.tga");
//  p_ecoMap->writeTGA("ecomap.tga");

	return ddgSuccess;
}

