/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <stdio.h>
#include <string.h>

#ifdef WIN32
#include <io.h>
#include <fcntl.h>
#endif

#include "util/ddgerror.h"
#include "util/ddgutil.h"
#include "loader/ddgrla.h"
// ----------------------------------------------------------------------
typedef struct {
	short left,right,bottom,top;
} WINDOW_S;

typedef struct {
	WINDOW_S window;
	WINDOW_S active_window;
	short	frame;
	short	storage_type;
	short	num_chan;
	short	num_matte;
	short	num_aux;
	short	revision;
	char	gamme[16];
	char	red_pri[24];
	char	green_pri[24];
	char	blue_pri[24];
	char	white_pri[24];
	long	job_num;
	char	name[128];
	char	desc[128];
	char	program[64];
	char	machine[32];
	char	user[32];
	char	date[20];
	char	aspect[24];
	char	aspect_ratio[8];
	char	chan[32];
	short	field;
	char	time[12];
	char	filter[32];
	short	chan_bits;
	short	matte_type;
	short	matte_bits;
	short	aux_type;
	short	aux_bits;
	char	aux[32];
	char	space[36];
	long	next;
} HEADER;

// Format
//
//  HEADER			740 bytes.
//
//  OFFSET TABLE	4 * activewindow y_resolution) bytes
//
//  IMAGE DATA		variable length.

	
bool ddgRLALoader::load(const char *filename)
{
	char *fullName = ddgFileHelper::getFullFileName(filename);
	FILE *fptr = fullName && fullName[0] ? fopen(fullName,"rb") : 0;
	if (!fptr)
	{
		ddgErrorSetStr(FileRead,"Could not open file " << ((char *) (fullName ? fullName : "(null)")));
		goto error;
	}

	HEADER header;

	// Read the header, 740 bytes.
	if (fread(&header,sizeof(HEADER),1,fptr) < 740)
	{
		ddgErrorSetStr(FileRead,"Could not header of RLA file " << ((char *) (fullName ? fullName : "(null)")));
		if (feof(fptr))
		{
			ddgErrorSetStr(FileAccess,"Reached end of file! " << ((char *) (fullName ? fullName : "(null)")));
		}
		goto error;
	}

	cerr << "Created by:" << header.program << ", " << header.user << ", " << header.machine << endl;
	fclose(fptr);
	return ddgSuccess;

error:
	if (fptr)
		fclose(fptr);

	ddgError::report();
	return ddgFailure;
}