/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "math/ddgmatrx.h"

ddgMatrix4& ddgMatrix4::operator *= ( const ddgMatrix4& x )
{	ddgMatrix4 c(*this);

	m[0].v[0] = c.m[0].v[0] * x.m[0].v[0] + c.m[1].v[0] * x.m[0].v[1] + c.m[2].v[0] * x.m[0].v[2] + c.m[3].v[0] * x.m[0].v[3];
    m[0].v[1] = c.m[0].v[1] * x.m[0].v[0] + c.m[1].v[1] * x.m[0].v[1] + c.m[2].v[1] * x.m[0].v[2] + c.m[3].v[1] * x.m[0].v[3];
	m[0].v[2] = c.m[0].v[2] * x.m[0].v[0] + c.m[1].v[2] * x.m[0].v[1] + c.m[2].v[2] * x.m[0].v[2] + c.m[3].v[2] * x.m[0].v[3];
	m[0].v[3] = c.m[0].v[3] * x.m[0].v[0] + c.m[1].v[3] * x.m[0].v[1] + c.m[2].v[3] * x.m[0].v[2] + c.m[3].v[3] * x.m[0].v[3];
  
	m[1].v[0] = c.m[0].v[0] * x.m[1].v[0] + c.m[1].v[0] * x.m[1].v[1] + c.m[2].v[0] * x.m[1].v[2] + c.m[3].v[0] * x.m[1].v[3];
	m[1].v[1] = c.m[0].v[1] * x.m[1].v[0] + c.m[1].v[1] * x.m[1].v[1] + c.m[2].v[1] * x.m[1].v[2] + c.m[3].v[1] * x.m[1].v[3];
	m[1].v[2] = c.m[0].v[2] * x.m[1].v[0] + c.m[1].v[2] * x.m[1].v[1] + c.m[2].v[2] * x.m[1].v[2] + c.m[3].v[2] * x.m[1].v[3];
	m[1].v[3] = c.m[0].v[3] * x.m[1].v[0] + c.m[1].v[3] * x.m[1].v[1] + c.m[2].v[3] * x.m[1].v[2] + c.m[3].v[3] * x.m[1].v[3];
  
	m[2].v[0] = c.m[0].v[0] * x.m[2].v[0] + c.m[1].v[0] * x.m[2].v[1] + c.m[2].v[0] * x.m[2].v[2] + c.m[3].v[0] * x.m[2].v[3];
	m[2].v[1] = c.m[0].v[1] * x.m[2].v[0] + c.m[1].v[1] * x.m[2].v[1] + c.m[2].v[1] * x.m[2].v[2] + c.m[3].v[1] * x.m[2].v[3];
	m[2].v[2] = c.m[0].v[2] * x.m[2].v[0] + c.m[1].v[2] * x.m[2].v[1] + c.m[2].v[2] * x.m[2].v[2] + c.m[3].v[2] * x.m[2].v[3];
	m[2].v[3] = c.m[0].v[3] * x.m[2].v[0] + c.m[1].v[3] * x.m[2].v[1] + c.m[2].v[3] * x.m[2].v[2] + c.m[3].v[3] * x.m[2].v[3];
  
	m[3].v[0] = c.m[0].v[0] * x.m[3].v[0] + c.m[1].v[0] * x.m[3].v[1] + c.m[2].v[0] * x.m[3].v[2] + c.m[3].v[0] * x.m[3].v[3];
	m[3].v[1] = c.m[0].v[1] * x.m[3].v[0] + c.m[1].v[1] * x.m[3].v[1] + c.m[2].v[1] * x.m[3].v[2] + c.m[3].v[1] * x.m[3].v[3];
	m[3].v[2] = c.m[0].v[2] * x.m[3].v[0] + c.m[1].v[2] * x.m[3].v[1] + c.m[2].v[2] * x.m[3].v[2] + c.m[3].v[2] * x.m[3].v[3];
	m[3].v[3] = c.m[0].v[3] * x.m[3].v[0] + c.m[1].v[3] * x.m[3].v[1] + c.m[2].v[3] * x.m[3].v[2] + c.m[3].v[3] * x.m[3].v[3];                                                                  

	return *this;

}

/*---------------------------------------------------
Transpose the matrix such that all values
m[i].v[j] = m[j].v[i] (mapped across the diagonal)
----------------------------------------------------*/
void ddgMatrix4::transpose()
{
	ddgMatrix4 a(*this);


	m[0].v[1] = a.m[1].v[0];
	m[0].v[2] = a.m[2].v[0];
	m[0].v[3] = a.m[3].v[0];
	m[1].v[2] = a.m[2].v[1];
	m[1].v[3] = a.m[3].v[1];
	m[2].v[3] = a.m[3].v[2];

	m[1].v[0] = a.m[0].v[1];
	m[2].v[0] = a.m[0].v[2];
	m[3].v[0] = a.m[0].v[3];
	m[2].v[1] = a.m[1].v[2];
	m[3].v[1] = a.m[1].v[3];
	m[3].v[2] = a.m[2].v[3];
}

/*-------------------------------------------------------------
Calculate the inverse of a matrix.

  Such that A => A*A' = A' *A = I (indenty)
--------------------------------------------------------------*/
void ddgMatrix4::invert()
{
	//invert matrix	
	ddgMatrix4 b = *this;

	b.m[0].v[0] = m[0].v[0];
  	b.m[0].v[1] = m[1].v[0];
	b.m[0].v[2] = m[2].v[0];
	b.m[1].v[0] = m[0].v[1];
	b.m[1].v[1] = m[1].v[1];
	b.m[1].v[2] = m[2].v[1];
	b.m[2].v[0] = m[0].v[2];
	b.m[2].v[1] = m[1].v[2];
	b.m[2].v[2] = m[2].v[2];
  
	/* the new displacement vector is:  d' = -(R^-1) * d */
	if (m[3].v[0] != 0 || m[3].v[0] != 0 || m[3].v[0] != 0)
	{
		b.m[3].v[0] = -( m[3].v[0] * b.m[0].v[0] + m[3].v[1] * b.m[1].v[0] + m[3].v[2] * b.m[2].v[0] );
		b.m[3].v[1] = -( m[3].v[0] * b.m[0].v[1] + m[3].v[1] * b.m[1].v[1] + m[3].v[2] * b.m[2].v[1] );
		b.m[3].v[2] = -( m[3].v[0] * b.m[0].v[2] + m[3].v[1] * b.m[1].v[2] + m[3].v[2] * b.m[2].v[2] );
	}
	else
	{
		b.m[3].v[0] = 0;
		b.m[3].v[1] = 0;
		b.m[3].v[2] = 0;
	}
	/* the rest is equal */
	b.m[0].v[3] = b.m[1].v[3] = b.m[2].v[3] = 0.0f; 

	b.m[3].v[3] = 1.0f;

	*this = b;
}
