/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _ddgCloud_Class
#define _ddgCloud_Class

#include "util/ddgclock.h"
#include "struct/ddgdome.h"
#include "render/ddgvbuf.h"
#include "render/ddgtexture.h"
/**
 * A semi transparent cloud texture object which scrolls.
 */
class WEXP ddgCloud : public ddgVBuffer {
    typedef ddgVBuffer super;
	/// Dome object.
	ddgDome			_dome;
	/// Image used for displayed texture.
	ddgImage	*_image;
	/// Number of octaves of noise to use.
	unsigned short	_octaves;
	/// Cloud cover from 0-1. Alpha limit.
	float	_cover;
	/// Cloud Brightness.
	float	_bright;
	/// Cloud day color.
	ddgColor4	_cday;
	/// Cloud day color.
	ddgColor4	_ctwilight;
	/// Cloud day color.
	ddgColor4	_cnight;
	/// Texture drift per system tick.
	ddgVector2 _drift;
	/// The texture object which will hold the image.
	ddgTexture *_texture;
	/// Shared initialization called by constructors.
	_init(ddgImage *i);
public:
	/// Create a Cloud object of a given size.
	ddgCloud( unsigned short textureSize, unsigned int o = 5, float size = 4900  );
	/// Create a Cloud object from a file.
	ddgCloud( ddgImage *i, float size = 4900 );

	/// Initialize the object.
	bool init(ddgContext *ctx );

	/// Render the cloud object.
	bool draw(ddgContext *c );
	/// Return a pointer to the cloud image.
	ddgImage *image(void) { return _image;}
	/// Set the cloud color.
	void color( ddgColor4 *d, ddgColor4 *t, ddgColor4 *n)
	{ _cday.set(d); _ctwilight.set(t); _cnight.set(n); }
	/// Drift.
	void drift(  float dx, float dy ) { _drift.set(dx,dy); }
	/// Set the Texture
	void texture( ddgTexture *t ) { _texture = t; }
	/// Return the Texture
	ddgTexture * texture( void ) { return _texture; }
};
#endif

