/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _ddgFrameBuffer_Class_
#define _ddgFrameBuffer_Class_

#include "struct/ddgimage.h"
#include "render/ddgcamera.h"

/**
 * A convenience object to store a 2D image.
 * Images can be read from .rgb and TGA files.
 * An image can store an arbitrary 2D set of data.
 */
class WEXP ddgFrameBuffer : public ddgImage {

public:
	/**
	 *  Create an image but don't allocate memory.
	 * ch is the number of channels (1-4).
	 * The t flag is true if this is an OpenGL texture which needs to be a power of 2.
	 */
	ddgFrameBuffer( unsigned short ch = 4)
		: ddgImage(ch) {}
	/// Create an image and allocate memory.
	ddgFrameBuffer( unsigned short r, unsigned short c, unsigned short ch = 4)
		: ddgImage(r,c,ch) { }

	/** Copy the Z-buffer from the screen into the image.
	 *  Allocates image to match screen size.
	 */
	bool readZBuffer( ddgCamera *c );
	/** Copy the Frame-buffer from the screen into the image.
	 *  Allocates image to match screen size.
	 */
	bool readFrameBuffer( ddgCamera *c );
	/** Copy the Alpha-buffer from the screen into the image.
	 *  Allocates image to match screen size.
	 */
	bool readAlphaBuffer( ddgCamera *c );
};

#endif
