/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _ddgHistograph_Class_
#define _ddgHistograph_Class_

#include "struct/ddgcolor.h"
#include "struct/ddghist.h"
#include "render/ddguiobj.h"

/**
 * Graphic Overlay rendering object which can be
 * used to render 2D histogram onto the canvas.
 */
class WEXP ddgHistograph : public ddgUIObj
{
    typedef ddgUIObj super;
	/// Histogram object
	ddgHistogram	_histogram;
	/// Width and Height of histogram as percentage of 100%.
	unsigned int	_width;
	unsigned int	_height;
public:
	/// Blend texture with background.
	bool		blend;
	/// Background color default black.
	ddgColor4	background;
	/// Foreground color default white.
	ddgColor4	foreground;
	/// Create a text object at a given canvas location.
	ddgHistograph(unsigned int s, unsigned int x = 0, unsigned int y = 0, unsigned int w = 100, unsigned int h = 0)
		: _histogram(s), ddgUIObj(x,y),
		  blend(true), foreground(255,255,255,255), background(0,0,0,255),
		  _width(w), _height(h) {}
	/// Initialize the font data.
	bool init(ddgContext *ctx);
	/// Render the histograph.
    bool draw(  ddgContext *ctx );
	/// Return the histogram object.
	ddgHistogram *histogram(void) { return &_histogram; }
};

#endif
