/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _ddgKeyControl_Class
#define _ddgKeyControl_Class

#include "struct/ddgcntrl.h"
#include "render/ddgviewport.h"

class WEXP ddgKeyControl : public ddgControl
{
	typedef ddgControl super;
    /// Internal variable for camera position.
    float				_beginX;
    /// Internal variable for camera position.
    float				_beginY;
    /// Internal variable to indicate if object is moving.
    bool				_moving;
    /// The movement of the camera multiplier.
    ddgVector3			_delta;
    /// Distance a single step forward should be.
    float   _stepForward;
    /// Distance a single step to the right should be.
    float   _stepRight;
protected:
	/// Event handler
	ddgEventHandler		_eventHandler;

public:
	/// Constructor.
	ddgKeyControl(ddgVector3 *position = NULL, ddgVector3 *orientation = NULL );
	/**
	 *  Update the current position and orientation.
	 *  Returns true if position or orientation was modified.
	 */
	bool update(void);
	/// Modify the external force moving the object.
	void delta( ddgVector3* d) { _delta.set( d); }

    /// Callback which is called to initialize the camera.
    bool init(void);
    /// Event handler, returns true if a change occurred.
    bool mouse(int button, int state, int x, int y);
    /// Event handler, returns true if a change occurred.
    bool mouseMotion(int x, int y);    
    /// Event handler, returns true if a change occurred.
    bool specialKey(int key, int x, int y);
    /// Event handler, returns true if a change occurred.
    bool key(unsigned char key, int x, int y);
    /// Event handler, returns true if a change occurred.
    static bool smouse(void *obj, int button, int state, int x, int y);
    /// Event handler, returns true if a change occurred.
    static bool smouseMotion(void *obj, int x, int y);    
    /// Event handler, returns true if a change occurred.
    static bool sspecialKey(void *obj, int key, int x, int y);
    /// Event handler, returns true if a change occurred.
    static bool skey(void *obj, unsigned char key, int x, int y);
	/// Attach control to recieve I/O from a viewport.
	void attach( ddgViewport *vp );
};

#endif
