/*
    Copyright (C) 1997, 1998, 1999 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _ddgMeshRender_Class_
#define _ddgMeshRender_Class_

#include "render/ddgvbuf.h"
#include "render/ddgtexture.h"

class ddgTBinTree;
class ddgTBinMesh;
class ddgHeightMap;

/** This class takes a TBinMesh and recalculates it (Used for profiling).
 *  It allows all calculation to be done at start of scene drawing even though
 *  terrain may be rendered at the end.
 */
class WEXP ddgMeshRender : public ddgVBuffer
{
    typedef ddgVBuffer super;
	/// The mesh that needs to be calculated.
	ddgTBinMesh	*_mesh;
	/// Split texture array.
	ddgTexture	**_texture;
	/// Do we calculate mesh colors?
	bool		_meshcolor;
	/// Colours used at various altitudes.
	ddgColor3 _cliff, _beach, _grass, _trees, _rock, _snow;
	/// Angle/Altitude at which color takes effect.
	float	_cliffangle, _beachalt, _grassalt, _treealt, _rockalt, _snowalt;
	/// Height map for texture synthesis.
	ddgHeightMap *_height;
	/// Texture file.
	ddgStr		_tfile;
	/// Ecomap output file.
	ddgStr		_efile;
	/// Size of the base mesh.
	int			_size;
	/// Scale of texture to synthesize (0 => do not synthesize).
	int			_scale;
	/// Planet Curvature
	float		_curvature;
public:
	/// Constructor.
	ddgMeshRender( ddgTBinMesh *m, ddgHeightMap *height = NULL, ddgStr *tfile = NULL, ddgStr *efile = NULL, int size = 0, int scale = 0, bool meshcolor = false );
	/// Destructor.
	~ddgMeshRender( );
	/// Initialize the class.
	bool init(ddgContext *ctx );
	/// Recalculate the mesh
	bool draw(ddgContext *ctx );
    /// Push a triangle into the buffer.
    bool drawTriangle( ddgTBinTree *bt, ddgVBIndex tvc );
    /// Draw a wire frame triangle.
    bool drawTriangleWire( ddgTBinTree *bt, ddgVBIndex tvc );
	/// Set the curvature
	void curvature(float c) { _curvature = c; }
	/// Selected triangle to be rendered in yellow.
	ddgVBIndex selected;

};

#endif
