/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "struct/ddggroup.h"
#include "struct/ddgcolor.h"
#include "render/ddgtexture.h"

/**
 * A Particle Cloud object.
 */

// Create A Structure For Particle
class WEXP ddgParticle
{
	friend class ddgParticleCloud;
	/// Particle life
	float	_life;
	/// Fade speed
	float	_fade;
	/// Position
	ddgVector3	p;  
	/// Direction
	ddgVector3	d;
};


class WEXP ddgParticleCloud : public ddgNode 
{
public:
	/// Enumeration of tree types.
	enum ddgParticleType{ flare = 0, star, circle };				
private:
	typedef ddgNode super;

	/// Cloud size
	int		_size;
	/// Particle set
	ddgParticle *_particle;	
	/// Texture to use for image
	ddgTexture	*_texture;
	/// Name of texture image.
	ddgParticleType		_type;
public:
	/// Force
	ddgVector3	force;
	/// Drag.
	float		drag;
	/// Color
	ddgColor3	color;

	/// Constructor.
	ddgParticleCloud( ddgParticleType t, int s );
	/// Destructor.
	virtual ~ddgParticleCloud();

	/// Initialize particle cloud
    bool init( ddgContext *ctx );
		
    /// Draw particle cloud    
    bool draw( ddgContext *ctx);


};

