/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _ddgReflection_Class_
#define _ddgReflection_Class_

#include "struct/ddggroup.h"

/**
 * This class uses a stencil buffer to render a number
 * of objects reflected.
 */
class WEXP ddgReflection : public ddgGroup {
    typedef ddgGroup super;
protected:
	/// Polygon describing the reflective surface.
	ddgVector3			*_polygon;

public:
	/// Create a flat Ground object of a given size with a given color.
	ddgReflection( void );
	/// Destructor.
	~ddgReflection( void );
	/// Initialize the object.
	bool init(ddgContext *ctx );
	/// Render the ground object.
	bool draw(ddgContext *ctx );
};

/**
 * This class defines a clip plane, any objects added to this node are
 * affected by the clip plane.
 */
class WEXP ddgClipper : public ddgGroup {
	typedef ddgGroup super;
	/// Private identified.
	int	_id;
public:
	// The clip plane defined in object space.
	ddgPlane3	plane;
	/// Create a flat clipping plane.
	ddgClipper( void );
	/// Destructor.
	~ddgClipper( void );
	/// Initialize the object.
	bool init(ddgContext *ctx );
	/// Render the ground object.
	bool draw(ddgContext *ctx );
};

#endif

