/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _ddgStar_class_
#define _ddgStar_class_

#include "util/ddgclock.h"
#include "math/ddgvec.h"
#include "struct/ddgcolor.h"
#include "struct/ddgnode.h"
#include "render/ddgfacet.h"

/// Maximum number of stars in the StarClass.
unsigned int const ddgStarClass_max = 20000;

/// Star object maintained by StarClass.
class WEXP ddgStar {
public:
	/// Face to which this star is assigned.
	unsigned char facet;
	/// Stars brightness.
	unsigned char brightness;
	/// Location of star in space.
	ddgVector3		pos;
	/// Function to sort with.
	static int compare(const void *s1, const void *s2);
};

/// A set of stars.
class WEXP ddgStarClass : public ddgClippedFacetSet {
    typedef ddgClippedFacetSet super;
	/// Number of stars.
	unsigned int _starCount;
	/// Set of all stars.
	ddgStar _stars[ddgStarClass_max];
	/// Stars grouped by facet.
	unsigned int *_group;
	/// A colour set to store a range of star colors.
	ddgColorSet	_colorset;
	/// Use smooth points.
	bool	_smooth;
	/// Array of vertex buffers one for each facet.
	ddgVBuffer	**_facets;
public:
	/// Scale of the star set.
	float	scale;
	/// Constructor.
	ddgStarClass(ddgStr *file, unsigned int l = 1);
	/// Destructor.
	~ddgStarClass();
	/// Render the stars
	bool draw( ddgContext *ctx);
	/// Initialize the all the stars.
	bool init( ddgContext *ctx );
	/// Set smooth or standard stars.
	void smooth( bool s ) { _smooth = s; }
	/// Return the number of stars read in.
	unsigned int starCount(void) { return _starCount; }
	/// Stars below this magnitude won't be rendered.
	float	minMagnitude;
};
#endif
