/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _ddgSun_Class_
#define _ddgSun_Class_

#include "util/ddgclock.h"
#include "struct/ddgnode.h"
#include "render/ddgtexture.h"
#include "render/ddgcamera.h"

class WEXP ddgSun : public ddgNode {
    typedef ddgNode super;
	/// Image used for displayed texture.
	ddgImage	*_image;
	/// The texture object which will hold the image.
	ddgTexture *_texture;
	/// The sun's color at Night.
	ddgColor4 _cnight;
	/// Twilight color.
	ddgColor4 _ctwilight;
	/// Daytime color.
	ddgColor4 _cday;
	// Over all size of the sun on screen (as a percentage of screen width).
	float   _scale;
	/// Sun's position relative to origin.
	ddgVector3	_pos;
	/// The sun's current position in world space.
	ddgVector3 _curpos;
	/// Camera through which sun is visible.
	ddgCamera   *_camera;
public:
	/** Create a Sun object.
	 * size is size of image to use.
	 */
	ddgSun( ddgVector3 pos, ddgCamera *c, unsigned short size = 128);
	/// Destructor
	~ddgSun();
	/// Render the cloud object.
	bool draw(ddgContext *ctx );
	/// Initialize the cloud object.
	bool init( ddgContext *ctx );
	/// Return a pointer to the cloud image.
	ddgImage *image(void) { return _image;}
	/// Set the sun's nightime color.
	void night(ddgColor4 *c) { _cnight.set(c); }
	/// Set the sun's nightime color.
	void twilight(ddgColor4 *c) { _ctwilight.set(c); }
	/// Set the sun's nightime color.
	void day(ddgColor4 *c) { _cday.set(c); }
};


#endif

