/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _ddgTransform_Class_
#define _ddgTransform_Class_

#include "math/ddgvec.h"
#include "struct/ddggroup.h"

/**
 * Class the performs a generic transformation which can include
 * translation, scaling and rotation.  It affects all node which are
 * managed by the transform.
 * $TODO: we should be updating the ctx for this class, or else
 * transformed nodes will not be able to correctly determine visibility etc.
 */
class WEXP ddgTransform : public ddgGroup
{
    typedef ddgGroup super;
public:
	/// The amount to translate world space by.
	ddgVector3		_translate;
	/// The amount to scale world space by.
	ddgVector3		_scale;
	/// The amount to rotate world space by in degrees about x, y and z axis.
	ddgVector3		_rotate;
	/// Flag to indicate if translation is used.
	bool		fTranslate:1;
	/// Flag to indicate if scaling is used.
	bool		fScale:1;
	/// Flag to indicate if rotation is used.
	bool		fRotate:1;
	/**
		Cylindrical mode always faces viewer in XZ plane.
	 */
	bool		fCylindricalBillBoard:1;
	/// Spherical mode always faces viewer in all dimensions.
	bool		fSphericalBillBoard:1;
	/// Constructor
	ddgTransform(void) { fTranslate = fScale = fRotate = fCylindricalBillBoard = fSphericalBillBoard = false; }
	/// Constructor
	ddgTransform(ddgVector3 *t, ddgVector3 *s = NULL, ddgVector3 *r = NULL, ddgNode *d = NULL)
	{
		fTranslate = fScale = fRotate = fCylindricalBillBoard = fSphericalBillBoard = false;
		if (t) translate(t);
		if (s) scale(s);
		if (r) rotate(r);
		if (d) add(d);
	}

	/// Set translation.
	void translate( ddgVector3 *t ) { _translate.set(t); fTranslate = true; }
	/// Set scale.
	void scale( ddgVector3 *s ) { _scale.set(s); fScale = true; }
	/// Set rotation.
	void rotate( ddgVector3 *r ) { _rotate.set(r); fRotate = true; }
	/// Set bill board forces object to face the camera allows rotation about Y axis only.
	void cylindricalBillBoard( void ) { fCylindricalBillBoard = true; }
	/// Set bill board forces object to face the camera..
	void sphericalBillBoard( void ) { fSphericalBillBoard = true; }

	/**
	*  Render this node.
	* Return true if error.
	*/
	bool draw( ddgContext *c );

};

/// Conveniant class to create a translate node.
class WEXP ddgTranslate : public ddgTransform
{
public:
	/// Constructor.
	ddgTranslate( ddgVector3 *t, ddgNode *d = NULL) : ddgTransform(t,0,0,d) {}
};

/// Convenient class to create a scale node.
class WEXP ddgScale: public ddgTransform
{
public:
	/// Constructor.
	ddgScale( ddgVector3 *s, ddgNode *d = NULL) : ddgTransform(0,s,0,d) { }
};

/// Convenient class to create a rotate node.
class WEXP ddgRotate : public ddgTransform
{
public:
	/// Constructor.
	ddgRotate( ddgVector3 *r, ddgNode *d = NULL) : ddgTransform(0,0,r,d) { }
};

#endif

