/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _ddgTree_Class_
#define _ddgTree_Class_

#include "struct/ddggroup.h"
#include "render/ddgvbuf.h"

/**
 * A Tree object.
 */
class WEXP ddgTree : public ddgNode {
	typedef ddgNode super;
	friend class ddgForest;
	friend class ddgForestBlock;
public:
	/// Enumeration of tree types.
	enum ddgTreeType{ oak = 0, pine, fern, birch1, birch2, bush, palm };
private:
	/// Global class initialization flag.
	static bool ddgTreeInit;
	/// Type of tree.
	ddgTreeType _type;
public:
	/// Constructor.
	ddgTree(ddgTreeType t);
	/// Render the tree.
	bool init( ddgContext *ctx );
	/// Render the tree.
	bool draw( ddgContext *ctx);
};

class ddgEcoSystem;
class ddgEcoBlock;
/**
 * A forest can render trees.
 */
class WEXP ddgForest : public ddgGroup {
	typedef ddgGroup super;
	friend class ddgForestBlock;
	/// The eco block maintaining the data for this Forest.
	ddgEcoSystem	*_ecoSystem;
	/// Tree type in this forest.
	ddgTree		*_tree;
public:
	/// Constructor
	ddgForest( ddgTree *t, ddgEcoSystem *e)
		: _ecoSystem(e), _tree(t) {}
	/// Render the forest.
	bool init( ddgContext *ctx );
	/// Render the forest.
	bool draw( ddgContext *ctx);
};

/**
 * A forest can render trees.
 */
class WEXP ddgForestBlock : public ddgVBuffer {
	typedef ddgVBuffer super;
	/// The eco block maintaining the data for this Forest block.
	ddgEcoBlock	*_eb;
	/// The tree type.
	ddgTree	*_tree;
	/// Which of the 2 tree panels is it.
	bool	_ortho;
public:
	/// Constructor
	ddgForestBlock( ddgTree *t, ddgEcoBlock *e, bool o)
		: ddgVBuffer(ddgVArray::quad), _tree(t), _eb(e), _ortho(o) {}
	/// Render the forest.
	bool init( ddgContext *ctx );
	/// Render the forest block.
	bool draw( ddgContext *ctx);
};
#endif
