/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _ddgUIColor_Class_
#define _ddgUIColor_Class_

#include "struct/ddgcolor.h"
#include "render/ddguiobj.h"

/// Abstract class from which to derive UI objects.
class WEXP ddgUIControl : public ddgUIObj
{
    typedef ddgUIObj super;
	/// Global identifier counter.
	static	unsigned int	_gid;
	/// List of all ui objects.
	static  ddgList	*_guicontrollist;
	/// The id for this control
	unsigned int	_id;
public:
	/// Constructor.
	ddgUIControl(unsigned short x, unsigned short y);
	/// Every sub class initializer should call this 1st.
	bool init( void );
	/// Dummy draw method, MUI does the rendering for us.
	bool draw( ddgContext *ctx ) { return false; }
	/// Update the ui due to some change.
	virtual bool update( unsigned int _id ) = 0;
	/// Get id for this control.
	unsigned int id(void) { return _id; }
	/// Get new id.
	static unsigned int genId(void);
	/// Find the control associated with a given id.
    static ddgUIControl* find(unsigned int id);
};

class WEXP ddgUINode : public ddgListNode
{
    typedef ddgListNode super;
public:
	ddgUIControl *uicontrol;
	ddgUINode( ddgUIControl *uic ) { uicontrol = uic; }
};

/**
 * UI color control.
 */
class WEXP ddgUIColor : public ddgUIControl
{
    typedef ddgUIControl super;
	/// The color object managed by this ui.
	union
	{
	ddgColor3	*_color3;
	ddgColor4	*_color4;
	};
	bool	_is4;
	/// Identifiers for callbacks.
	unsigned int		_idred;
	unsigned int		_idgreen;
	unsigned int		_idblue;
	unsigned int		_idalpha;
	/// Mui Object handles.
	void	*_slred;
	void	*_slgreen;
	void	*_slblue;
	void	*_slalpha;
	void	*_labred;
	void	*_labgreen;
	void	*_labblue;
	void	*_labalpha;

public:
	/// Constructor.
	ddgUIColor( unsigned short x, unsigned short y, ddgColor3 *c = NULL);
	/// Constructor.
	ddgUIColor( unsigned short x, unsigned short y, ddgColor4 *c = NULL);
	/// Initialize the color ui.
	bool init(void);
	/// Update the sliders color ui.
    bool update( unsigned int id );
	/// Set the color that this ui should control.
	void color( ddgColor3 *c ) { _color3 = c; _is4 = false; }
	/// Set the color that this ui should control.
	void color( ddgColor4 *c ) { _color4 = c; _is4 = true; }
};

/**
 * UI color viewer.  Shows red,green and blue components as well
 * as the mixed result.
 */
class WEXP ddgUIColorView : public ddgUIObj
{
    typedef ddgUIObj super;
	/// The color object managed by this ui.
	union {
	ddgColor3	*_color3;
	ddgColor4  *_color4;
	};
	/// If we are viewing a 4 component color, this is true.
	bool	_is4;
public:
	/// Constructor.
	ddgUIColorView( unsigned short x, unsigned short y, ddgColor3 *c = NULL)
		: super(x,y) { _color3 = c; _is4 = false; }
	/// Constructor.
	ddgUIColorView( unsigned short x, unsigned short y, ddgColor4 *c = NULL)
		: super(x,y) { _color4 = c; _is4 = true; }
	/// Update the color part ui.
	bool draw( ddgContext *ctx );
	/// Set the color that this ui should control.
	void color( ddgColor3 *c ) { _color3 = c; _is4 = false; }
	/// Set the color that this ui should control.
	void color( ddgColor4 *c ) { _color4 = c; _is4 = true; }
};

/**
 * Manage a list of ui controls for colors.
 */
class WEXP ddgUIColorList : public ddgUIControl
{
public:
	/// Update the color list.
    bool update( unsigned int id );
	/// Initialize the color ui.
	bool init(void);
};

#endif
