/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _ddgVBuffer_Class_
#define _ddgVBuffer_Class_

#include "struct/ddgvarr.h"
#include "render/ddglnode.h"

/**
 * A class supporting vertex array buffers for rendering.
 * the object supports rendering a sub range of the vertex array.
 */
class WEXP ddgVBuffer : public ddgDLNode
 {
private:
    typedef ddgDLNode super;
	/// Start of range.
	unsigned int _rstart;
	///  Number of objects in the range.
	unsigned int _rnum;
public:
	/// Data is held by this object
	ddgVArray	varray;

	/// Create a object but don't allocate any memory.
	ddgVBuffer( ddgVArray::ddgBufType type = ddgVArray::triangle) : varray(type) ,_rstart(0), _rnum(0) { }
	/// Render the buffer object.
	bool draw( ddgContext *ctx );
	/// Initialize the vector Buffer object and allocate buffers if size is set.
	bool init(ddgContext *ctx );
	/// Set the range of objects in the buffer to render.
	void range( unsigned int start, unsigned int num)
	{
		ddgAssert(start >= 0 && start + num <= varray.inum());
		_rstart = start;
		_rnum = num;
	}
	/// Reset the buffer before filling with data.
	void reset(void)
	{
		varray.reset();
		_rnum = 0; 
	}
};

#endif

