/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <iomanip.h>

#include "struct/ddgdome.h"

// ----------------------------------------------------------------------
ddgDome::ddgDome( bool f )
{
	_radius = 50000;
	_arc = 20;
	_step = 7;
	_flat = f;
}


/* Dome.
 */
bool ddgDome::init( ddgVArray *varray, bool texture, bool color)
{
	int size = (1+_step)*(1+_step);

	varray->vbuf = new ddgVector3[size];
	ddgAsserts(varray->vbuf,"Failed to Allocate memory");
	ddgMemorySet(ddgVector3,size);
	varray->ibuf = new ddgVBIndex[_step*_step*2*3];
	ddgAsserts(varray->ibuf,"Failed to Allocate memory");
	ddgMemorySet(ddgVBIndex,_step*_step*2*3);
	if (color)
	{
		varray->cbuf = new ddgColor4[size];
		ddgAsserts(varray->cbuf,"Failed to Allocate memory");
		ddgMemorySet(ddgColor4,size);
	}
	if (texture)
	{
		varray->tbuf = new ddgVector2[size];
		ddgAsserts(varray->tbuf,"Failed to Allocate memory");
		ddgMemorySet(ddgVector2,size);
	}
	float i, j,
	di = 1.0/_step, dj = 1.0/_step; // step size.
	float ao = _arc/-2;     // Arc offset from 0 degrees
	float x,y,z;
	float miny = ddgAngle::cos(ao)*ddgAngle::cos(ao);
	float dy = 1-miny;
	y = ddgAngle::cos(ao)*ddgAngle::cos(ao);
	unsigned int si,sj, n = 0;
	// Precompute vertex array for rendering.
	for (si = 0; si <= _step; si++)
	{
		i = si *di + di;
		for (sj = 0; sj <= _step; sj++)
		{
			j = sj * dj;
			x = ddgAngle::sin(i*_arc+ao);
			z = ddgAngle::sin(j*_arc+ao);
			if (!_flat)
			{
				y = ddgAngle::cos((i-di)*_arc+ao)*ddgAngle::cos(j*_arc+ao);
				if (si==0||sj==0||sj==_step||si==_step)
					y = miny;
			}

			varray->vbuf[n].set(_radius*x,_radius*(y-miny),_radius*z);
			if (texture )
				varray->tbuf[n].set(  i, j);
            ddgColor4 ctmp;
			if (color)
			{
				varray->cbuf[n].v[0] = 0;
				varray->cbuf[n].v[1] = 255;
				varray->cbuf[n].v[2] = 0;
				varray->cbuf[n].v[3] = 128;
			}
			n++;
		}
	}
	// Precompute the indices.
	n = 0;
	for (si = 0; si < _step; si++)
	{
		for (sj = 0; sj < _step; sj++)
		{
			if (!_flat)
			{
				/// Push a triangle into the buffer.
				varray->pushTriangle(si*(_step+1)+sj,(si+1)*(_step+1)+sj,(si+1)*(_step+1)+sj+1);
				varray->pushTriangle(si*(_step+1)+sj,(si+1)*(_step+1)+sj+1,si*(_step+1)+sj+1);
			}
			else
			{
				varray->pushTriangle(si*(_step+1)+sj,(si+1)*(_step+1)+sj+1,(si+1)*(_step+1)+sj);
				varray->pushTriangle(si*(_step+1)+sj,si*(_step+1)+sj+1,(si+1)*(_step+1)+sj+1);
			}
		}
	}

	return false;
}



