/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "math/ddgchull.h"
#include "struct/ddgtmesh.h"
#include "struct/ddgecos.h"
#include "struct/ddgimage.h"

ddgEcoSystem::~ddgEcoSystem(void)
{
	unsigned int i = 0;
	while (i < _size)
	{
		delete ecoBlocks[i];
		i++;
	}
	delete[] ecoBlocks;
}

bool ddgEcoSystem::init(void)
{
	if (blockNum)
		return ddgFailure;

	unsigned int _blockNumSQRT = 16;
	blockNum = _blockNumSQRT*_blockNumSQRT;
	unsigned int i = 0, j = 0, k = _size / (blockNum), e = 0;
	if (k == 0) k = 1;
	ddgVector3 p, s, wsize;
	// Size of the world.
	wsize= _bbox->size();
	// Size of the eco blocks.
	s.set(wsize[0]/_blockNumSQRT,-199999,wsize[2]/_blockNumSQRT);
	ecoBlocks = new ddgEcoBlock*[blockNum];
	ddgMemorySet(ddgEcoBlock*,blockNum);
	ddgMemorySet(ddgEcoBlock,blockNum);
	for (i = 0; i < _blockNumSQRT; i++)
	{
		for (j = 0; j < _blockNumSQRT; j++)
		{
			p.set(i*wsize[0]/_blockNumSQRT,99999,j*wsize[2]/_blockNumSQRT);
   			ecoBlocks[e] = new ddgEcoBlock(this,k,new ddgBBox3(p,p+s));
			ecoBlocks[e]->init(this);
			e++;
		}
		ddgConsoleMessage::progress("Populating eco system",i*_blockNumSQRT,blockNum);
	}
	ddgConsoleMessage::progress("Populating eco system",1,1);
	return ddgSuccess;
}

bool ddgEcoBlock::init( ddgEcoSystem *e )
{
	unsigned int i = 0, j = 0;
	bool		seedSet = false;
	unsigned int c = e->_ecoMap->cols(), r = e->_ecoMap->rows();
	// Map world coords to ecoMap coords
	float		scalex = ((float)c)/(e->_bbox->max[0] - e->_bbox->min[0]);
	float		scalez = ((float)r)/(e->_bbox->max[2] - e->_bbox->min[2]);

	pos = new ddgVector3[_size];
	ddgMemorySet(ddgVector3,_size);
	ddgVector3 bsize = bbox->size(),
				*pmin = &(bbox->min),
				*pmax = &(bbox->max);
	// Brute for approach to seed trees where ever the ecomap indicates the objects
	// can appear.
	while (i < _size && j < _size *4)
	{
		seedSet = true;
		pos[i][0] = (*pmin)[0] + (float)(rand() % ((int)bsize[0]*1000))/1000.0;
		pos[i][2] = (*pmin)[2] + (float)(rand() % ((int)bsize[2]*1000))/1000.0;
		pos[i][1] = e->_mesh ? e->_mesh->height(pos[i][0],pos[i][2]) : 0.0;
		unsigned short ex = ddgUtil::clamp((pos[i][0]*scalex),0,c),
					   ez = ddgUtil::clamp((pos[i][2]*scalez),0,r);
		
		j++;
		// See if this is a valid location for the item.
		if ( e->_ecoMap )
		{
			unsigned char ep;
			e->_ecoMap->get(ex,ez,&ep);
			if (ep < e->_min || ep > e->_max)
				seedSet = false;
		}

		if (seedSet)
		{
			(*pmax)[1] = ddgUtil::max((*pmax)[1],pos[i][1]+e->_osize[1]);
			(*pmin)[1] = ddgUtil::min((*pmin)[1],pos[i][1]);
			i++;
		}
	}
	// We may not have been able to reach the required number of objects due to
	// land scape restrictions.
	_size = i;
	return ddgSuccess;
}

ddgInside ddgEcoBlock::vis( ddgCHull3 *frustrum, ddgVector2 pos)
{
	_vis = ddgOUT;
	if (_size)
	{
		ddgVector3 bc = bbox->centre();
		ddgVector2 c(bc.v[0],bc.v[2]);
		// See if the triangle is too far from the camera.  Ingore height component.
		c -= pos;

		// Do a rough distance test and optionally a true distance test.
		// XXXX todo scale _farClipSQ.
		if (c[0] > _es->_farClip || c[0] < - _es->_farClip
		  ||c[1] > _es->_farClip || c[1] < - _es->_farClip
		  ||c.sizesq() > _es->_farClipSQ
		  )
			_vis = ddgOUT;
		else
			_vis = frustrum->clip( bbox );
	}
	return _vis;
}

