/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _ddgEvent_Class_
#define _ddgEvent_Class_

#include "util/ddgutil.h"

/**
 * This object maintains a set of event handlers.
 */
class WEXP ddgEventHandler
{
public:
	/// Event handler, returns true if a change occurred.
	typedef	bool (*MouseHandlerFn)(void*, int button, int state, int x, int y);
	/// Event handler, returns true if a change occurred.
	typedef bool (*MouseMotionHandlerFn)(void*, int x, int y);    
	/// Event handler, returns true if a change occurred.
	typedef bool (*SpecialKeyHandlerFn)(void*, int key, int x, int y);
	/// Event handler, returns true if a change occurred.
	typedef bool (*KeyHandlerFn)(void*, unsigned char key, int x, int y);
private:
	/// A method which will handle mouse click events.
	MouseHandlerFn			_mhfn;
	/// A method which will handle mouse motion events.
	MouseMotionHandlerFn	_mmhfn;
	/// A method which will handle special modifier key events.
	SpecialKeyHandlerFn		_skhfn;
	/// A method which will handle general keyboard events.
	KeyHandlerFn			_khfn;
	/// A default object which will handle all events not handled by this object.
	ddgEventHandler*		_defaultHandler;
	/// A context object which will handle all events.
	void*			_hobj;
public:
	ddgEventHandler(void);
	/// Set Key handler method.
	void mouseHandler( MouseHandlerFn h ) { _mhfn = h; }
	/// Get Key handler method.
	MouseHandlerFn mouseHandler( void ) { return _mhfn; }
	/// Set Key handler method.
	void mouseMotionHandler( MouseMotionHandlerFn h ) { _mmhfn = h; }
	/// Set Key handler method.
	void keyHandler( KeyHandlerFn h ) { _khfn = h; }
	/// Set Key handler method.
	void specialKeyHandler( SpecialKeyHandlerFn h ) { _skhfn = h; }
	/// Get Key handler method.
	MouseMotionHandlerFn mouseMotionHandler( void ) { return _mmhfn; }
	/// Get Key handler method.
	KeyHandlerFn keyHandler( void ) { return _khfn; }
	/// Get Key handler method.
	SpecialKeyHandlerFn specialKeyHandler( void ) { return _skhfn; }
	/// Set the handler object.
	void handlerObject( void *obj ) { _hobj = obj; }
	/// Get Key handler object.
	void* handlerObject( void ) { return _hobj; }
	/// Set Default handler object.
	void defaultHandler( ddgEventHandler *handler ) { _defaultHandler = handler; }
	/// Get Default handler object.
	ddgEventHandler* defaultHandler( void ) { return _defaultHandler; }
	/// Set the handler functions and context object on this event handler.
	void set( 
										   void *obj,
										   KeyHandlerFn skey,
										   SpecialKeyHandlerFn sspecialKey,
										   MouseHandlerFn smouse,
										   MouseMotionHandlerFn smouseMotion
										   );
	/**
	 * Overrride another handlers event handlers with the specified functions.
	 * store the original handlers in this object and call those if we do not
	 * handle the events ourselves.
	 */
	void overrideEventHandler( ddgEventHandler *handler );
	/// Default key handler.
	static bool smouse(void *obj, int button, int state, int x, int y);

	/// Default Event handler, returns true if a change occurred.
	static bool smouseMotion(void *obj, int x, int y);

	/// Default Event handler, returns true if a change occurred.
	static bool sspecialKey(void *obj, int k, int x, int y);

	/// Default Event handler, returns true if a change occurred.
	static bool skey(void *obj, unsigned char k, int x, int y);
};

#endif
