/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _ddgHistogram_Class_
#define _ddgHistogram_Class_

#include "util/ddgerror.h"
/**
 * Abstract histogram object to collect data.
 */
class WEXP ddgHistogram
{
	unsigned int	*_data;
	/// Number of elements.
	unsigned int	_size;
	/// Total number of samples registered.
	unsigned int	_totalSamples;
	/// Maximum value in any column.
	unsigned int	_maxVal;
public:
	/// Create a text object at a given canvas location.
	ddgHistogram(unsigned int s)
		: _totalSamples(0), _maxVal(0), _data(0), _size(s) {}
	/// Free the memory of this histogram.
	~ddgHistogram(void);
	/// Allocate memory if needed and clear the data samples.
	bool init(void);
	/// Increment one range entry.
    void incr(unsigned int i) { ddgAssert(i < _size); _data[i]++; _totalSamples++; if (_data[i] >_maxVal) _maxVal++; }
	/// Decrement one range entry.
    void decr(unsigned int i) { ddgAssert(i < _size); _data[i]--; _totalSamples--; }
	/// Return the size of the histogram.
	unsigned int size(void) { return _size; }
	/// Return the maxVal of the histogram.
	unsigned int maxVal(void) { return _maxVal; }
	/// Return the totalSample of the histogram.
	unsigned int totalSample(void) { return _totalSamples; }
	/// Return the totalSample of the histogram.
	unsigned int data(unsigned int i) { return _data[i]; }
};

#endif
