/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _ddgNode_Class_
#define _ddgNode_Class_

#include "struct/ddgcntxt.h"

/**
 * Class to render a generic object in the scene.
 * Other objects can be subclassed from this object to implement
 * differing drawing capabilities.
 * This is an Abstract class.
 */
class WEXP ddgNode
{
public:
	union RenderFlags {
		 unsigned int all;
		 struct {
			 bool	wire:1;			// Wire frame rendered.
			 bool	solid:1;		// Render solid.
			 bool	normal:1;		// Use normals.
			 bool	texture:1;		// Render texture.
			 bool	smooth:1;		// Render smooth.
			 bool	text:1;			// Render text labels.
			 bool	disabled:1;		// Rendering is not needed.
			 bool	init:1;		    // Has object been initialized.
		 } flags;
	};
	RenderFlags	mode;
	/// Constructor
	ddgNode(void) { mode.all = 0; }

	/**
	*  Render this node.
	*  This function must be overidden.
	*  Return true if error.
	*/
	virtual bool draw( ddgContext *ctx ) = 0;
	/**
	*  Initialize the node.
	*  This function should be overridden.
    *  Return false if no error occured.
	*         true if object is already initialized.
    *         true if initialization failed, in that case mode.flags.init should be reset to false.
    *  All sub classes should have the following at the start of their init method:
    *  if (super::init()) return true;
	*/
	virtual bool init( ddgContext *ctx );
};

#endif

