/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
// General includes
#include <stdio.h>   // DEBUG
#include <string.h>  // DEBUG
#include <stdlib.h>
#include <math.h>

// Our own headers.
#include "struct/ddgoctree.h"

// ======================================================================


// ----------------------------------------------------------------------
// depthCompare:
//   Compare the distance to two bounding boxes.
// p1,p2:
//   Pointers into the triangle array.
// return:
//   Returns 1 if p1 > p2, -1 if p1 < p2, 0 otherwise.
//
// ----------------------------------------------------------------------
/*
int ddgOTree::compareBox( const void* p1, const void* p2)
{
VisNode *t1 = (VisNode*)p1;
VisNode *t2 = (VisNode*)p2;
if      ( t1->zmin < t2->zmin ) return -1;
else if ( t2->zmin < t1->zmin ) return 1;
return 0;
}

// ----------------------------------------------------------------------
// drawhom:
//   render the leaf nodes front to back.
// ----------------------------------------------------------------------
void ddgOTree::drawFrontToBack()
{
   int i;
   int _limit = 0;
   if (visNodeFull) visNodeCount = ddgMaxVisNodes;
   qsort((void*)visNodes,visNodeCount,sizeof(VisNode),compareBox);
printf("Nodes %d, limit %d\n",visNodeCount,_limit);
   for (i = 0; i < visNodeCount; i++)
     {
     currentNode = i;
     visNodes[i].node->draw(NULL);
     }
}



// ----------------------------------------------------------------------
// drawSubNodes:
//   Render all children from nearset to farthest from renderInfo origin.
// mode:
//   The drawing mode, NORMAL, or HOM.
// quality:
//
// limit:
// return:
//   Returns the number of triangles rendered.
//
bool ddgOTree::drawSubNodes(ddgContext *ctx)
  {
  int i, pcount = 0;
  _level++;
  ddgOTree **subtree = (ddgOTree**)_data;
 
  for (i=0; i < 8; i++)
    if (subtree[i])
        pcount += subtree[i]->ddgOTree::draw(ctx);

  return false;
  }
*/
// ----------------------------------------------------------------------
// draw:
//   Render an octree node and all its children.
// mode:
//   The drawing mode, NORMAL, or HOM.
// quality:
//   The count of leaf octree nodes rendered so far.
// return:
//   Return the number of polygons rendered.
// ----------------------------------------------------------------------
/*
bool ddgOctTree::draw( ddgContext *ctx )
{

  int i, clip = 0,                   // 1 - clipped, 0 - definitely render.
         pcount=0;                   // Number of polygons drawn by this call.
  static GLdouble  mm[16], mp[16];   // modelview and projection matrix.
  static GLint vp[4];
  float xmin, xmax, ymin, ymax, zmin, zmax;
  zmin = MAXFLOAT;
  zmax = -MAXFLOAT;

  // Get the current orientation once per rendering.
  if (cullingInfo.viewCulling && (_level == 1) && m == Render::VIEWCULL)
    {
    glGetDoublev(GL_MODELVIEW_MATRIX,mm);
    glGetDoublev(GL_PROJECTION_MATRIX,mp);
    glGetIntegerv(GL_VIEWPORT,vp);
    visNodeCount = 0;  // Reset the visible leaf list.
    visNodeFull = 0;
    }

  if (m == Render::NORMAL && !cullingInfo.viewCulling && clipped )
     clipped = false; // Restore branch to active.

  // Perform view frustrum culling.
  else if (m == Render::VIEWCULL)
    {
    if (clipped)
      {
      clipped = false; // Restore branch to active.
      }
    if (cullingInfo.viewCulling)
      {
      // Get each bounding box corner point in screen coordinates.
      xmin = ymin = zmin =  MAXFLOAT;
      xmax = ymax = zmax = -MAXFLOAT;
     
      GLdouble res[4], res2[4];
      for (i=0; i<8; i++)
        {
        // Convert the world coordinates to screen coordinates.
	// Note this should be done at the same resolution at
	// the final rendering or errors will occur along the edges.
	// Perform matrix multiplication.
        res[0] = mm[0]*_data->cornerx(i)+
                 mm[4]*_data->cornery(i)+
                 mm[8]*_data->cornerz(i)+mm[12];
        res[1] = mm[1]*_data->cornerx(i)+
                 mm[5]*_data->cornery(i)+
                 mm[9]*_data->cornerz(i)+mm[13];
        res[2] = mm[2]*_data->cornerx(i)+
                 mm[6]*_data->cornery(i)+
                 mm[10]*_data->cornerz(i)+mm[14];
        // Multiply by the projection matrix.
        res2[0] = mp[0]*res[0]+
                  mp[4]*res[1]+
                  mp[8]*res[2]+mp[12];
        res2[1] = mp[1]*res[0]+
                  mp[5]*res[1]+
                  mp[9]*res[2]+mp[13];
        res2[2] = mp[2]*res[0]+
                  mp[6]*res[1]+
                  mp[10]*res[2]+mp[14];
        // Perform perspective scaling.
        res2[0] = vp[0]+(1+(res2[0]/res2[2]))*vp[2]/2;
        res2[1] = vp[1]+(1+(res2[1]/res2[2]))*vp[3]/2;

	// Find bounding box.
        if (xmin > res2[0]) xmin = res2[0];
        if (xmax < res2[0]) xmax = res2[0];
        if (ymin > res2[1]) ymin = res2[1];
        if (ymax < res2[1]) ymax = res2[1];
        if (zmin > res2[2]) zmin = res2[2];
        if (zmax < res2[2]) zmax = res2[2];
	}

      // If the bounding box does not overlap the view volume clip.
      if (xmax < vp[0]-1 || xmin > vp[2]+1 ||
          ymax < vp[0]-1 || ymin > vp[3]+1 ||
          zmax < camera->near() || zmin > camera->far())
	{
        // Indicate that this branch is clipped for next time when real
        // rendering takes place.
        clip = 1;
        clipped = true;
        }
      }  
    }
  // View frustrum culling is done.
  // Perform occlusion culling if needed.
  if (!clip)
    {
    if (leaf) // This is a leaf node.
      {
      // Collect but don't draw all visible leaf nodes.
      if (m == Render::VIEWCULL)
	{
        float *e = camera->eye();
        // We are full, find a node which is further than this one.
        if (visNodeCount == MaxVisNodes) 
          {
          visNodeCount == 0;
          visNodeFull++;
          }
        if (visNodeFull)
          {
          while(visNodes[visNodeCount].zmin > zmin && visNodeCount < MaxVisNodes)
	    { visNodeCount++; }
	  }
        if (visNodeCount < MaxVisNodes)
          {
          visNodes[visNodeCount].node = this;
          visNodes[visNodeCount].zmin = zmin;
          visNodes[visNodeCount].zmax = zmax;
          visNodeCount++;
          if (visNodeFull) visNodeFull++;
          }
	}
      // If still survived, render the thing.
      // Draw bounding box.

      if (m == Render::NORMAL)
        {
        pcount += _data->draw(m, q);
        }
      // Perform the hom test handling.
      else if (m == Render::HOM && cullingInfo.occlusionCulling)
        {
        _limit++;
        // record the furthest corner distance of a leaf bbox
        // that is used to clip with.  Any bboxes whose min z is
        // further away than the HOMDepth can be tested against the HOM.
        
printf("%d %d    %f %f %f\n",_limit, cullingInfo.HOMLimit,
        visNodes[currentNode].zmin, visNodes[currentNode].zmax,
        cullingInfo.HOMDepth);
        if (_limit < cullingInfo.HOMLimit)
	  {
	    // printf("zmin %f  HOMDepth %f\n",visNodes[currentNode].zmax,cullingInfo.HOMDepth);
          pcount += draw(m,q);
          if (visNodes[currentNode].zmax > cullingInfo.HOMDepth)
            cullingInfo.HOMDepth = visNodes[currentNode].zmax;
	  }
        else if (_limit == cullingInfo.HOMLimit)
          {
printf("Building maps \n");
          // Scale from 256x256 to 128x128 coordinate system and build nodes.
          cullingInfo.scaleBox(256,256);
          glColor3ub(255,255,255);

          cullingInfo.buildHOMMap();
          }
        else if (visNodes[currentNode].zmin > cullingInfo.HOMDepth)
          {
	  // This node can be tested against the HOMs.
          // Map to screen coords.
printf("Testing node. ");
          cullingInfo.boxInit();
          GLdouble x,y,z;
          for (i =0; i < 8; i++)
	    {
            gluProject((double)_data->cornerx(i), 
                       (double)_data->cornery(i), 
                       (double)_data->cornerz(i), mm, mp, vp, &x, &y, &z);

            cullingInfo.limitBoxMinX(x);
            cullingInfo.limitBoxMaxX(x);
            cullingInfo.limitBoxMinY(y);
            cullingInfo.limitBoxMaxY(y);
	    }

          cullingInfo.limitBox(vp[0],vp[1],vp[0]+vp[2],vp[1]+vp[3]);

          if (cullingInfo.HOMvisible())
	    {
printf("Kept\n");
            if (ctx->quality() != Context::DEBUG)
				{
				glColor3ub(0,0,255);
				draw(m,q); // XXXXXXXX TODO BBox
				}
	    }
          else 
            {      
printf("Culled\n");
            if (ctx->quality() != Context::DEBUG)
              {
              clipped = true;
              glColor3ub(255,0,0);
	      draw(m,q); // XXXXX
              }
            }
	  }
        }
      // DEBUG.

      else
	{
        if (m == Render::TOP)
	  {
          static float cols[8][3] = 
                 {{255,0,0},{0,255,0},{0,0,255},{0,255,255},
                  {255,255,0},{255,0,255},{255,0,0},{0,255,0}};
          glColor3ub(cols[_level][0],cols[_level][1],cols[_level][2]);
          draw( m,q); // TODO XXXX BBox
	  }
	}

      }
    if (!leaf) // We have children.
      {
      pcount += drawSubNodes(m,q);
      }
    }

  if (cullingInfo.viewCulling && (_level == 1) && m == Render::VIEWCULL)
    {
    drawFrontToBack();
    }

  return false;
}
*/
/// Insert an object into the octree.
bool ddgOctTree::insert(void *pn, ddgBBox3 *pbb)
{
	return true;
}
/// Insert an object from the octree.
bool ddgOctTree::remove(void *pn, ddgBBox3 *pbb)
{
	return true;
}
