/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <fstream.h>
//
#include "util/ddgpath.h"
// ----------------------------------------------------------------------
ddgVector3	campos[ddgMAXKEY];
ddgVector3 camdir[ddgMAXKEY];
unsigned int frame[ddgMAXKEY];

/// Create a path from a file.
bool ddgPath::load( void )
{
#ifdef DDGSTREAM
	unsigned int i;
	char *fullName = ddgFileHelper::getFullFileName(_pathFile);
	ifstream fin(fullName);
	if (!fin.good())
	{
		ddgErrorSet(FileRead,(char *) (fullName ? fullName : "(null)"));
		ddgError::report();
		_frameCount = 0;
		return ddgFailure;
	}
	fin >> _frameCount;		// Number of frames.
	ddgConsole("Path file loaded with " << _frameCount << " frames" << endl);
	if (_frameCount > ddgMAXKEY)
	{
		cerr << _frameCount;
		ddgErrorSet(FileContent,"Too many frames in path file");
		ddgError::report();
		_frameCount = 0;
		return ddgFailure;
	}
	for ( i = 0; i < _frameCount && i < ddgMAXKEY; i++)
	{
		ddgVector3 cp, cd;
		unsigned int f;
		fin >> cp >> cd >> f;
		campos[i] = cp;
		camdir[i] = cd;
		frame[i] = f;
	}
	fin.close();
	return ddgSuccess;
#else
	return ddgFailure;
#endif
}
/// Set the camera position and direction for a given frame.
void ddgPath::get( ddgVector3 *p, ddgVector3 *o, unsigned int f )
{
	p->set(campos[f][0],campos[f][1],campos[f][2]);
	o->set(camdir[f][0],camdir[f][1],camdir[f][2]);
}

/// Set the camera position and direction for a given frame.
void ddgPath::set( ddgVector3 *p, ddgVector3 *o, unsigned int f )
{
	if (f < ddgMAXKEY)
	{
		campos[f].set(p);
		camdir[f].set(o);
		frame[f] = f;
		if (f > _frameCount)
			_frameCount = f;
	}
}

/// Set the camera position and direction for a given frame.
void ddgPath::add( ddgVector3 *p, ddgVector3 *o )
{
	if (_frameCount < ddgMAXKEY)
	{
		campos[_frameCount].set(p);
		camdir[_frameCount].set(o);
		frame[_frameCount] = _frameCount;
		_frameCount++;
	}
}

/// Create a path from a file.
bool ddgPath::save( void )
{
#ifdef DDGSTREAM
	unsigned int i;
	char *fullName = ddgFileHelper::getFullFileName(_pathFile);
	ofstream fout(fullName);
	if (!fout.good())
	{
		ddgErrorSet(FileWrite,(char *) (fullName ? fullName : "(null)"));
		ddgError::report();
		return ddgFailure;
	}
	fout << _frameCount << endl;		// Number of frames.

	for ( i = 0; i < _frameCount; i++)
	{
		ddgVector3 cp, cd;
		fout << campos[i] << ' ' << camdir[i] << ' ' << frame[i] << endl;
	}
	fout.close();
	return ddgSuccess;
#else
	return ddgFailure;
#endif
}
