/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _ddgPath_Class_
#define _ddgPath_Class_

#include "math/ddgvec.h"
#include "util/ddgerror.h"

/**
 * This object maintains a set of positions and orientations.
 * These can be recorded and played back to/from a file.
 * A path object is usually connected to a Context object
 * which feeds it to the camara.
 */
class WEXP ddgPath
{
#define ddgMAXKEY	10000
	ddgVector3	campos[ddgMAXKEY];
	ddgVector3 camdir[ddgMAXKEY];
	unsigned int frame[ddgMAXKEY];
	/// Frames in the path.
	unsigned int _frameCount;
	/// Name of path file.
	ddgStr		_pathFile;
public:
    /// Create a path from a file.
	ddgPath( ddgStr *filename ) : _frameCount(0)
	{
		_pathFile.assign(filename);
	}
    /// Create a path.
	ddgPath( void ) : _frameCount(0) { }
	/// Set the camera position and direction for a given frame.
	void get( ddgVector3 *p, ddgVector3 *o, unsigned int frame );
	/// Record a camera position.
	void set( ddgVector3 *p, ddgVector3 *o, unsigned int frame );
	/// Record a camera position.
	void add( ddgVector3 *p, ddgVector3 *o );
	/// Load the path to a file.
	bool load( void );
	/// Save the path to a file.
	bool save( void );
	/// Return the number of frames recorded.
	unsigned int frames(void) { return _frameCount; }
};

#endif
