/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

// ---------------------------------------------------------------------
//
// DDG definitions.
//
#include "util/ddgpath.h"
#include "struct/ddgbtree.h"
#include "struct/ddgecos.h"
#include "render/ddgmeshrender.h"
#include "render/ddgviewport.h"
#include "render/ddgobjects.h"
#include "render/ddgmaterial.h"
#include "render/ddgtransform.h"
#include "render/ddglight.h"
#include "render/ddgbox.h"
#include "render/ddgtext.h"
#include "render/ddgcloud.h"
#include "render/ddgsky.h"
#include "render/ddgsun.h"
#include "render/ddgstars.h"
#include "render/ddgfog.h"
#include "render/ddgground.h"
#include "render/ddggeodesic.h"
#include "render/ddgsquare.h"
#include "render/ddgtree.h"
#include "render/ddggrass.h"
#include "render/ddghistg.h"
#include "render/ddgreflection.h"
#include "render/ddgkcntl.h"

// ---------------------------------------------------------------------
//
// Global Objects used in this app.
//
ddgCamera		*camera1 = 0,
				*camera2 = 0,
				*camera3 = 0;
ddgClock		*clock = 0;
ddgViewport		*rootvp = 0,
				*orthovp = 0,
				*colorvp = 0,
				*controlvp = 0;
ddgGrid			*grid = 0;
ddgAxis			*daxis = 0;
ddgCameraObj	*dcam = 0;
ddgScene		*scene1 = 0,
				*scene2 = 0;
ddgText      	*text1 = 0,
				*text2 = 0,
				*text3 = 0,
				*text4 = 0,
				*text5 = 0;
ddgViewportSet	*vps = 0;
ddgLightModel	*lightmodel = 0;
ddgLight		*sunlight = 0;
ddgMaterial		*material = 0;
ddgTransform	*meshTrans = 0,
				*starTrans = 0,
				*cloudTrans = 0,
				*skyTrans = 0,
				*groundTrans = 0,
				*lightTrans = 0,
				*moonTrans = 0,
				*sunTrans = 0;
ddgTBinMesh		*mesh = 0;
ddgMeshRender   *meshr = 0;
ddgImage		*heightf = 0;
ddgImage		*ecomap = 0;
ddgHeightMap	*height = 0;
ddgSun			*sun = 0;
ddgStarClass	*starfield = 0;
ddgCloud		*cloud = 0;
ddgSky 			*sky = 0;
ddgGround		*ground = 0;
ddgPath			*path = 0;
ddgFog			*fog = 0;
ddgGeodesic		*moon = 0;
ddgContext		*context = 0,
				*dbgcontext = 0;
ddgGrass		*grass = 0;
ddgTransform	*forest = 0;
ddgEcoSystem	*es = 0;
ddgReflection   *reflection = 0;
ddgHistograph	*histograph = 0;
ddgClipper		*clipperUp = 0;
ddgClipper		*clipperDown = 0;
static bool		wireOn = false,
				solidOn = true,
				lightingOn = false,
				textOn = true,
				infoOn = false,
				cloudOn = true,
				skyOn = true,
				sunOn = true,
				terrainOn = true,
				starsOn = true,
				forestOn = false,
				fogOn = true,
				moonOn = false,
				groundOn = true,
				perf = false,
				perf2 = false,
				record = false,
				play= false,
				sunMoves = true,
				textureOn = true,
				meshcolor = false,
				topviewOn = false,
				reflectionOn = false,
				collisionOn = true,
				histographOn = false;

static float	nearclip = 1.0,
				farclip = 400.0;
static int		selected = 0,
				detail = 4000;	
static float sealevel = 0.0;
static float startTime = 12*60*60; // NOON
static unsigned int fullscreen = 0;
static unsigned int forestSize = 0;
static unsigned int synthtexture = 8;
ddgStr *pfile = 0; // camera path file.
ddgStr *prffile = 0; // camera path file.
ddgStr *memlogfile = 0; // memory usage log file.
bool worldSpace = true;

static unsigned int wwidth = 800, wheight = 600;
static float hmin, hmax;
static float tThick = 1.0;
static unsigned int tsize = 257; //	Terrain size.
static float groundHeight = 1.1;
// ---------------------------------------------------------------------
// Define a player object which can handle collision detection against
// the terrain.
class ddgPlayer : public ddgKeyControl
{
	typedef	ddgKeyControl super;

public:
	ddgPlayer( ddgVector3 *p, ddgVector3 *o) : super(p,o) {}
	/**
	 *  Update the current position and orientation.
	 *  Returns true if position or orientation was modified.
	 */
	bool update(void)
	{
		if( super::update() )
		{
			if (collisionOn)
			{
				float terrainHeight;
				// Keep camera above the terrain and within bounds.
				ddgVector3 campos(position());
				ddgBBox3 *mb = mesh->bbox();
				bool collided = false;

				// Stay within x,z limits.
				if (campos[0] < mb->min[0]) {campos[0] = mb->min[0]; collided = true;}
				if (campos[2] < mb->min[2]) {campos[2] = mb->min[2]; collided = true;}
				if (campos[0] > mb->max[0]) {campos[0] = mb->max[0]; collided = true;}
				if (campos[2] > mb->max[2]) {campos[2] = mb->max[2]; collided = true;}

				// Follow terrain.
				terrainHeight = mesh ? mesh->height(campos[0],campos[2]) : 0.0;
				// Don't go below sea level.
				if (terrainHeight < sealevel)
					terrainHeight = sealevel;

				terrainHeight +=  groundHeight;
				if (campos[1] < terrainHeight) { campos[1] = terrainHeight; collided = true;}
				if (collided)
				{
					position()->set(campos[0],campos[1],campos[2]);
					_dirty = true;
				}
			}
		}
		return _dirty;
	}
	void attach( ddgViewport *vp )
	{
		ddgAssert(vp);
		// Set up the default handlers.
		ddgKeyControl::attach( vp );
		// Override the key handler with ours.
		_eventHandler.keyHandler(skey);

	}
    /// Event handler, returns true if a change occurred.
    static bool skey(void *obj, unsigned char key, int x, int y);
};

/// Event handler, returns true if a change occurred.
bool ddgPlayer::skey(void *obj, unsigned char k, int x, int y)
{
	ddgPlayer *kc = (ddgPlayer*)obj;
	// Assume we will handle the key press.
	bool h = true;

    unsigned char key = k ? tolower(k) : 0;
	bool on = (k && k == key) ? false : true;

	switch(key)
	{
	case 'b': forestOn = on; break;
	case 'c': cloudOn = on; break;
	case 'd':
		if (mesh)
		{
			detail = mesh->minDetail() + (on ? 1 : -1) *50;
			if (detail < 2 || detail > 8000)
				detail = mesh->minDetail();
			mesh->minDetail( detail);
			mesh->maxDetail( detail*1.1);
		}
		break;
	case 'e': textOn = on; break;
	case 'f':
		if (mesh)
		{
			farclip = mesh->farClip() + (on ? 1.0 : -1.0) *10.0;
			if (farclip < nearclip || farclip > 1000)
				farclip = mesh->farClip();
			mesh->farClip( farclip);
		}
		break;
	case 'g': groundOn = on; break;
	case 'i': infoOn = on; break;
	case 'k': skyOn = on; break;
	case 'l': lightingOn = on; break;
	case 'm': moonOn = on; break;
	case 'n':
		if (mesh)
		{
			nearclip = mesh->nearClip() + (on ? 1.0 : -1.0) *0.05;
			if (nearclip < 0.5 || nearclip > farclip)
				nearclip = mesh->nearClip();
			mesh->nearClip( nearclip);
		}
		break;
	case 'o': solidOn = on; break;
	case 'q':
		if (mesh)
		{
			selected = meshr->selected + (on ? 1 : -1);
			if (selected < 0 || selected >= ddgTBinMesh_size*ddgTBinMesh_size)
				selected = meshr->selected;
			 meshr->selected = selected;
		}
		break;
	case 'r': starsOn = on; break;
	case 's': sunOn = on; break;
	case 't': terrainOn = on; break;
	case 'v': fogOn = on; break;
	case 'w': wireOn = on; break;
	case '1': ddgLogFile::enabled(ddgLogFile::enabled()? false : true); break;

	case '?':
		cerr << "Demo Commands:" << endl
			<< "b/B ................ disable/enable forest rendering" << endl
			<< "c/C ................ disable/enable cloud rendering" << endl
			<< "d/D ................ decrement/increment mesh detail" << endl
			<< "e/E ................ disable/enable text rendering" << endl
			<< "f/F ................ decrement/increment far clip plane" << endl
			<< "g/G ................ disable/enable ground rendering" << endl
			<< "i/I ................ disable/enable info rendering" << endl
			<< "k/K ................ disable/enable sky rendering" << endl
			<< "l/L ................ disable/enable lighting" << endl
			<< "m/M ................ disable/enable moon rendering" << endl
			<< "n/N ................ decrement/increment near clip plane" << endl
			<< "o/O ................ disable/enable solid rendering" << endl
			<< "r/R ................ disable/enable starfield rendering" << endl
			<< "s/S ................ disable/enable sun rendering" << endl
			<< "t/T ................ disable/enable terrain rendering" << endl
			<< "v/V ................ disable/enable fog rendering" << endl
			<< "w/W ................ disable/enable wireframe rendering" << endl
			<< "1................... toggle logfile output on/off" << endl
			;
		// Pretend we didn't handle it so that other handlers can report their help.
	default:
		h = false;
	}

	// Let the base class handle it.
	if (!h)
		h = ddgKeyControl::skey(kc->_eventHandler.handlerObject(),k,x,y);
	return h;
}
ddgPlayer *player = 0;
// ---------------------------------------------------------------------
//
// Clean up Global Objects used in this app.
//
void errorHandler(ddgError *e)
{
	if (vps)
		vps->doShutdown(e);
	else
		exit (ddgError::report());
}

bool shutdown (void)
{
	char * s = new char[512];
#ifdef DDGSTREAM
	ostrstream msg(s,512);
	if (meshr)
	{
		msg << "Window size: " << wwidth << "," << wheight << endl;
		msg << "Detail: " << detail << endl
			<< "Terrain Size: " << tsize << endl
			<< "Block Size: " << ddgTBinMesh_size << endl
			<< "Far clip: " << mesh->farClip() << endl
			<< "Render: " << (perf2 ? "off" : "on") << endl
			<< "Priority Resolution: " << ddgPriorityResolution << endl;

		if (!perf2)
		{
			char * mbuf = new char[32];
			msg << "Texture ";
			if (textureOn)
			{
				msg << "size: " << synthtexture * tsize +1 << "x"
								<< synthtexture * tsize +1 << endl;
			}
			else
				msg << "off";
			msg << endl;
			msg	<< "Mesh color " << (meshcolor ? "on" : "off") << endl
				<< "Lighting " << (lightingOn ? "on" : "off") << endl
				<< "Fullscreen mode " << (fullscreen ? ddgViewportSet::mode(fullscreen,mbuf,32)	 : "off") << endl
				<< "FOV " << context->fov() << endl;
		}
		msg << endl
			<< "tri: " << mesh->triCount() << " pri: " << mesh->priCount() << endl
			<< "ins: " << mesh->insCount() << " rem: " << mesh->remCount() << endl
			<< "mov: " << mesh->movCount() << " vis: " << mesh->visCount() << endl << '\0';
	}
	if (perf && prffile)
		clock->perfDump(prffile,s);
	if (record && path)
		path->save();
#ifdef _DEBUG
	if (memlogfile)
		ddgError::reportMemory(memlogfile, s);
#endif
#endif
	// All file I/O should be done by now.
	ddgFileHelper::clear();

	return ddgSuccess;
}

// ---------------------------------------------------------------------
//
// Methods called each frame by each camera to update state.
//
void update(void)
{
	static unsigned int s = 0;
	if (mesh)
	{
		if (camera1->context()->dirty() || s != mesh->minDetail())
		{
			s = mesh->minDetail();
		}
	}

	if (perf)
	{
		if (meshr)
		{
			meshr->mode.flags.wire = false;
			meshr->mode.flags.text = false;
			if (meshr && perf2)
			{
				meshr->mode.flags.texture = false;
				meshr->mode.flags.solid = false;
			}
		}
		if (sky)
			sky->mode.flags.disabled = true;
		if (cloud)
			cloud->mode.flags.disabled = true;
		if (sun)
			sun->mode.flags.disabled = true;
		if (ground)
			ground->mode.flags.disabled = true;
		if (starfield)
			starfield->mode.flags.disabled = true;
		if (path && (path->frames() == camera1->context()->frame()))
		{
			vps->doShutdown(NULL);
		}
	}
	else
	{
		// Lock light position to the sun.
		float h = startTime;
		float day = 1.0;
		if (!sunMoves)
			clock->set(startTime);
		day = clock->getDayLight();

		static ddgColor4 cnight(0,0,0,0);		// Night color.
		static ddgColor4 ctwilight(130,0,0,0);	// Twilight color.
		static ddgColor4 cday(100,100,100,0);	// Daytime color.
		static ddgColor4 cnow;
		clock->getSimTime(&h);

		if (sunTrans)
			sunTrans->rotate(new ddgVector3(0,0,h * 360/24));

		if (sunlight)
		{
			if (lightTrans)
				lightTrans->rotate(new ddgVector3(0,0,h * 360/24));
			if (moonTrans)
			{
				float mh = h+9;
				if (mh>24) mh -= 24;
				moonTrans->rotate(new ddgVector3(0,0, mh * 360/24));
			}

			if (day < 0.5)
				cnow.linterp(cnight,ctwilight,day*2);
			else
				cnow.linterp(ctwilight,cday,(day-0.5)*2);

			sunlight->diffuse(cnow);
			sunlight->ambient(cnow);
		}
		if (lightmodel)
			lightmodel->state(lightingOn);
		if (meshr)
		{
//			if (showtexture)
//				xtexture->mode(meshcolor ? ddgTexture::MODULATE : ddgTexture::DECAL,cnow);
//			meshr->renderMode(showtexture?true:false,lighting->get(),meshcolor);

			meshr->mode.flags.wire = (wireOn ? true : false);
			meshr->mode.flags.solid = (solidOn ? true : false);
			meshr->mode.flags.disabled = (terrainOn ? false : true);
		}
		if (cloud)
			cloud->mode.flags.disabled = (cloudOn ? false : true);
		if (sky)
			sky->mode.flags.disabled = (skyOn ? false : true);
		if (sun)
			sun->mode.flags.disabled = (sunOn ? false : true);
		if (moon)
			moon->mode.flags.disabled = (moonOn ? false : true);
		if (ground)
			ground->mode.flags.disabled = (groundOn ? false : true);
		if (starfield)
			starfield->mode.flags.disabled = (starsOn ? false : true);
		if (forest)
			forest->mode.flags.disabled = (forestOn ? false : true);
		{
			static ddgFog::Type t = ddgFog::NONE;
			if (fog)
			{
				float a = day * 0.5;
				const ddgColor4 *h = ddgColor4(255,255,255,0);
				fog->setColor(ddgColor3(h->v[0]*a,h->v[1]*a,h->v[2]*a));

				if(!fogOn && (fog->type() != ddgFog::NONE))
					{
					t = fog->type();
					fog->setType( ddgFog::NONE );
					}
				else if (fogOn && (fog->type() == ddgFog::NONE))
					{
					fog->setType( t );
					}
			}
		}
		ddgColor3 *c3 = 0;

		if (textOn)
		{
			unsigned int qssize = 0, qspri = 0;
			unsigned int qmsize = 0, qmpri = 0;
			unsigned int mindetail = 0, vistri = 0, balanceOperations = 0;
			if (meshr)
			{
				mindetail = mesh->minDetail();
				vistri = meshr->varray.inum()/3;

				qssize = mesh->qscache()->size();
				ddgCacheIndex ci = mesh->qscache()->head();
				ddgQNode *qn = mesh->qscache()->get(ci);
				qspri = mesh->getBinTree(qn->tree())->priority(qn->tindex());

				qmsize = mesh->qmcache()->size();
				ci = mesh->qmcache()->head();
				qn = mesh->qmcache()->get(ci);
				qmpri = mesh->getBinTree(qn->tree())->priority(qn->tindex());
			}

			unsigned int tn = 0;
			if (forest && es)
			{
				unsigned int bl = 0;
				while(bl < es->blockNum)
				{
					tn += es->ecoBlocks[bl]->vis() ? es->ecoBlocks[bl]->size() : 0;
					bl++;
				}
			}
			static unsigned int cnt = 0;

			// Show frame clock.
			char clk[] = "-\\|/";
			cnt++;
			if (cnt == 4) cnt = 0;

			// Calculate the number of frames per second using a circular buffer.
			float fps = clock->getAvgFrameRate();

			// Display information on screen.
			if (infoOn)
			{
				text1->printf("%5.2f %c Sp%d SQ%d(%d) MQ%d(%d) H%d V%d C%d T%d",
					fps,
					clk[cnt],
					mindetail,
					qssize, qspri,
					qmsize, qmpri,

					(int)h, vistri, mesh ? mesh->balCount() : 0, tn);
				
				ddgVector3 *pos = player->position(),
						*rot = player->orientation();
				text2->printf("%d Pos: %3.0f %3.0f %3.0f",camera1->context()->frame(),
					pos->v[0],pos->v[1],pos->v[2]); 
				text3->printf("Rot: %3.0f %3.0f %3.0f",
					rot->v[0],rot->v[1],rot->v[2]);
			}
			else
			{
				text1->printf("%5.2fFPS",fps);
				text2->printf("");
				text3->printf("");
			}
		}
	}

}

// Run without being driven by OpenGL but loop directly.
void perfloop(void)
{
	// Do as many frames as fast as possible.
	bool done = false;

	while(!done)
	{
		if (path && (path->frames() == camera1->context()->frame()))
		{
			done = true;
		}
		camera1->update();
		update();
		camera1->draw();
	}
}

// ---------------------------------------------------------------------
//
// Create and Initialize Global Objects and enter the event loop.
//
int main(int argc, char **argv)
{
	ddgStr *dataDir = new ddgStr("../data/");	// location of the data directory.
	ddgStr *hfile = new ddgStr("terrain/height.ter");	// single channel height map.
	ddgStr *tfile = new ddgStr("texture/height/ground.tga"); // 3 channel color map.
	ddgStr *efile = new ddgStr("texture/height/ecomap.tga"); // 1 channel ecomap.
	ddgStr *ofile = new ddgStr("texture/ocean.tga"); // 4 channel color map.
	ddgStr *cfile = new ddgStr("texture/cloud.tga"); // 4 channel color map.
	ddgStr *sfile = new ddgStr("misc/starchart"); // star data.
	ddgGroup *group = NULL;
	ddgVector3 campos;
	

	ddgError::errorHandler(errorHandler);
	clock = new ddgClock;
	clock->simfactor(1200);
	clock->start(startTime);    // Set to NOON

	// For fast software only mode try:
	// -topview off -sky off -sun off -cloud off -stars off -staticlight on -fog off -texture off -ground off -light off -meshcolor on
    // Process command line arguments.
	int argi = 1;	// Skip program name.
	while ( argi < argc )
	{
		if (ddgArgMatchValue("help")||ddgArgMatchValue("?"))
		{
#ifdef DDGSTREAM
			cerr << argv[0] << " -options" << endl
				 <<"-terrainMap filename     height map." << endl
				 <<"-groundTexture filename  ground texture map RGB or TGA." << endl
				 <<"-seaTexture filename     ocean texture map RGB or TGA." << endl
				 <<"-cloudTexture filename   cloud texture map RGBA map." << endl
				 <<"-starData filename       star data file." << endl
				 <<"-size n                  size of terrain 2^n+1." << endl
				 <<"-detail n                number of polygons to show." << endl
				 <<"-farclip f               far clipping plane distance." << endl
				 <<"-sealevel f              sea level setting (default 0)." << endl
				 <<"-sun {on/off}            sun rendering." << endl
				 <<"-cloud {on/off}          cloud rendering." << endl
				 <<"-sky {on/off}            sky rendering." << endl
				 <<"-ground {on/off}         background rendering." << endl
				 <<"-stars {on/off}          stars rendering." << endl
				 <<"-terrain {on/off}        terrain mesh rendering." << endl
				 <<"-fog {on/off}            fog rendering." << endl
				 <<"-topview {on/off}        topview window." << endl
				 <<"-text {on/off}           show text on main window with FPS." << endl
				 <<"-info {on/off}           show additional information on window." << endl
				 <<"-texture {on/off}        textureing." << endl
				 <<"-wire {on/off}			 wireframe mode." << endl
				 <<"-light {on/off}          lighting." << endl
				 <<"-meshcolor {on/off}      vertex color interpolation by height."<< endl
				 <<"-staticlight {on/off}    movement of sun." << endl
				 <<"-collision {on/off}      force viewer to stay above terrain."<< endl
				 <<"-reflection {on/off}     show reflections in the water."<< endl
				 <<"-logfile {on/off}        enable output to log file."<< endl
				 <<"-time h		             time of day in 24hr clock." << endl
				 <<"-forest n				 enable tree rendering n = 1000 - 500000." << endl
				 <<"-synthtexture scale      synthesize a texture for the heightmap based on an ecofile (scale = 0,1,2,4,8,16)." << endl
				 <<"-record filename         record camera movement to path file." << endl
				 <<"-play filename           playback camera movement from path file." << endl
				 <<"-perf filename           record FPS to perf file." << endl
				 <<"-perf2 filename          record FPS to perf file (no rendering)." << endl
				 <<"-memoryLogFile filename  record the memory usage (debug only)." << endl
				 <<"-fullscreen n            render in fullscreen mode (1-12)." << endl
				 <<"-width n		         window width default 640 (ignored in fullscreen mode)." << endl
				 <<"-height n		         window height default 480 (ignored in fullscreen mode)." << endl
				 <<"-dataDirectory filename	 specify location of data directory." << endl
				 ;
#endif
		}
		else if (ddgArgMatch("terrainMap"))
		{
			hfile->assign(argv[argi]);
		}
		else if (ddgArgMatch("groundTexture"))
		{
			tfile->assign(argv[argi]);
		}
		else if (ddgArgMatch("seaTexture"))
		{
			ofile->assign(argv[argi]);
		}
		else if (ddgArgMatch("cloudTexture"))
		{
			cfile->assign(argv[argi]);
		}
		else if (ddgArgMatch("starData"))
		{
			sfile->assign(argv[argi]);
		}
		else if (ddgArgMatch("size"))
		{
            tsize = ddgStr::stoi(argv[argi]);
		}
		else if (ddgArgMatch("detail"))
		{
			detail = ddgStr::stoi(argv[argi]);
		}
		else if (ddgArgMatch("farclip"))
		{
            farclip = -1.0 * ddgStr::stof(argv[argi]);
		}
		else if (ddgArgMatch("sealevel"))
		{
            sealevel = ddgStr::stof(argv[argi]);
		}
		else if (ddgArgMatch("light"))
		{
            lightingOn = ddgStr::equal(argv[argi],"on");
		}
		else if (ddgArgMatch("meshcolor"))
		{
            meshcolor = ddgStr::equal(argv[argi],"on");
		}
		else if (ddgArgMatch("reflection"))
		{
            reflectionOn = ddgStr::equal(argv[argi],"on");
		}
		else if (ddgArgMatch("logfile"))
		{
			ddgLogFile::enabled( ddgStr::equal(argv[argi],"on") ? true : false );
		}
		else if (ddgArgMatch("perf"))
		{
            perf = true;
			prffile = new ddgStr(argv[argi]);
		}
		else if (ddgArgMatch("perf2"))
		{
            perf = perf2 = true;
			prffile = new ddgStr(argv[argi]);
		}
		else if (ddgArgMatch("play"))
		{
            play = true;
			pfile = new ddgStr(argv[argi]);
			path = new ddgPath(pfile);
		}
		else if (ddgArgMatch("record"))
		{
            record = true;
			pfile = new ddgStr(argv[argi]);
			path = new ddgPath(pfile);
		}
		else if (ddgArgMatch("memoryLogFile"))
		{
			memlogfile = new ddgStr(argv[argi]);
		}
		else if (ddgArgMatch("dataDirectory"))
		{
			dataDir->assign(argv[argi]);
		}
		else if (ddgArgMatch("sun"))
		{
            sunOn = ddgStr::equal(argv[argi],"on");
		}
		else if (ddgArgMatch("moon"))
		{
            moonOn = ddgStr::equal(argv[argi],"on");
		}
		else if (ddgArgMatch("stars"))
		{
            starsOn = ddgStr::equal(argv[argi],"on");
		}
		else if (ddgArgMatch("cloud"))
		{
            cloudOn = ddgStr::equal(argv[argi],"on");
		}
		else if (ddgArgMatch("sky"))
		{
            skyOn = ddgStr::equal(argv[argi],"on");
		}
		else if (ddgArgMatch("ground"))
		{
            groundOn = ddgStr::equal(argv[argi],"on");
		}
		else if (ddgArgMatch("terrain"))
		{
            terrainOn = ddgStr::equal(argv[argi],"on");
		}
		else if (ddgArgMatch("forest"))
		{
            forestOn = true;
            forestSize = ddgStr::stoi(argv[argi]);
		}
		else if (ddgArgMatch("fog"))
		{
            fogOn = ddgStr::equal(argv[argi],"on");
		}
		else if (ddgArgMatch("topview"))
		{
            topviewOn = ddgStr::equal(argv[argi],"on");
		}
		else if (ddgArgMatch("text"))
		{
            textOn = ddgStr::equal(argv[argi],"on");
		}
		else if (ddgArgMatch("info"))
		{
            infoOn = ddgStr::equal(argv[argi],"on");
		}
		else if (ddgArgMatch("wire"))
		{
            wireOn = ddgStr::equal(argv[argi],"on");
			solidOn = wireOn ? false : true;
		}
		else if (ddgArgMatch("texture"))
		{
            textureOn = ddgStr::equal(argv[argi],"on");
		}
		else if (ddgArgMatch("synthtexture"))
		{
			synthtexture = ddgStr::stoi(argv[argi]);
			ddgAsserts(synthtexture >= 0 && synthtexture <= 16,"Scale should be a power of 2 less than or equal to 16");
		}
		else if (ddgArgMatch("staticlight"))
		{
            sunMoves = ddgStr::equal(argv[argi],"on");
		}
		else if (ddgArgMatch("collision"))
		{
            collisionOn = ddgStr::equal(argv[argi],"on");
		}
		else if (ddgArgMatch("histogram"))
		{
            histographOn = ddgStr::equal(argv[argi],"on");
		}
		else if (ddgArgMatch("time"))
		{
			clock->start(ddgStr::stof(argv[argi])*60.0*60.0);    // Set start time.
		}
		else if (ddgArgMatch("fullscreen"))
		{
            fullscreen = ddgStr::stoi(argv[argi]);
			ddgAsserts(fullscreen >= 0 && fullscreen < 13,"Modes supported are 0 to 12");
		}
		else if (ddgArgMatch("width"))
		{
            wwidth = ddgStr::stoi(argv[argi]);
			ddgAsserts(wwidth >= 100 && wwidth < 1600,"Width sizes supported are 100 to 1600");
		}
		else if (ddgArgMatch("height"))
		{
            wheight = ddgStr::stoi(argv[argi]);
			ddgAsserts(wheight >= 100 && wheight < 1200,"Height sizes supported are 100 to 1200");
		}
		else
		{
			if (argi < argc)
			{
				ddgErrorSetStr(CommandArgUnknownOption, "Unknown option [" << argv[argi] << "]");
			}
			else
			{
				ddgErrorSetStr(CommandArgMissingParameter,"Missing parameter for option [" << argv[argi-1] << "]");
			}
			goto error;

		}
		argi++;
	}
    
	ddgFileHelper::setDataDirectory(*dataDir);
	if (terrainOn)
	{
		// Load the data set.
		height = new ddgHeightMap();

		// Try loading a .raw file.
		if (!strcmp(".raw",&hfile->s[hfile->l-4]))
		{
			if (height->readRAW(*hfile,1024,1024) == ddgFailure)
			{
				ddgErrorSetStr(Failure,"Failed to load RAW height file");
				goto error;
			}
			height->setScaleAndBase(0.1);
			tsize = height->rows();
			synthtexture = 0;		// Use existing texture.
		}
		// Otherwise read file, if that fails generate a random
		// map.
		else if ( height->loadTerrain(*hfile))
		{
			if (!tsize)
				tsize = 256+1;

			ddgConsole("Creating height field");
			float oct = log (tsize-1)/ log(2.0);

			if (!height->createSpectralMap(oct,oct/8.0))
				tsize = height->rows();

			else if (height->generateHeights(tsize,tsize,oct) == ddgFailure)
			{
				ddgErrorSetStr(Failure,"Failed to generate heights");
				goto error;
			}

			height->setScaleAndBase(/*10*/ 0.5);
			// Save it for use next time.
			height->writeTGN(hfile->s);
		}
		else
		{
			tsize = height->rows();
		}

		// Terrain must be at least big enough for 1 mesh surface.
		if (tsize < ddgTBinMesh_size+1)
		{
			ddgErrorSetStr(Failure,"Terrain too small, must be bigger than " << ddgTBinMesh_size+1);
			goto error;
		}

		hmin = height->minf();
		hmax = height->maxf();
		tThick = hmax - hmin;

		ddgAssert(height->rows() == tsize);
		ddgAssert(height->cols() == tsize);

		// Create the mesh
		{
			ddgVector3 offset(0,height->base(),0);
			ddgVector3 scale(1.5,height->scale(),1.5);
			// Scale the fligh path
			if (path)
			{
				unsigned int i;
				for (i = 0; i < path->frames(); i++)
				{
					ddgVector3 p, o;
					path->get(&p, &o, i);
					p.v[0] *= scale[0];
					p.v[2] *= scale[2];

					p.v[0] += offset[0];
					p.v[2] += offset[2];
					path->set(&p,&o,i);
				}
			}
			mesh = new ddgTBinMesh(height);
			mesh->minDetail( detail );
			mesh->maxDetail( detail*1.1 );
			mesh->absMaxDetail( detail * 2);
			mesh->nearClip(nearclip);
			mesh->farClip(farclip);
		}

		if (textureOn)
		{
			meshr = new ddgMeshRender(mesh, height, tfile, efile, tsize-1, synthtexture, meshcolor);
		}
		else
		{
			meshr = new ddgMeshRender(mesh, NULL, NULL, NULL, 0, 0, meshcolor);
		}
		meshr->mode.flags.wire = wireOn;
		meshr->mode.flags.solid = solidOn;
		meshr->mode.flags.disabled = (terrainOn ? false : true);
		if (!lightingOn)
		{
			meshr->varray.renderMode(true, false, false);
		}
		else
			meshr->varray.renderMode(true, true, meshcolor ? true : false);
	}

	{
		float	x = tsize/2,
				y = height ? height->getf(tsize/2,tsize/2)+groundHeight: groundHeight,
				z = tsize/2,
				h = 0,  // heading  ry
				p = 0,    // pitch    rx
				b = 0;    // bank     rz
		player = new ddgPlayer(new ddgVector3(x,y,z),new ddgVector3(p,h,b));
		context = new ddgContext(ddgContext::ALL, ddgContext::MEDIUM, player, clock);
		context->path(path,record);
		context->aspect((float)wwidth/(float)wheight);
	}

	camera1 = new ddgCamera(context);
	if (path)
	{
		if (!record && (path->load() == ddgFailure))
			goto error;
		clock->perf( path->frames() );
	}

	dcam = new ddgCameraObj(camera1,ddgColor3(0,0,255));
	dcam->scale(10);

	scene1 = new ddgScene();

	if (perf2)
		scene1->clear(false);

	campos.set(player->position());

	campos[1] = sealevel;
	// Add a ground object.  Not affected by reflection/clipper.

	group = new ddgGroup();
	// Clip at y = sealevel.

	clipperUp = new ddgClipper();
	clipperUp->plane.d = sealevel;
	scene1->add(clipperUp);
	clipperUp->add(group);

	if (reflectionOn)
	{
		ddgReflection* reflection = new ddgReflection();
		clipperDown = new ddgClipper();
		clipperDown->plane.d = sealevel;
		ddgTranslate *trans = new ddgTranslate(new ddgVector3(0,2*sealevel,0));
		scene1->add(trans);
		trans->add(reflection);
		reflection->add(clipperDown);
		clipperDown->add(group);
	}

	// Add a star field.  Just inside the far clipping plane.
	if (!perf && starsOn && (starfield = new ddgStarClass(sfile,2)) && starfield->starCount())
	{
		
		starfield->scale = 5000;	// Just outside the cloud dome.
//		starfield->minMagnitude = 0.1; // Don't show all 20,000 stars.
		group->add(starTrans = new ddgTranslate(&campos));
		starTrans->add(starfield);
	}

	// Add sun object.
	if (sunOn && (sun = new ddgSun(new ddgVector3(0,-1*(float)tsize,0), camera1)))
	{
		group->add(sunTrans = new ddgTranslate(
			new ddgVector3(tsize/2,0,tsize/2),sun));
	}

	// Fog does not affect star or sun rendering.
	if( !perf && fogOn)
	{
		// Make same as horizon color.
		fog = new ddgFog(ddgFog::EXPONENTIAL,0.01);
		group->add(fog);
		group = fog;
	}

	// Add a sky object, this object is unlit but affected by fog.
	if (skyOn)
	{
		sky = new ddgSky();
		if (sky)
		{
			group->add(skyTrans = new ddgTranslate(&campos,sky));
		}
	}

	group->add(lightmodel = new ddgLightModel());
	group = lightmodel;
	lightmodel->colormaterial(true);
	if (perf)
		lightingOn = false;

	// Add lights.
    if (lightingOn)
    {
		lightmodel->add(lightTrans = new ddgTranslate(new ddgVector3(tsize/2,0,tsize/2)));
	    lightTrans->add(sunlight = new ddgLight());
		sunlight->pos(0,-1*(float)tsize,0);
	    lightmodel->add(material = new ddgMaterial(200,200,200));
	    material->ambient(50,50,50);
    }
    else
    {
        lightmodel->state(false);
    }

	// Add a cloud object.
	if (cloudOn)
	{
		ddgImage *cImg = new ddgImage();
		if (cImg->readFile(cfile))
			cloud = new ddgCloud(128);
		else
		{
			cloud = new ddgCloud(cImg);
		}
		if (cloud)
		{
			group->add(cloudTrans = new ddgTranslate(&campos,cloud));
		}
	}

	camera1->scene(scene1);
/*
	// Moon. To test the lighting.
	if (moonOn && !perf2)
	{
		group->add(moonTrans = 
			new ddgTranslate(ddgVector3(tsize/2,0,tsize/2),
						   new ddgTransform(ddgVector3(0,-10*(float)tsize,0),
			               ddgVector3(100,100,100),
						   NULL,
						   moon = new ddgGeodesic(2))));
		moon->color(ddgColor4(60,60,60,255));
	}
*/

	// Add all lighted surfaces
	if (sealevel == 0.0)
	{
		sealevel = height->minf()-1.0;
	}
	// Add a terrain mesh.
	if (meshr)
	{
//	height->closeEdge(sealevel);
		group->add(meshr);
	}

	if (groundOn)
	{
		ddgImage* img = new ddgImage();
		if (img->readFile(ofile) == ddgSuccess)
		{
			// Make it semi transparent.
			ground = new ddgGround(img,ddgColor4(0,50,125,150));
			groundTrans = new ddgTranslate(&campos,ground);

			if (fogOn)
			{
				// Make same as horizon color.
				fog = new ddgFog(ddgFog::EXPONENTIAL,0.01);
				fog->setColor(new ddgColor3(1,1,1));
				scene1->add(fog);
				fog->add(groundTrans);
			}
			else

				scene1->add(groundTrans);
		}
	}

//	grass = new ddgGrass(ddgGrass::grass);
//	group->add(new ddgTransform(ddgVector3(tsize/2+20,1,tsize/2),ddgVector3(3,1,3),NULL,grass));

	// Plant trees.
	// Should do this for each pair of bin trees, each with their own bbox.

	if (forestOn)
	{
		ecomap = new ddgImage();
		// Make sure we already have eco map.
		if ( ecomap->readFile(efile) != ddgSuccess)
		{
			delete ecomap;
			ecomap = NULL;
		}

		forest = new ddgTransform();
		// Size of objects which will be added to EcoSystem.
		ddgVector3 osize;
		// Create some objects.
		osize.set(0.5,0.5,0.5);
		es = new ddgEcoSystem(forestSize/7, mesh, mesh->bbox(), &osize,ecomap,3,3);
		forest->add( new ddgForest(new ddgTree(ddgTree::fern), es));

		osize.set(1,2,1);
		es = new ddgEcoSystem(forestSize/7, mesh, mesh->bbox(), &osize,ecomap,3,4);
		forest->add( new ddgForest(new ddgTree(ddgTree::palm), es));

		osize.set(2,2,2);
		es = new ddgEcoSystem(forestSize/7, mesh, mesh->bbox(), &osize,ecomap,4,5);
		forest->add( new ddgForest(new ddgTree(ddgTree::oak), es));

		osize.set(0.5,0.5,0.5);
		es = new ddgEcoSystem(forestSize/7, mesh, mesh->bbox(), &osize,ecomap,4,5);
		forest->add( new ddgForest(new ddgTree(ddgTree::bush), es));

		osize.set(1,2,1);
		es = new ddgEcoSystem(forestSize/7, mesh, mesh->bbox(), &osize,ecomap,5,6);
		forest->add( new ddgForest(new ddgTree(ddgTree::birch1), es));

		osize.set(1,2,1);
		es = new ddgEcoSystem(forestSize/7, mesh, mesh->bbox(), &osize,ecomap,6,7);
		forest->add( new ddgForest(new ddgTree(ddgTree::birch2), es));

		osize.set(2,2,2);
		es = new ddgEcoSystem(forestSize/7, mesh, mesh->bbox(), &osize,ecomap,7,9);
		forest->add(new ddgForest(new ddgTree(ddgTree::pine), es));

		group->add(forest);
	}

//	group->add(new ddgTranslate(ddgVector3(campos[0],campos[1]-0.1,campos[2]-0.3),new ddgParticleCloud()));
	// Scene2 and Camera2 are the orthographic debug view.
	// Centred about 0,0
	{
		int osize = tsize;
		dbgcontext = new ddgContext(ddgContext::ALL, ddgContext::DEBUG,
			new ddgControl(new ddgVector3(-osize/3,500,-osize/3), new ddgVector3(90, -90,0)));
	}
	camera2 = new ddgCamera(dbgcontext); // Look down.
	camera2->ortho(3*tsize,3*tsize); // Orthographic camera.

	scene2 = new ddgScene;
	if (!perf)
	{
		daxis = new ddgAxis();
		scene2->add(new ddgScale(new ddgVector3(10,10,10),daxis));
		scene2->add(dcam);
		if (starTrans)
			scene2->add(starTrans);
	}

	if (meshr)
	{
		scene2->add(meshr);
	}
	if (forest)
		scene2->add(forest);
	camera2->scene(scene2);

	// Create the main viewport and put some text up.
	rootvp = new ddgViewport(camera1,10,10, wwidth, wheight);
	rootvp->update(perf2 ? perfloop : update);

	rootvp->uigroup()->add(text1 = new ddgText(1,92));
	rootvp->uigroup()->add(text2 = new ddgText(1,87));
	rootvp->uigroup()->add(text3 = new ddgText(1,82));
	player->attach(rootvp);

	// Set up the rendering viewports.
	vps = new ddgViewportSet("Digital Dawn Graphics Terrain Engine",
		fullscreen);

#ifdef WIN32
	if (!perf2 && topviewOn && (fullscreen == 0))
	{
		int tvsize = 30; // percentage of total window size.
		// Try to get a squarish window about 30% of the width.
		int tvheight = (tvsize*wwidth) / wheight ;
		orthovp = new ddgViewport(camera2,100-tvsize,0,tvsize,tvheight);
		orthovp->slave(true);
		orthovp->control(false);
		vps->add(orthovp);
	}

#endif
	vps->add(rootvp);

	ddgConsole("Initializing..."<<endl);

	// Initialize viewports and scene graph.
	vps->init( argc, argv );
	// Check there was an error during initialization.
	if (ddgError::isError())
		goto error;

	ddgConsole("Rendering..."<<endl);
	// Define the shutdown function.
	vps->shutdown(shutdown);

	// Free the height map.
	delete height;

	ddgConsole(endl<<"Memory allocated: "<<ddgError::reportTotalInUse()<<" Max: "<<ddgError::reportMaxInUse()<<endl);

	// Enter the event loop.
	vps->enable();

error:
	return ddgError::report();
}
