/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
// ======================================================================
//
// Object:  ddgCamera Class
// Name:    ddgCamera
// Description:   Class which supports OpenGL camera capabilities.
//                Contains parameters to control various settings
//                As well as allowing camera motion control via GLUT.
//
// ----------------------------------------------------------------------

#include <iostream.h>

#include "render/ddgcamera.h"

ddgCamera::ddgCamera(ddgContext *ctx)
{	
	_scene = 0;
	_ortho = false;
	_ctx = ctx;
}

bool ddgCamera::init(void)
{
	// Setup the view from the camera.
	glShadeModel(GL_FLAT);	// TODO is this needed?
	GLint dp[2];
	glGetIntegerv(GL_VIEWPORT,_vp);
	glGetIntegerv(GL_DEPTH_RANGE,dp); //ERROR: doesn't return expected values.
	_ctx->clipbox()->setx(_vp[0],_vp[2]-_vp[0]);
	_ctx->clipbox()->sety(_vp[1],_vp[3]-_vp[1]);

	return (_scene ? _scene->init(_ctx) : 0);
}

void ddgCamera::ortho( unsigned int w, unsigned int h)
{
	_ortho = true;
	_width = w;
	_height = h;
}

void ddgCamera::update( void )
{
	_ctx->update();
}

void ddgCamera::draw(void)
{
	if (_ctx->dirty())
	{
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		if (_ortho)
		{
			glOrtho(_width/-2, _width/2, _height/-2, _height/2, 
				_ctx->clipbox()->cornerz(0), _ctx->clipbox()->cornerz(7));
		}
		else
		{
			gluPerspective( _ctx->fov(), _ctx->aspect(),
				_ctx->clipbox()->cornerz(0), _ctx->clipbox()->cornerz(7));
		}
	}
	ddgVector3 f2(_ctx->control()->position());
	f2.add(_ctx->forward());

	// Set the openGL orientation.
	glMatrixMode(GL_MODELVIEW);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	gluLookAt(_ctx->control()->position()->v[0], _ctx->control()->position()->v[1], _ctx->control()->position()->v[2],	// lens location
			  f2.v[0],  f2.v[1],  f2.v[2],		// view point/target
			  _ctx->up()->v[0],  _ctx->up()->v[1],  _ctx->up()->v[2]);	// up is in positive Y direction
	// Get the current orientation once per rendering.
	glGetFloatv(GL_MODELVIEW_MATRIX,_ctx->transformation()->m[0].v);
	glGetFloatv(GL_PROJECTION_MATRIX,_ctx->projection()->m[0].v);
	glGetIntegerv(GL_VIEWPORT,_vp);
	// Update the clipping frustrum.
	_ctx->updateClippingInfo();

	if (_scene)
	{
		if (_scene->draw( _ctx ))
		{
			ddgErrorSet(Render,"Scene Rendering error");
			ddgError::report();
			exit(ddgError::Render);
		}

	}
	_ctx->clean();
}



bool ddgCamera::map(ddgVector3 vin, ddgVector3 *vout)
{
	double mm[16], mp[16];
	double cx,cy,cz;
	glGetDoublev(GL_MODELVIEW_MATRIX,mm);
	glGetDoublev(GL_PROJECTION_MATRIX,mp);
	if (gluProject(vin[0],vin[1],vin[2],mm,mp,_vp,
		&cx,&cy,&cz) == GL_TRUE)
	{
		vout->set(cx,cy,cz);
		return ddgSuccess;
	}
	else
		return ddgFailure;

}

bool ddgCamera::unmap(ddgVector3 vin, ddgVector3 *vout)
{
	double mm[16], mp[16];
	double cx,cy,cz;
	glGetDoublev(GL_MODELVIEW_MATRIX,mm);
	glGetDoublev(GL_PROJECTION_MATRIX,mp);
	if (gluUnProject(vin[0],vin[1],vin[2],mm,mp,_vp,
		&cx,&cy,&cz) == GL_TRUE)
	{
		vout->set(cx,cy,cz);
		return ddgSuccess;
	}
	else
		return ddgFailure;
}





