/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
//
#include "render/ddgcylinder.h"
#include "util/ddgerror.h"

// ----------------------------------------------------------------------

ddgVector3 ddgCylinder::buf[300]; // 100 points top bottom and normal.
unsigned short ddgCylinder::bufsize = 0;
void ddgCylinder::init(float x, float y, float z, float rx, float ry,
                    float l, float r, float g, float b, float s, float e)
{
	pos.set(x,y,z);
	len = l;
	radiusx = rx;
	radiusy = ry;
	ts = s;
	te = e;
	inside = 0;
	botcap = 0;
	topcap = 0;
	botin = 0;
	topin = 0;
    color.set(r,b,g);
}

void ddgCylinder::drawq(ddgVector3* cap)
{
	ddgVector3 v;
	glColor3ubv(color.v);

	glBegin(GL_TRIANGLE_STRIP);
	int i = 0;
	while ( i < bufsize )
	{
		glNormal3fv(buf[i]); i++;	// Both points have the same normal.
		if (inside)
		{
			glVertex3fv(buf[i+1]); i++;
			glVertex3fv(buf[i-1]); i++;
		}
		else
		{
			glVertex3fv(buf[i]); i++;
			glVertex3fv(buf[i]); i++;
		}
	}
	glEnd();
	if (inside)
	{
		if (topcap)
		{
			glBegin(GL_TRIANGLE_FAN);
			glNormal3f(0, topin ? 1 :-1,0);
			v.set(&pos);
			if (!topin) v.add(cap);
			v.v[1] += len;
			glVertex3fv(v);
			int i = bufsize -2;
			while ( i > 0 )
			{
				glVertex3fv(buf[i]);
				i += -3;
			}
			glEnd();
		}
		if (botcap)
		{
			glBegin(GL_TRIANGLE_FAN);
			glNormal3f(0, botin ? -1 :1 ,0);
			v.set(&pos);
			if (!botin) v.add(cap);
			v.v[1] -= len;
			glVertex3fv(v);
			int i = 2;
			while ( i < bufsize )
			{
				glVertex3fv(buf[i]);
				i += 3;
			}
			glEnd();
		}
	}
	else // Not inside
	{
		if (topcap)
		{
			glBegin(GL_TRIANGLE_FAN);
			glNormal3f(0, topin ? -1 :1,0);
			v.set(&pos);
			if (!topin) v.add(cap);
			v.v[1] += len;
			glVertex3fv(v);
			int i = 1;
			while ( i < bufsize )
			{
				glVertex3fv(buf[i]);
				i += 3;
			}
			glEnd();
		}
		if (botcap)
		{
			glBegin(GL_TRIANGLE_FAN);
			glNormal3f(0, botin ? 1 :-1,0);
			v.set(&pos);
			if (!botin) v.add(cap);
			v.v[1] -= len;
			glVertex3fv(v);
			int i = bufsize-1;
			while ( i > 0 )
			{
				glVertex3fv(buf[i]);
				i += -3;
			}
			glEnd();
		}
	}
	// Load 1st point for next quadrant.
	buf[0] = buf[bufsize-3];
	buf[1] = buf[bufsize-2];
	buf[2] = buf[bufsize-1];
	bufsize = 3;
}

void ddgCylinder::calcq( ddgContext *ctx, float rs, float re, ddgVector3 *ps, ddgVector3 *pe)
{
	float rm = (rs+re)/2.0f;
	ddgAsserts(bufsize < 300,"Buffer overflow...");
	ddgAsserts(re-rs > 0.04,"Bailed out...");  // No more than 100 subdivisions for now.
	bool render = true;		                // Assume we are ready to render.
                                            // gets too small.
	ddgVector3 pa(ps);                    	// Average of p1 and p2
    pa.add(pe);
	pa.divide(2.0f);
    ddgVector3 nm(sinf(rm),0,cosf(rm));		// Normal vector.
	ddgVector3 pm(radiusx*nm[0],0,radiusy*nm[2]);// Calculated mid point.
	if (inside)
		nm.multiply(-1.0);
	// Translate object to its world space location.
    ddgVector3 pt(pm);
	if (re-rs < 0.05) render = true;         // Stop when difference between te and ts too small.
    else
    {
        // Try both coordinates.
        pt.add(&pos);
        pt.v[1] += len;
        pa.add(&pos);
        pa.v[1] += len;
	    // Check if we need to subdivide based on screen space error.
		render = ctx->ssdistance(&pt,&pa,&nm,true);
		
 	    if (render)
        {       // Try second coordinate.
			pt.v[1] -= len +len;
			pa.v[1] -= len +len;
			render = ctx->ssdistance(&pt,&pa,&nm,true);
        }
    }
	if (render)
	{
		if (inside) 
			buf[bufsize].set(-sinf(re),0,-cosf(re));
		else
			buf[bufsize].set(sinf(re),0,cosf(re));
		bufsize++;
		buf[bufsize] = pe;
		buf[bufsize].add(&pos);
		buf[bufsize].v[1] += len;
		bufsize++;
		buf[bufsize] = buf[bufsize-1];
		buf[bufsize].v[1] -= len + len;
		bufsize++;
	}
	else
	{
		calcq(ctx, rs, rm, ps, &pm);
		calcq(ctx, rm, re, &pm, pe);
	}
}

bool ddgCylinder::draw(ddgContext *ctx)
{
	bufsize = 0;
	float rs = ddgAngle::ttorad(ts), re = ddgAngle::ttorad(te);
	float r1, r2;
	ddgVector3 nm(sinf(rs),0,cosf(rs));
	ddgVector3 ps(radiusx*nm[0],nm[1],radiusy*nm[2]), pe;
	ddgVector3 pt(ps);
	if (inside)
		buf[bufsize].set(-nm[0],-nm[1],-nm[2]);	// Normal.
	else
		buf[bufsize].set(sinf(rs),0,cosf(rs));	// Normal.
	bufsize++;
	buf[bufsize] = ps + pos;					// Point1.					 
	buf[bufsize].v[1] += len;
	bufsize++;
	buf[bufsize] = buf[bufsize-1];			// Point2.
	buf[bufsize].v[1] -= len + len;
	bufsize++;
	int t = 0;
	// draw Q1.
	if (rs < ddgPi*0.5)
	{
		r1 = rs;
		r2 = re > ddgPi*0.5 ? ddgPi*0.5 : re;
		pe.set(radiusx*sinf(r2),0,radiusy*cosf(r2));
		calcq(ctx,r1,r2,&ps,&pe);
		t += bufsize /3;
		if (!topin || !botin) pt.set(radiusx,0,radiusy);
		drawq(&pt);
	}
	// draw Q2
	if (rs < ddgPi && re > ddgPi*0.5)
	{
		r1 = rs < ddgPi*0.5 ? ddgPi*0.5 : rs;
		r2 = re > ddgPi     ? ddgPi     : re;
		ps.set(radiusx*sinf(r1),0,radiusy*cosf(r1));
		pe.set(radiusx*sinf(r2),0,radiusy*cosf(r2));
		calcq(ctx,r1,r2,&ps,&pe);
		t += bufsize /3 ;
		if (!topin || !botin) pt.set(radiusx,0,-radiusy);
		drawq(&pt);
	}
	// draw Q3
	if (rs < ddgPi*1.5 && re > ddgPi )
	{
		r1 = rs < ddgPi     ? ddgPi     : rs;
		r2 = re > ddgPi*1.5 ? ddgPi*1.5 : re;
		ps.set(radiusx*sinf(r1),0,radiusy*cosf(r1));
		pe.set(radiusx*sinf(r2),0,radiusy*cosf(r2));
		calcq(ctx,r1,r2,&ps,&pe);
		t += bufsize /3 ;
		if (!topin || !botin) pt.set(-radiusx,0,-radiusy);
		drawq(&pt);
	}
	// draw Q4
	if (re > ddgPi*1.5)
	{
		r1 = rs < ddgPi*1.5 ? ddgPi*1.5 : rs;
		r2 = re;
		ps.set(radiusx*sinf(r1),0,radiusy*cosf(r1));
		pe.set(radiusx*sinf(r2),0,radiusy*cosf(r2));
		calcq(ctx,r1,r2,&ps,&pe);
		t += bufsize /3 ;
		if (!topin || !botin) pt.set(-radiusx,0,radiusy);
		drawq(&pt);
	}

	return false;
}


