/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "render/ddgfog.h"
// ----------------------------------------------------------------------

bool ddgFog::init(ddgContext *ctx)
{
	if ( super::init(ctx))
        return ddgFailure;
	if (_fog != NONE)
	{
		setColor();
		setDensity(_fogDensity);
		setType(_fog);
	}
    return ddgSuccess;
}

void ddgFog::setDensity( float d )
{
	_fogDensity = d;
	glFogf(GL_FOG_DENSITY,_fogDensity);
}

void ddgFog::setType( Type t )
{
	if (_fog != t)
	{
		_fog = t;
		switch(_fog)
		{
		case LINEAR:
			glFogi(GL_FOG_MODE,GL_LINEAR);
			glFogf(GL_FOG_START,_fogRange[0]);
			glFogf(GL_FOG_END,_fogRange[1]);
			break;
		case EXPONENTIAL:
			glFogi(GL_FOG_MODE,GL_EXP);
			break;
		case EXPONENTIAL2:
			glFogi(GL_FOG_MODE,GL_EXP2);
			break;
		}
	}
}

void ddgFog::setColor( ddgColor3 *c )
{
	ddgVector4 v;
	if (c)
		_fogColor.set(c);
	v.set( _fogColor[0], _fogColor[1], _fogColor[2], 1.0);
	v.divide(255);
	glFogfv(GL_FOG_COLOR,v);
}

bool ddgFog::draw( ddgContext *ctx )
{
	if (ctx->quality() == ddgContext::DEBUG)
		return ddgSuccess;

	bool retval = ddgSuccess;
	if (_fog != NONE)
		glEnable(GL_FOG);
	retval = super::draw(ctx);
	if (_fog != NONE)
		glDisable(GL_FOG);
	return retval;
}
