/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
// 
#include "render/ddggrass.h"
#include "render/ddgtexture.h"
/// Image used for displayed texture.

const unsigned int ddgNoGrassType = 1;
static char *g_grassImages[ddgNoGrassType] =
	{ "texture/grass3d.tga"};
ddgTexture	*g_grassTexture[ddgNoGrassType];
bool ddgGrass::ddgGrassInit = false;
ddgGrass::ddgGrass(ddgGrassType t, unsigned int depth, unsigned int density, float space)
{
	_type = t;
	_depth = depth;
	_density = density;
	_space = space;
}

bool ddgGrass::init(ddgContext *c)
{
    if (super::init(c))
        return ddgFailure;

    if (!ddgGrassInit)
	{
		ddgGrassInit = true;
		ddgStr		s;
		// Load the images & create textures
		int i;
		for (i = 0; i < ddgNoGrassType; i++)
		{
			ddgImage *cImg = new ddgImage();
			ddgMemorySet(ddgImage,1);
			s.assign(g_grassImages[i]);
			if (cImg->readFile(s))
			{
				unsigned int r, c, j;
				cImg->allocate(64,64,4);

				// Create a transparent image.
				for (r=0; r < cImg->rows(); r++)
				{
					for (c=0; c < cImg->cols(); c++)		
					{
						cImg->set(r,c,  255,255,255,0);
					}
				}
				// Add grass.
				for (j = 0; j < _density * cImg->rows() * cImg->cols() / 100; j++)
				{
					r = rand() % cImg->rows();
					c = rand() % cImg->cols();
					cImg->set(r,c,  100,255,100,255);
				}
				// Save it for use next time.
				if (cImg->writeTGA(g_grassImages[i]) == ddgFailure)
					return ddgFailure;
			}
			g_grassTexture[i] = new ddgTexture(cImg,false, true, ddgTexture::MODULATE);
			ddgMemorySet(ddgTexture,1);
			g_grassTexture[i]->init();

		}
	}
	return ddgSuccess;
}

bool ddgGrass::draw(ddgContext *ctx)
{
	glEnable(GL_ALPHA_TEST);
	glAlphaFunc(GL_GEQUAL,0.9);
	g_grassTexture[_type]->activate();
    glBegin(GL_QUADS);
	glNormal3f(0,1,0);
	float k = 0;
	unsigned int i;
	for (i = 0; i < _depth; i++)
	{
		k = (float)i * _space;
		glTexCoord2f(0,0);
		glVertex3f(0,k,0); 
		glTexCoord2f(2,0);
		glVertex3f(0,k,1); 
		glTexCoord2f(2,2);
		glVertex3f(1,k,1); 
		glTexCoord2f(0,2);
		glVertex3f(1,k,0); 
	}
	glEnd();
	g_grassTexture[_type]->deactivate();
    glDisable(GL_ALPHA_TEST);
	return ddgSuccess;
}

