/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <iomanip.h>


#include "math/ddgnoise.h"
#include "render/ddgground.h"

// ----------------------------------------------------------------------

ddgGround::ddgGround( ddgImage *i, ddgColor4 *color, float size ): super(ddgVArray::triangle), _dome(true)
{
	_image = i;
	_texture = new ddgTexture(_image,true);
	ddgMemorySet(ddgTexture,1);
	_texture->scale(_image->rows()*200,_image->cols()*200);
	_texture->mode(ddgTexture::MODULATE,color);
	_dome.sphere(size,20);
}

ddgGround::~ddgGround(void)
{
	delete _texture;
}

bool ddgGround::init( ddgContext *ctx)
{
	if (super::init(ctx)) return ddgFailure;

	varray.renderMode(true,false,false);
	_texture->init();

	_dome.init(&varray,true,false);

	return ddgSuccess;
}
// Render flat ground object.
bool ddgGround::draw(ddgContext *ctx )
{
	if (ctx->quality() == ddgContext::DEBUG)
		return ddgSuccess;

	if (_texture->color()->v[3] != 1.0)
	{
		glEnable(GL_ALPHA_TEST);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glEnable(GL_BLEND);
	}

	glShadeModel(GL_SMOOTH);

	_texture->activate();

	glNormal3f(0,1,0);
//	if (!listValid())
//	{
//		listStart();
		super::draw(ctx);
//		listEnd();
//	}
//	listRender();

	_texture->deactivate();

	if (_texture->color()->v[3] != 1.0)
	{
		glDisable(GL_ALPHA_TEST);
		glDisable(GL_BLEND);
	}

	return ddgSuccess;
}


