/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
// General includes


#include "render/ddglight.h"

// ====================================================================
unsigned int ddgLight::lightid = GL_LIGHT0;
// ----------------------------------------------------------------------
// Initialize a fixed white light at infinite distance.
ddgLight::ddgLight(void)
{
	_id = lightid++;
	_colour_ambient.set(25,25,25,255);
	_colour_diffuse.set(255,255,255,0);
	_colour_specular.set(255,255,255,0);
	_fixed = true;
    _spotlight = 0.0;
    _pos.set(0,0,0,0);
    _direction.set(1,1,1);
	_fixed = true;
	_fspotlight = false;
	_fdirection = false;
	_fpos = false;
	_fambient = true;
	_fdiffuse = true;
	_fspecular = false;
}

bool  ddgLight::draw( ddgContext *ctx)
{
	ddgVector4 v;
	glEnable((enum GLenum)_id);
	if (_fambient)
	{
		v.set( _colour_ambient[0], _colour_ambient[1], _colour_ambient[2], _colour_ambient[3]);
		v.divide(255);
		glLightfv((enum GLenum)_id, GL_AMBIENT, v);
	}
	if (_fdiffuse)
	{
		v.set( _colour_diffuse[0], _colour_diffuse[1], _colour_diffuse[2], _colour_diffuse[3]);
		v.divide(255);
		glLightfv((enum GLenum)_id, GL_DIFFUSE, v);
	}
	if (_fspecular)
	{
		v.set( _colour_specular[0], _colour_specular[1], _colour_specular[2], _colour_specular[3]);
		v.divide(255);
		glLightfv((enum GLenum)_id, GL_SPECULAR, v);
	}
	if (_fpos)
		glLightfv((enum GLenum)_id, GL_POSITION, _pos);
	if (_fdirection)
		glLightfv((enum GLenum)_id, GL_SPOT_DIRECTION, _direction);
	if (_fspotlight)
		glLightf((enum GLenum)_id, GL_SPOT_CUTOFF, _spotlight);
    return false;
}


bool ddgLightModel::draw( ddgContext *ctx )
{
	if (_state)
	{
		ddgVector4 v(_lightmodel_ambient[0], _lightmodel_ambient[1], _lightmodel_ambient[2], _lightmodel_ambient[3]);
		v.divide(255);
		glEnable(GL_LIGHTING);
		glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, _local? GL_TRUE:GL_FALSE);
		glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, _twosided? GL_TRUE:GL_FALSE);
		glLightModelfv(GL_LIGHT_MODEL_AMBIENT, v);
	}
    glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);
	if (_colorMaterial)
		glEnable (GL_COLOR_MATERIAL);

	bool retval = super::draw(ctx);
	if (_state)
		glDisable(GL_LIGHTING);
	return retval;
}
