/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
// General includes
#include "render/ddgmaterial.h"
// ----------------------------------------------------------------------

ddgMaterial::ddgMaterial(unsigned char r, unsigned char g, unsigned char b)
{
	_ambient.set(50,50,50,255);
	_diffuse.set(r,g,b,255);
	_emission.set(0,0,0,255);
	_specular.set(0,0,0,255);
	_shininess      = 0;
	_fambient = false;
	_fdiffuse = true;
	_femission = false;
	_fspecular = false;
	_fshininess = false;
}


/**
 * Store the ambient color.
 * r:   The red color component.
 * g:   The green color component.
 * b:   The blue color component.
 */
void ddgMaterial::ambient(unsigned char r, unsigned char g, unsigned char b, unsigned char t)
{
	_ambient.set(r,g,b,t);
	_fambient = true;
}


/**
 *      Store the ambient color.
 * r:   The red color component.
 * g:   The green color component.
 * b:   The blue color component.
 */
void ddgMaterial::diffuse(unsigned char r, unsigned char g, unsigned char b, unsigned char t)
{
	_diffuse.set(r,g,b,t);
	_fdiffuse = true;
}


//      Store the specularity color.
// r:   The red color component.
// g:   The green color component.
// b:   The blue color component.
void ddgMaterial::specular(unsigned char r, unsigned char g, unsigned char b, unsigned char t)
{
	_specular.set(r,g,b,t);
	_fspecular = true;
}


//      Store the emission color.
// r:   The red color component.
// g:   The green color component.
// b:   The blue color component.
void ddgMaterial::emission(unsigned char r, unsigned char g, unsigned char b, unsigned char t)
{
	_emission.set(r,g,b,t);
	_femission = true;
}


//      Store the shininess color.
void ddgMaterial::shininess(float s)
{
	_shininess = s;
	_fshininess = true;
}


bool ddgMaterial::draw( ddgContext *c )
{
	ddgVector4 v;
	if (_fambient)
	{
		v.set( _ambient[0], _ambient[1], _ambient[2], _ambient[3]);
		v.divide(255);
		glMaterialfv(GL_FRONT, GL_AMBIENT, v);
	}
	if (_fdiffuse)
	{
		v.set( _diffuse[0], _diffuse[1], _diffuse[2], _diffuse[3]);
		v.divide(255);
		glMaterialfv(GL_FRONT, GL_DIFFUSE, v);
	}
	if (_femission)
	{
		v.set( _emission[0], _emission[1], _emission[2], _emission[3]);
		v.divide(255);
		glMaterialfv(GL_FRONT, GL_EMISSION, v);
	}
	if (_fspecular)
	{
		v.set( _specular[0], _specular[1], _specular[2], _specular[3]);
		v.divide(255);
		glMaterialfv(GL_FRONT, GL_SPECULAR, v);
	}
	if (_fshininess)
	{
		glMaterialf(GL_FRONT, GL_SHININESS, _shininess);
	}
	return false;
}