/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "render/ddgobjects.h"
#include "util/ddgerror.h"
// ----------------------------------------------------------------------

bool ddgCameraObj::draw( ddgContext *ctx)
{
	if (ctx->mode() == ddgContext::MINIMAL) return ddgSuccess;
	
	glColor3ubv(_color->v);
	glLineWidth(1.0);
	glPushMatrix();
	ddgVector3	*pos = _camera->context()->control()->position(),
				*rot = _camera->context()->control()->orientation();
	glTranslatef(pos->v[0],pos->v[1],pos->v[2]);
	glRotatef(rot->v[0],1,0,0);
	glRotatef(rot->v[1],0,1,0);
	glRotatef(rot->v[2],0,0,1);
	glScalef(_scale,_scale,_scale);
	
	// Camera Box 
	glBegin(GL_LINE_LOOP);
	glVertex3f(-1,-1,2);
	glVertex3f( 1,-1,2);
	glVertex3f( 1,-1,0);
	glVertex3f(-1,-1,0);
	glEnd();
	glBegin(GL_LINE_LOOP);
	glVertex3f(-1,1,2);
	glVertex3f( 1,1,2);
	glVertex3f( 1,1,0);
	glVertex3f(-1,1,0);
	glEnd();
	
	glBegin(GL_LINES);
	glVertex3f(-1,-1,2); glVertex3f(-1, 1,2);
	glVertex3f( 1,-1,2); glVertex3f( 1, 1,2);
	glVertex3f( 1,-1,0); glVertex3f( 1, 1,0);
	glVertex3f(-1,-1,0); glVertex3f(-1, 1,0);
	glVertex3f( 0, 0,0); glVertex3f( 0, 2,0);	// Up
	glEnd();
	
	// Field of view frustrum.
	float dx = _camera->context()->tanHalfFOV();
	float dy = dx / _camera->context()->aspect();
	float n = _camera->context()->clipbox()->cornerz(0);
	float f = _camera->context()->clipbox()->cornerz(7);
	
	glBegin(GL_LINE_LOOP);
	glVertex3f(-dx * n,-dy * n, -n);
	glVertex3f( dx * n,-dy * n, -n);
	glVertex3f( dx * n, dy * n, -n);
	glVertex3f(-dx * n, dy * n, -n);
	glEnd();
	glBegin(GL_LINE_LOOP);
	glVertex3f(-dx * f,-dy * f, -f);
	glVertex3f( dx * f,-dy * f, -f);
	glVertex3f( dx * f, dy * f, -f);
	glVertex3f(-dx * f, dy * f, -f);
	glEnd();
	glBegin(GL_LINES);
	glVertex3f(-dx * n,-dy * n, -n); glVertex3f(-dx * f,-dy * f, -f);
	glVertex3f( dx * n,-dy * n, -n); glVertex3f( dx * f,-dy * f, -f);
	glVertex3f( dx * n, dy * n, -n); glVertex3f( dx * f, dy * f, -f);
	glVertex3f(-dx * n, dy * n, -n); glVertex3f(-dx * f, dy * f, -f);
	glEnd();
	
    glPopMatrix();
	
	return ddgSuccess;
}

bool ddgLightObj::draw( ddgContext *ctx)
{
	if (ctx->mode() == ddgContext::MINIMAL) return ddgSuccess;
	
	glLineWidth(1.0);
	glPushMatrix();
    ddgVector4 *pos = _light->pos();
    ddgVector3 *dir = _light->direction();

	glColor3ubv(_light->ambient()->v);

	glTranslatef(pos->v[0],pos->v[1],pos->v[2]);
    if (_light->spotlight())
    {
	    glRotatef(dir->v[0],1,0,0);
	    glRotatef(dir->v[1],0,1,0);
	    glRotatef(dir->v[2],0,0,1);
    }
	glScalef(_scale,_scale,_scale);
	
	// Light Box 
	glBegin(GL_LINE_LOOP);
	glVertex3f(0,0,1);
	glVertex3f(0,-1,0);
	glVertex3f(0,0,-1);
	glVertex3f(0,1,0);
	glEnd();
	glBegin(GL_LINE_LOOP);
	glVertex3f(-1,0,0);
	glVertex3f( 0,1,0);
	glVertex3f( 1,0,0);
	glVertex3f(0,-1,0);
	glEnd();
	glBegin(GL_LINE_LOOP);
	glVertex3f(-1,0,0);
	glVertex3f( 0,0,-1);
	glVertex3f( 1,0,0);
	glVertex3f( 0,0,1);
	glEnd();
	
	// Light rays
	glBegin(GL_LINES);
    // Cross
    if (_color)
        glColor3ubv(_color->v);
    else
        glColor3ubv(_light->diffuse()->v);
	glVertex3f(-1,-1,1); glVertex3f( 1, 1,-1);
	glVertex3f( 1,-1,1); glVertex3f(-1, 1,-1);
	glVertex3f( 1,-1,-1); glVertex3f(-1, 1,1);
	glVertex3f(-1,-1,-1); glVertex3f( 1, 1,1);
	glVertex3f( 0, 0,0); glVertex3f( 0, 2,0);	// Up
	glEnd();
	
    glColor3ub(0,255,255);
    if (_light->spotlight() > 0)
    {
	    // Field of view frustrum.
	    float dx = 2 * cosf(ddgAngle::degtorad(_light->spotlight()/2));
	    float dy = dx;
	    
	    glBegin(GL_LINE_LOOP);
	    glVertex3f(-dx,-dy, -2);
	    glVertex3f( dx,-dy, -2);
	    glVertex3f( dx, dy, -2);
	    glVertex3f(-dx, dy, -2);
	    glEnd();
	    glBegin(GL_LINES);
	    glVertex3f(0,0,0); glVertex3f(-dx,-dy, -2);
	    glVertex3f(0,0,0); glVertex3f( dx,-dy, -2);
	    glVertex3f(0,0,0); glVertex3f( dx, dy, -2);
	    glVertex3f(0,0,0); glVertex3f(-dx, dy, -2);
	    glEnd();
    }
    glPopMatrix();
	
	return ddgSuccess;
}

bool ddgAxis::draw( ddgContext *ctx)
{
	if (ctx->mode() == ddgContext::MINIMAL) return ddgSuccess;
	
	glLineWidth(1.0);
	glColor3ubv(color.v);
	glPushMatrix();
	glTranslatef(orig.v[0],orig.v[1],orig.v[2]);
	glBegin(GL_LINES);
    // X
    glVertex3f(d+d/10,-d/10,-d/10);
    glVertex3f(d+d/10, d/10, d/10);
    glVertex3f(d+d/10, d/10,-d/10);
    glVertex3f(d+d/10,-d/10, d/10);
    // Y
    glVertex3f(-d/10,d+d/10,-d/10);
    glVertex3f(0,    d+d/10,0);
    glVertex3f(0,    d+d/10,d/10);
    glVertex3f(0,    d+d/10,0);
    glVertex3f(d/10, d+d/10,-d/10);
    glVertex3f(0,    d+d/10,0);
    // Z
    glVertex3f(-d/10,-d/10,d+d/10);
    glVertex3f(d/10,-d/10,d+d/10);
    glVertex3f(-d/10,d/10,d+d/10);
    glVertex3f(d/10,d/10,d+d/10);
    glVertex3f(-d/10,-d/10,d+d/10);
    glVertex3f(d/10,d/10,d+d/10);
	glColor3ub(255,0,0);
    glVertex3f(0,0,0);
    glVertex3f(d,0,0);
	glColor3ub(0,255,0);
    glVertex3f(0,0,0);
    glVertex3f(0,d,0);
	glColor3ub(0,0,255);
    glVertex3f(0,0,0);
    glVertex3f(0,0,d);
	glEnd();
	glPopMatrix();
	return ddgSuccess;
}

bool ddgGrid::draw( ddgContext *ctx )
{
	if (ctx->mode() == ddgContext::MINIMAL) return ddgSuccess;
	
	int i;
	glLineWidth(1.0);
	glColor3ubv(color.v);
	glPushMatrix();
	glTranslatef(orig.v[0],orig.v[1],orig.v[2]);
	glBegin(GL_LINES);
	for (i = 0; i <= nw; i++ )
    {
		switch (dir)
		{
		case X:
			glVertex3f(0,i*w/nw,0);
			glVertex3f(0,i*w/nw,h);
			break;
		case Y:
			glVertex3f(i*w/nw,0,0);
			glVertex3f(i*w/nw,0,h);
			break;
		case Z:
			glVertex3f(0,i*w/nw,0);
			glVertex3f(h,i*w/nw,0);
			break;
		}
    }
	for (i = 0; i <= nh; i++ )
    {
		switch (dir)
		{
		case X:
			glVertex3f(0,0,i*h/nh);
			glVertex3f(0,w,i*h/nh);
			break;
		case Y:
			glVertex3f(0,0,i*h/nh);
			glVertex3f(w,0,i*h/nh);
			break;
		case Z:
			glVertex3f(i*h/nh,0,0);
			glVertex3f(i*h/nh,w,0);
			break;
		}
    }
	glEnd();
	glPopMatrix();
	return ddgSuccess;
}

bool ddgLattice::draw( ddgContext *ctx )
{
	if (ctx->mode() == ddgContext::MINIMAL) return ddgSuccess;
	
	float ow = w, oh = h, od = d, onw = nw, onh = nh, ond = nd;
#ifdef GL_POLYGON_OFFSET_EXT
	glPolygonOffsetEXT(1.5, 0.00001);
	glEnable(GL_POLYGON_OFFSET_EXT);
#endif
	// Bottom/Top
	dir = Y;
	w = ow; h = od; nw = onw; nh = ond;
	color.set(255,0,0);
	super::draw( ctx );
	glPushMatrix();
	glTranslatef(0,oh,0);
	super::draw( ctx );
	glPopMatrix();
	
	glColor3ub(255,200,200);
	glPushMatrix();
	glTranslatef(orig.v[0],orig.v[1],orig.v[2]);
	glBegin(GL_QUADS);
	glVertex3f(0,0,0);
	glVertex3f(0,0,od);
	glVertex3f(ow,0,od);
	glVertex3f(ow,0,0);
	glVertex3f(0,oh,0);
	glVertex3f(ow,oh,0);
	glVertex3f(ow,oh,od);
	glVertex3f(0,oh,od);
	glEnd();
	glPopMatrix();
	
	// Left/ Right
	dir = X;
	w = od; h = oh; nw = ond; nh = onh;
	color.set(0,255,0);
	super::draw( ctx );
	glPushMatrix();
	glTranslatef(ow,0,0);
	super::draw( ctx );
	glPopMatrix();
	glColor3ub(200,255,200);
	glPushMatrix();
	glTranslatef(orig.v[0],orig.v[1],orig.v[2]);
	glBegin(GL_QUADS);
	glVertex3f(0,0,0);
	glVertex3f(0,od,0);
	glVertex3f(0,od,ow);
	glVertex3f(0,0,ow);
	glVertex3f(ow,0,0);
	glVertex3f(ow,0,ow);
	glVertex3f(ow,od,ow);
	glVertex3f(ow,od,0);
	glEnd();
	glPopMatrix();
	
	// Front/Back
	dir = Z;
	w = ow; h = oh; nw = onw; nh = onh;
	color.set(0,0,255);
	super::draw( ctx );
	glPushMatrix();
	glTranslatef(0,0,od);
	super::draw( ctx );
	glPopMatrix();
	glColor3ub(200,200,255);
	glPushMatrix();
	glTranslatef(orig.v[0],orig.v[1],orig.v[2]);
	glBegin(GL_QUADS);
	glVertex3f(0,0,0);
	glVertex3f(ow,0,0);
	glVertex3f(ow,od,0);
	glVertex3f(0,od,0);
	glVertex3f(0,0,od);
	glVertex3f(0,od,od);
	glVertex3f(ow,od,od);
	glVertex3f(ow,0,od);
	glEnd();
	glPopMatrix();
	
#ifdef GL_POLYGON_OFFSET_EXT
	glDisable(GL_POLYGON_OFFSET_EXT);
#endif
	
	return ddgSuccess;
}



bool ddgLineObj::draw( ddgContext *ctx )
{
	// I am ambiguous about the next line. hrm...
	if (ctx->mode() == ddgContext::MINIMAL) return ddgSuccess;

	glLineWidth(1.0);
	glColor3ubv(color.v);
	glPushMatrix();
	glTranslatef(origin->v[0],origin->v[1],origin->v[2]);
	glBegin(GL_LINES);
		  glVertex3f(0,0,0);
		  glVertex3f(direction->v[0],direction->v[1],direction->v[2]);
	glEnd();
	glPopMatrix();
	return ddgSuccess;
}



