/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifdef _WIN32
#include <windows.h>
#endif
#include <iomanip.h>
#include "util/ddgerror.h"

#include "glrender/ddgoglxt.h"

//NV_vertex_array_range -- wglext.h
typedef void * (APIENTRY * PFNWGLALLOCATEMEMORYNVPROC) (int size, float readfreq, float writefreq, float priority);
typedef void (APIENTRY * PFNWGLFREEMEMORYNVPROC) (void *pointer);

// NV_fence -- glext.h
#ifndef GL_NV_fence
#define GL_NV_fence
#define GL_ALL_COMPLETED_NV               0x84F2
#define GL_FENCE_STATUS_NV                0x84F3
#define GL_FENCE_CONDITION_NV             0x84F4
typedef void (APIENTRY * PFNGLDELETEFENCESNVPROC) (GLsizei n, const GLuint *fences);
typedef void (APIENTRY * PFNGLGENFENCESNVPROC) (GLsizei n, GLuint *fences);
typedef GLboolean (APIENTRY * PFNGLISFENCENVPROC) (GLuint fence);
typedef GLboolean (APIENTRY * PFNGLTESTFENCENVPROC) (GLuint fence);
typedef void (APIENTRY * PFNGLGETFENCEIVNVPROC) (GLuint fence, GLenum pname, GLint *params);
typedef void (APIENTRY * PFNGLFINISHFENCENVPROC) (GLuint fence);
typedef void (APIENTRY * PFNGLSETFENCENVPROC) (GLuint fence, GLenum condition);
#endif

//#define GLH_EXT_SINGLE_FILE
#include "glext.h"
#include "glh_genext.h"
#include "glh_obs.h"
#include "glh_glut.h"


#ifdef _WIN32
WINGDIAPI PROC  WINAPI wglGetProcAddress(LPCSTR);

/* ARB_multitexture command function pointers */
PFNGLMULTITEXCOORD2IARBPROC glMultiTexCoord2iARB;
PFNGLMULTITEXCOORD3FARBPROC glMultiTexCoord3fARB;
PFNGLMULTITEXCOORD3FVARBPROC glMultiTexCoord3fvARB;
PFNGLACTIVETEXTUREARBPROC glActiveTextureARB;

/* NV_register_combiners command function pointers */
PFNGLCOMBINERPARAMETERFVNVPROC glCombinerParameterfvNV = 0;
PFNGLCOMBINERPARAMETERIVNVPROC glCombinerParameterivNV = 0;
PFNGLCOMBINERPARAMETERFNVPROC glCombinerParameterfNV = 0;
PFNGLCOMBINERPARAMETERINVPROC glCombinerParameteriNV = 0;
PFNGLCOMBINERINPUTNVPROC glCombinerInputNV = 0;
PFNGLCOMBINEROUTPUTNVPROC glCombinerOutputNV = 0;
PFNGLFINALCOMBINERINPUTNVPROC glFinalCombinerInputNV = 0;
PFNGLGETCOMBINERINPUTPARAMETERFVNVPROC glGetCombinerInputParameterfvNV = 0;
PFNGLGETCOMBINERINPUTPARAMETERIVNVPROC glGetCombinerInputParameterivNV = 0;
PFNGLGETCOMBINEROUTPUTPARAMETERFVNVPROC glGetCombinerOutputParameterfvNV = 0;
PFNGLGETCOMBINEROUTPUTPARAMETERIVNVPROC glGetCombinerOutputParameterivNV = 0;
PFNGLGETFINALCOMBINERINPUTPARAMETERFVNVPROC glGetFinalCombinerInputfvNV = 0;
PFNGLGETFINALCOMBINERINPUTPARAMETERIVNVPROC glGetFinalCombinerInputivNV = 0;

PFNGLLOCKARRAYSEXTPROC glLockArraysEXT = 0;
PFNGLUNLOCKARRAYSEXTPROC glUnlockArraysEXT = 0;

PFNGLARRAYELEMENTEXTPROC glArrayElementEXT = 0;
PFNGLCOLORPOINTEREXTPROC glColorPointerEXT = 0;
PFNGLDRAWARRAYSEXTPROC glDrawArraysEXT = 0;
PFNGLEDGEFLAGPOINTEREXTPROC glEdgeFlagPointerEXT = 0;
PFNGLGETPOINTERVEXTPROC glGetPointerVEXT = 0;
PFNGLINDEXPOINTEREXTPROC glIndexPointerEXT = 0;
PFNGLNORMALPOINTEREXTPROC glNormalPointerEXT = 0;
PFNGLTEXCOORDPOINTEREXTPROC glTexCoordPointerEXT = 0;
PFNGLVERTEXPOINTEREXTPROC glVertexPointerEXT = 0;
#ifdef GL_NV_vertex_array_range
PFNGLFLUSHVERTEXARRAYRANGENVPROC glFlushVertexArrayRangeNV = 0;
PFNGLVERTEXARRAYRANGENVPROC glVertexArrayRangeNV = 0;
PFNWGLALLOCATEMEMORYNVPROC wglAllocateMemoryNV = 0;
PFNWGLFREEMEMORYNVPROC wglFreeMemoryNV  = 0;
#endif
#ifdef GL_NV_fence
PFNGLGENFENCESNVPROC glGenFencesNV = 0;
PFNGLDELETEFENCESNVPROC glDeleteFencesNV = 0;
PFNGLSETFENCENVPROC glSetFenceNV = 0;
PFNGLTESTFENCENVPROC glTestFenceNV = 0;
PFNGLFINISHFENCENVPROC glFinishFenceNV = 0;
PFNGLISFENCENVPROC glIsFenceNV = 0;
PFNGLGETFENCEIVNVPROC glGetFenceivNV = 0;
#endif

#endif

/// true if we support locked vertex arrays.
bool	ddgOpenGLExt::_lockedArrays = false;
/// true if we support the window's native color buffer.
bool	ddgOpenGLExt::_brga = false;
/// true if we support NVidia's register combiners extentions.
bool	ddgOpenGLExt::_registerCombiners = false;
/// true if we support multitexturing.
bool	ddgOpenGLExt::_multiTexture = false;
/// true if we support cube mapping.
bool	ddgOpenGLExt::_cubeMapping = false;
/// true if we support swap control for the color buffers.
bool	ddgOpenGLExt::_swapControl = false;
/// true if we support destination alpha for specular effects.
bool	ddgOpenGLExt::_destinationAlpha = false;
/// true if we support nvidia fenced memory allocation.
bool	ddgOpenGLExt::_NV_fence = false;
/// true if we support nvidia ranged array rendering.
bool	ddgOpenGLExt::_NV_vertex_array_range = false;

bool ddgOpenGLExt::extensionSupported(const char *extension)
{
	static const GLubyte *extensions = NULL;
	const GLubyte *start;
	GLubyte *where, *terminator;

	/* Extension names should not have spaces. */
	where = (GLubyte *) strchr(extension, ' ');
	if (where || *extension == '\0')
		return false;

	if (!extensions)
	{
		extensions = glGetString(GL_EXTENSIONS);
		if (!extensions)
			return false;
	}
	/* It takes a bit of care to be fool-proof about parsing the
	 OpenGL extensions string.  Don't be fooled by sub-strings,
	 etc. */
	start = extensions;
	for (;;)
	{
		/* If your application crashes in the strstr routine below,
		   you are probably calling glutExtensionSupported without
		   having a current window.  Calling glGetString without
		   a current OpenGL context has unpredictable results.
		   Please fix your program. */
		where = (GLubyte *) strstr((const char *) start, extension);
		if (!where)
			break;
		terminator = where + strlen(extension);
		if (where == start || *(where - 1) == ' ')
		{
			if (*terminator == ' ' || *terminator == '\0')
			{
				return true;
			}
		}
		start = terminator;
	}
	return false;
}

/* Check if required extensions exist. */
/// Test for the available extentions.
bool ddgOpenGLExt::init(void )
{

	_brga = extensionSupported("GL_EXT_bgra");

	_multiTexture = extensionSupported("GL_ARB_multitexture");

	_registerCombiners = extensionSupported("GL_NV_register_combiners");

	_cubeMapping = extensionSupported("GL_EXT_texture_cube_map");

	_swapControl = extensionSupported("WGL_EXT_swap_control");

	_lockedArrays = extensionSupported("GL_EXT_compiled_vertex_array");

	_NV_fence = extensionSupported("GL_NV_fence");

	_NV_vertex_array_range = extensionSupported("GL_NV_vertex_array_range");

	GLint alphaBits;

	glGetIntegerv(GL_ALPHA_BITS, &alphaBits);

	_destinationAlpha = (alphaBits > 0)? true : false;


#ifdef WIN32
	if (_multiTexture)
	{
		/* Retrieve some ARB_multitexture routines. */
		glMultiTexCoord2iARB =
			(PFNGLMULTITEXCOORD2IARBPROC)
			wglGetProcAddress("glMultiTexCoord2iARB");
		glMultiTexCoord3fARB =
			(PFNGLMULTITEXCOORD3FARBPROC)
			wglGetProcAddress("glMultiTexCoord3fARB");
		glMultiTexCoord3fvARB =
			(PFNGLMULTITEXCOORD3FVARBPROC)
			wglGetProcAddress("glMultiTexCoord3fvARB");
		glActiveTextureARB =
			(PFNGLACTIVETEXTUREARBPROC)
			wglGetProcAddress("glActiveTextureARB");
        if (NULL == glActiveTextureARB)
            return ddgFailure;
	}

	if (_registerCombiners)
	{
		/* Retrieve all NV_register_combiners routines. */
		glCombinerParameterfvNV =
			(PFNGLCOMBINERPARAMETERFVNVPROC)
			wglGetProcAddress("glCombinerParameterfvNV");
		glCombinerParameterivNV =
			(PFNGLCOMBINERPARAMETERIVNVPROC)
			wglGetProcAddress("glCombinerParameterivNV");
		glCombinerParameterfNV =
			(PFNGLCOMBINERPARAMETERFNVPROC)
			wglGetProcAddress("glCombinerParameterfNV");
		glCombinerParameteriNV =
			(PFNGLCOMBINERPARAMETERINVPROC)
			wglGetProcAddress("glCombinerParameteriNV");
		glCombinerInputNV =
			(PFNGLCOMBINERINPUTNVPROC)
			wglGetProcAddress("glCombinerInputNV");
		glCombinerOutputNV =
			(PFNGLCOMBINEROUTPUTNVPROC)
			wglGetProcAddress("glCombinerOutputNV");
		glFinalCombinerInputNV =
			(PFNGLFINALCOMBINERINPUTNVPROC)
			wglGetProcAddress("glFinalCombinerInputNV");
		glGetCombinerInputParameterfvNV =
			(PFNGLGETCOMBINERINPUTPARAMETERFVNVPROC)
			wglGetProcAddress("glGetCombinerInputParameterfvNV");
		glGetCombinerInputParameterivNV =
			(PFNGLGETCOMBINERINPUTPARAMETERIVNVPROC)
			wglGetProcAddress("glGetCombinerInputParameterivNV");
		glGetCombinerOutputParameterfvNV =
			(PFNGLGETCOMBINEROUTPUTPARAMETERFVNVPROC)
			wglGetProcAddress("glGetCombinerOutputParameterfvNV");
		glGetCombinerOutputParameterivNV =
			(PFNGLGETCOMBINEROUTPUTPARAMETERIVNVPROC)
			wglGetProcAddress("glGetCombinerOutputParameterivNV");
		// These two may not be present.
		glGetFinalCombinerInputfvNV =
			(PFNGLGETFINALCOMBINERINPUTPARAMETERFVNVPROC)
			wglGetProcAddress("glGetFinalCombinerInputfvNV");
		glGetFinalCombinerInputivNV =
			(PFNGLGETFINALCOMBINERINPUTPARAMETERIVNVPROC)
			wglGetProcAddress("glGetFinalCombinerInputivNV");
        if (NULL == glGetCombinerOutputParameterivNV)
            return ddgFailure;
	}

    if (_NV_fence) {
        glGenFencesNV = (PFNGLGENFENCESNVPROC)wglGetProcAddress("glGenFencesNV");
        if (NULL == glGenFencesNV)
            return ddgFailure;
        glDeleteFencesNV = (PFNGLDELETEFENCESNVPROC)wglGetProcAddress("glDeleteFencesNV");
        if (NULL == glDeleteFencesNV)
            return ddgFailure;
        glSetFenceNV = (PFNGLSETFENCENVPROC)wglGetProcAddress("glSetFenceNV");
        if (NULL == glSetFenceNV)
            return ddgFailure;
        glTestFenceNV = (PFNGLTESTFENCENVPROC)wglGetProcAddress("glTestFenceNV");
        if (NULL == glTestFenceNV)
            return ddgFailure;
        glFinishFenceNV = (PFNGLFINISHFENCENVPROC)wglGetProcAddress("glFinishFenceNV");
        if (NULL == glFinishFenceNV)
            return ddgFailure;
        glIsFenceNV = (PFNGLISFENCENVPROC)wglGetProcAddress("glIsFenceNV");
        if (NULL == glIsFenceNV)
            return ddgFailure;
        glGetFenceivNV = (PFNGLGETFENCEIVNVPROC)wglGetProcAddress("glGetFenceivNV");
        if (NULL == glGetFenceivNV)
            return ddgFailure;
	}
    if (_NV_vertex_array_range) {
        glFlushVertexArrayRangeNV = (PFNGLFLUSHVERTEXARRAYRANGENVPROC)wglGetProcAddress("glFlushVertexArrayRangeNV");
        if (NULL == glFlushVertexArrayRangeNV)
            return ddgFailure;
        glVertexArrayRangeNV = (PFNGLVERTEXARRAYRANGENVPROC)wglGetProcAddress("glVertexArrayRangeNV");
        if (NULL == glVertexArrayRangeNV)
            return ddgFailure;
        wglAllocateMemoryNV = (PFNWGLALLOCATEMEMORYNVPROC)wglGetProcAddress("wglAllocateMemoryNV");
        if (NULL == wglAllocateMemoryNV)
            return ddgFailure;
        wglFreeMemoryNV = (PFNWGLFREEMEMORYNVPROC)wglGetProcAddress("wglFreeMemoryNV");
        if (NULL == wglFreeMemoryNV)
            return ddgFailure;
	}

	if (_lockedArrays)
	{
		glLockArraysEXT		= (PFNGLLOCKARRAYSEXTPROC)	wglGetProcAddress("glLockArraysEXT");
		if (NULL == glLockArraysEXT)
			return ddgFailure;
		glUnlockArraysEXT	= (PFNGLUNLOCKARRAYSEXTPROC) wglGetProcAddress("glUnlockArraysEXT");
		if (NULL == glUnlockArraysEXT)
			return ddgFailure;
	}
#endif WIN32

	return ddgSuccess;
}

/// Return the availability of locked vertex arrays.
bool ddgOpenGLExt::lockedArrays(void) { return _lockedArrays; }
/// Return the availability of brga color buffer.
bool ddgOpenGLExt::brga(void) { return _brga; }
/// Return the availability of NVIDIA's register combiners.
bool ddgOpenGLExt::registerCombiners(void) { return _registerCombiners; }
/// Return the availability of multitexture capabilities.
bool ddgOpenGLExt::multiTexture(void) { return _multiTexture; }
/// Return the availability of cubemapping functions.
bool ddgOpenGLExt::cubeMapping(void) { return _cubeMapping; }
/// Return the availability of color buffer swap interval control.
bool ddgOpenGLExt::swapControl(void) { return _swapControl; }
/// Return the destination alpha frame buffers.
bool ddgOpenGLExt::destinationAlpha(void) { return _destinationAlpha; }

void ddgOpenGLExt::lockArrays( unsigned int s, unsigned int c)
{
	ddgAsserts(_lockedArrays,"Forgot to initialize extention");
	glLockArraysEXT(s,c);
}

void ddgOpenGLExt::unlockArrays(void)
{
	ddgAsserts(_lockedArrays,"Forgot to initialize extention");
	glUnlockArraysEXT();
}

void *ddgOpenGLExt::allocateMemoryNV(size_t size, float p)
{
	ddgAsserts(_NV_vertex_array_range,"Forgot to initialize extention");
    return wglAllocateMemoryNV(size, 0, 0, p);
}

void ddgOpenGLExt::freeMemoryNV(void *buffer )
{
	ddgAsserts(_NV_vertex_array_range,"Forgot to initialize extention");
	wglFreeMemoryNV(buffer);
}