/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef _ddgOpenGLExt_Class_
#define _ddgOpenGLExt_Class_

#include "struct/ddgnode.h"


/**
 * Determine the OpenGL extentions state at run time.
 * This object should be added somewhere at the top of the scene graph.
 * subsequent objects can then query the state of this object to determine
 * the availability of various features.
 */
class WEXP ddgOpenGLExt {
    typedef ddgNode super;
	/// true if we support locked vertex arrays.
	static bool	_lockedArrays;
	/// true if we support the window's native color buffer.
	static bool	_brga;
	/// true if we support NVidia's register combiners extentions.
	static bool	_registerCombiners;
	/// true if we support multitexturing.
	static bool	_multiTexture;
	/// true if we support cube mapping.
	static bool	_cubeMapping;
	/// true if we support swap control for the color buffers.
	static bool	_swapControl;
	/// true if we support destination alpha for specular effects.
	static bool	_destinationAlpha;
	/// true if we support NVidia fenced arrays.
	static bool _NV_fence;
	/// true if we support NVidia ranged arrays.
	static bool _NV_vertex_array_range;
public:
	/// Test for the available extentions.
	static bool init(void );
	// Check if the extention given by name is supported.
	static bool extensionSupported(const char *extension);
	/// Return the availability of locked vertex arrays.
	static bool lockedArrays(void);
	/// Return the availability of brga color buffer.
	static bool brga(void);
	/// Return the availability of NVIDIA's register combiners.
	static bool registerCombiners(void);
	/// Return the availability of multitexture capabilities.
	static bool multiTexture(void);
	/// Return the availability of cubemapping functions.
	static bool cubeMapping(void);
	/// Return the availability of color buffer swap interval control.
	static bool swapControl(void);
	/// Return the destination alpha frame buffers.
	static bool destinationAlpha(void);
	/// Lock vertex arrays if extention is available.
	static void lockArrays( unsigned int s, unsigned int c);
	/// Unlock vertex arrays.
	static void unlockArrays(void);
	/// Allocate video or fast memory.
	static void *allocateMemoryNV(size_t size, float p = 0.5f);
	/// Free video or fast memory.
	static void freeMemoryNV(void *buffer );

};


        
#endif
