/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "render/ddgparticle.h"
	
ddgParticleCloud::ddgParticleCloud( ddgParticleType t, int s )
{
	_type = t;
	_size = s;
	drag = 0.001;
	force.set(0,-0.0008,0);
	color.set(40,40,255);
	_particle = 0;
	_texture = 0;
}

ddgParticleCloud::~ddgParticleCloud( )
{
	delete [] _particle;
	_size = 0;
	delete _texture;
	_texture = 0;
}

static char *g_particlecloudImage[] ={ "texture/flare.tga", NULL};

bool ddgParticleCloud::init( ddgContext *ctx )
{
	int i;
	if (super::init(ctx))
		return ddgFailure;

	_particle = new ddgParticle[_size];
	ddgMemorySet(ddgParticle,_size);

	ddgStr		s;
	ddgImage *cImg = new ddgImage();
	ddgMemorySet(ddgImage,1);
	s.assign(g_particlecloudImage[_type]);
	if (cImg->readFile(s))
		return ddgFailure;

	cImg->mapBrightNessToAlpha();

	_texture = new ddgTexture(cImg,true,true,ddgTexture::MODULATE);
	ddgMemorySet(ddgTexture,1);
	_texture->init();		

	// Initialize the particle cloud the first time
	for (i=0;i<_size;i++)				
	{
		_particle[i]._life=0.0;
	}

	return ddgSuccess;
}


bool ddgParticleCloud::draw( ddgContext *ctx)
{
	int i;
	glEnable(GL_BLEND);
	glEnable(GL_ALPHA_TEST);
	glAlphaFunc(GL_GEQUAL,0.3);

	glBlendFunc(GL_SRC_ALPHA,GL_ONE);
	glColor3ubv(color);
	_texture->activate();
	// Render all the particles.
	glBegin(GL_QUADS);
	for (i=0;i<_size;i++)					 
	{
		// If Particle Is Burned Out
		if (_particle[i]._life<=0.0f)					
			{
			_particle[i]._life=1.0f;					// Give It New Life
			_particle[i]._fade=float(rand()%100)/1000.0f+0.003f;	// Random Fade Value
			_particle[i].p.set(0,0,0);						// start in emitter again					
			_particle[i].d.set(float((rand()%60))-32.0f,float((rand()%60)-30.0f),float((rand()%60)-30.0f));
			_particle[i].d.divide(10.0);
			}

		float x=_particle[i].p[0];
		float y=_particle[i].p[1];
		float z=_particle[i].p[2];
	
		glNormal3f(0,0,1);
		//draw one particle 
        glTexCoord2f(0,0);

		glVertex3f(x-0.3f, y-0.3f,z);  // Bottom Left
		
		glTexCoord2f(1,0);
		glVertex3f(x+0.3f, y-0.3f,z);  // Bottom Right
		
		glTexCoord2f(1,1);
		glVertex3f(x+0.3f, y+0.3f,z);  // Top Right
		
		glTexCoord2f(0,1);
		glVertex3f(x-0.3f, y+0.3f,z);  // Bottom Right
          
		_particle[i].p.v[0]+=_particle[i].d[0]*drag+force[0];
		_particle[i].p.v[1]+=_particle[i].d[1]*drag+force[1];
		_particle[i].p.v[2]+=_particle[i].d[2]*drag+force[2];

		_particle[i]._life-=_particle[i]._fade;	// Reduce Particles Life By 'Fade'
	}
	glEnd();
	
	_texture->deactivate();
    glDisable(GL_ALPHA_TEST);
    glDisable(GL_BLEND);

	return ddgSuccess;
}







			

       