/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <iomanip.h>


#include "math/ddgnoise.h"
#include "render/ddgreflection.h"

// ----------------------------------------------------------------------

ddgReflection::ddgReflection( void  )
{
	_polygon = new ddgVector3[5];
	_polygon[0].set(-200,0.5,-200);
	_polygon[1].set( 200,0.5,-200);
	_polygon[2].set( 200,0.5, 200);
	_polygon[3].set(-200,0.5, 200);
}

ddgReflection::~ddgReflection( void  )
{
	delete [] _polygon;
}

bool ddgReflection::init( ddgContext *ctx)
{
	if (super::init(ctx)) return ddgFailure;

	return ddgSuccess;
}
// Render flat ground object.
bool ddgReflection::draw(ddgContext *ctx )
{
	if (ctx->quality() == ddgContext::DEBUG)
		return ddgSuccess;

      /* We can eliminate the visual "artifact" of seeing the "flipped"
	 dinosaur underneath the floor by using stencil.  The idea is
	 draw the floor without color or depth update but so that 
	 a stencil value of one is where the floor will be.  Later when
	 rendering the dinosaur reflection, we will only update pixels
	 with a stencil value of 1 to make sure the reflection only
	 lives on the floor, not below the floor. */
/*
      // Don't update color or depth.
      glDisable(GL_DEPTH_TEST);
      glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);

      // Draw 1 into the stencil buffer.
      glEnable(GL_STENCIL_TEST);
      glStencilOp(GL_REPLACE, GL_REPLACE, GL_REPLACE);
      glStencilFunc(GL_ALWAYS, 1, 0xffffffff);
*/
      /* Now render floor; floor pixels just get their stencil set to 1. */
	/*
	  glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
	  glBegin(GL_POLYGON);
	  for (i=0;i<4;i++)
		  glVertex3fv(_polygon[i].v);
	  glEnd();
	*/
/*
  glDisable(GL_LIGHTING);
  glBegin(GL_QUADS);
    glVertex3f(-200.0, 3.0, 200.0);
    glVertex3f(200.0, 3.0, 200.0);
    glVertex3f(200.0, 3.0, -200.0);
    glVertex3f(-200.0, 3.0, -200.0);
  glEnd();
  glEnable(GL_LIGHTING);

      // Re-enable update of color and depth.
      glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
      glEnable(GL_DEPTH_TEST);

      // Now, only render where stencil is set to 1. 
      glStencilFunc(GL_EQUAL, 1, 0xffffffff);  // draw if ==1 
      glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
*/
      glPushMatrix();

      /* The critical reflection step: Reflect dinosaur through the floor
         (the Y=0 plane) to make a relection. */
      glScalef(1.0, -1.0, 1.0);

      /* Unfortunately the back face culling reverses when we reflect
	 using scaling.  Easy solution is just disable back face culling for
	 rendering the reflection.  Also, the normals for lighting get screwed
	 up by the scale; enabled normalize to ensure normals are still
	 properly normalized despite the scaling. */
//     glEnable(GL_NORMALIZE);
     glCullFace(GL_FRONT);

      /* Draw the reflected scene. */
	  super::draw(ctx);

      /* Disable normalize again and re-enable back face culling. */
//     glDisable(GL_NORMALIZE);
     glCullFace(GL_BACK);

      glPopMatrix();

//     glDisable(GL_STENCIL_TEST);

	return ddgSuccess;
}


// ----------------------------------------------------------------------
static int g_cplanes[4] = {GL_CLIP_PLANE0,GL_CLIP_PLANE1,GL_CLIP_PLANE2,GL_CLIP_PLANE3};
static int g_used = 0;
double g_ceq[4][4];
ddgClipper::ddgClipper( void  )
{
	_id = g_used;
	g_used++;
	ddgAssert(g_used < 5);
	// Default is a horizontal clip plane at Y = 0.
	plane.set(new ddgVector3(0,1,0),0);
}

ddgClipper::~ddgClipper( void  )
{
}

bool ddgClipper::init( ddgContext *ctx)
{
	if (super::init(ctx)) return ddgFailure;

	g_ceq[_id][0] = plane.n[0];
	g_ceq[_id][1] = plane.n[1];
	g_ceq[_id][2] = plane.n[2];
	g_ceq[_id][3] = -1*plane.d;
	return ddgSuccess;
}


bool ddgClipper::draw(ddgContext *ctx )
{
	if (ctx->quality() == ddgContext::DEBUG)
		return false;

	glClipPlane(g_cplanes[_id], g_ceq[_id]);  
	glEnable(g_cplanes[_id]);  

	super::draw(ctx);
	glDisable(g_cplanes[_id]);

	return ddgSuccess;
}
