/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <iomanip.h>

#include "render/ddgsky.h"

// ----------------------------------------------------------------------
/* Gradiated sky.
 */
ddgSky::ddgSky(float size)
{
	// Set the colours that make up the horizon and so on.
	_dome.sphere(size,20);
	_cday.set(100,100,255,255);
	_ctwilight.set(100,0,50,255);
	_cnight.set(50,50,50,255);
	varray.renderMode(false,false,false);
}

bool ddgSky::init(ddgContext *ctx )
{
    if (super::init(ctx)) return ddgFailure;
 
	_dome.init(&varray,false,false);
    return ddgSuccess;
}

bool ddgSky::draw(ddgContext *ctx )
{
	if (ctx->quality() == ddgContext::DEBUG)
		return false;

    // 1.0 = 100 light blocked (noon).
	float day = ctx->clock()->getDayLight();
	static ddgColor4 cnow;
	if (day < 0.5)
		cnow.linterp(_cnight,_ctwilight,day*2);
	else
		cnow.linterp(_ctwilight,_cday,(day-0.5)*2);

    // Color of sky.
	glColor4ubv(cnow.v);

	if (!listValid())
	{
		listStart();

		// Draw sky dome.
		// Fades in and stars fade out.
		glEnable(GL_BLEND);
		glShadeModel(GL_SMOOTH);
		// I would like to use GL_DST_ALPHA to keep sun active and stars dimmer, but it doesnt work.
		glBlendFunc(GL_SRC_ALPHA,GL_DST_COLOR);

		super::draw(ctx);

		listEnd();
	}
	listRender();

    return false;
}



