/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
// General includes
#include "render/ddgtexture.h"
// ----------------------------------------------------------------------

bool ddgTexture::init(void)
{
	ddgAsserts(_image && _image->cols() < 2049 && _image->rows() < 2049 && _image->channels() < 5, "Texture size too large");
	/// Image part of allocated size.
	unsigned int txtwidth = pow(2.0,(int)(log(_image->cols()-1)/log(2.0)+1.0));
	/// Image part of allocated size.
	unsigned int txtheight = pow(2.0,(int)(log(_image->rows()-1)/log(2.0)+1.0));
	// For GL textures memory must be allocated in powers of 2 + 1.
	// so check that the current image complies, if not create and
	// image which does.
	if (   _image->cols() != txtwidth
		|| _image->rows() != txtheight)
	{
		ddgImage *pimage = new ddgImage( txtwidth, txtheight, _image->channels());
		ddgMemorySet(ddgImage,1);
		// Copy the data from one image to the other.
		pimage->import(_image,0,0,0,0,_image->cols(),_image->rows());
		_image = pimage;
	}

    glGenTextures( 1, &_textureID );
	glBindTexture(GL_TEXTURE_2D, _textureID );

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, _repeat ? GL_REPEAT : GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, _repeat ? GL_REPEAT : GL_CLAMP);

    // Set up the filters used to minify and magnify the texture when
	// mapping to the surface.
	if (_linear ) {
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	} else {
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	}

    // Set up the two-dimensional texture map as created previously.
    if (_image->channels() == 1)
      gluBuild2DMipmaps(GL_TEXTURE_2D, GL_LUMINANCE, _image->cols(), _image->rows(),
        GL_LUMINANCE, GL_UNSIGNED_BYTE, (GLubyte*) _image->buffer());
	else if ( gluBuild2DMipmaps(GL_TEXTURE_2D, _image->channels(), _image->cols(), _image->rows(),
					  (_image->channels() == 1) ? GL_LUMINANCE :
					  (_image->channels() == 2) ? GL_LUMINANCE_ALPHA :
					  (_image->channels() == 3) ? GL_RGB :
					  GL_RGBA,
					  GL_UNSIGNED_BYTE, (GLubyte*) _image->buffer()))
	    return ddgErrorSet(Render,"MipMap failed");



    return ddgSuccess;
}  

// ----------------------------------------------------------------------
// activate:
bool ddgTexture::activate(void)
{
	// set up the texturing parameters for OpenGL.

	if (_mode == MODULATE)
	{
		glColor4fv(_color.v);
	}
	glMatrixMode(GL_TEXTURE);
	glLoadIdentity();
	if (_scalewidth != (float)_image->cols() || _scaleheight != (float)_image->rows())
	{
		// Set the scaling for the texture map.
		glScalef(_scalewidth / (float)_image->cols(),
				 _scaleheight / (float)_image->rows(),1.0);
	}
	glMatrixMode(GL_MODELVIEW);
	// Once the texture array has been set up correctly
    glBindTexture(GL_TEXTURE_2D, _textureID);
    // drawing colour.
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE,
        (_mode == MODULATE) ? GL_MODULATE : ( (_mode == BLEND) ? GL_BLEND :GL_DECAL));
    if (_mode == BLEND)
    {
        glTexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, _color.v);
    }
	glEnable(GL_TEXTURE_2D);

	return ddgSuccess;
}

// ----------------------------------------------------------------------
// deactivate:
bool ddgTexture::deactivate( void )
{
    // Switch off texturing...
    glDisable(GL_TEXTURE_2D);
    return ddgSuccess;
}


