/*
    Copyright (C) 1997, 1998, 1999 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
// GL includes
#include <GL/glut.h>
//
#include "util/ddgerror.h"
#include "render/ddgtorus.h"
// ----------------------------------------------------------------------

bool ddgTorus::init(ddgContext *c )
{
    if (ddgNode::init(c)) return ddgFailure;
    
    return ddgSuccess;
}


bool ddgTorus::draw(ddgContext*)
{
	float PI = 3.141596;
	int i, j;
	float theta, phi, theta1;
	float cosTheta, sinTheta;
	float cosTheta1, sinTheta1;
	float ringDelta, sideDelta;
	
	ringDelta = 2.0 * PI / _rings;
	sideDelta = 2.0 * PI / _nsides;
	theta = 0.0;
	cosTheta = 1.0;
	sinTheta = 0.0;

	glColor3ubv(_color.v);
	for (i = _rings - 1; i >= 0; i--) {
		theta1 = theta + ringDelta;
		cosTheta1 = cos(theta1);
		sinTheta1 = sin(theta1);
		glBegin(GL_QUAD_STRIP);
		phi = 0.0;
		for (j = _nsides; j >= 0; j--) {
		  GLfloat cosPhi, sinPhi, dist;

		  phi += sideDelta;
		  cosPhi = cos(phi);
		  sinPhi = sin(phi);
		  dist = _R + _r * cosPhi;

		  glNormal3f(cosTheta1 * cosPhi, -sinTheta1 * cosPhi, sinPhi);
		  glVertex3f(cosTheta1 * dist, -sinTheta1 * dist, _r * sinPhi);
		  glNormal3f(cosTheta * cosPhi, -sinTheta * cosPhi, sinPhi);
		  glVertex3f(cosTheta * dist, -sinTheta * dist,  _r * sinPhi);
		}
		glEnd();
		theta = theta1;
		cosTheta = cosTheta1;
		sinTheta = sinTheta1;
	}
	return false;
}

void ddgTorus::outer(float R)
{
	_R = R;
}

void ddgTorus::inner(float r)
{
	_r = r;
}

void ddgTorus::nsides(float n)
{
	_nsides = n;
}

void ddgTorus::rings(float r)
{
	_rings = r;
}

