/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "render/ddgtransform.h"
#include "render/ddgcamera.h"

// Generic billboard rotation.
void
buildRot(float theta, float x, float y, float z, float m[16]) {
    float d = x*x + y*y + z*z;
    float ct = cosf(ddgAngle::degtorad(theta)), st = sinf(ddgAngle::degtorad(theta));

    /* normalize */
    if (d > 0) {
	d = 1/d;
	x *= d;
	y *= d;
	z *= d;
    }

    m[ 0] = 1; m[ 1] = 0; m[ 2] = 0; m[ 3] = 0;
    m[ 4] = 0; m[ 5] = 1; m[ 6] = 0; m[ 7] = 0;
    m[ 8] = 0; m[ 9] = 0; m[10] = 1; m[11] = 0;
    m[12] = 0; m[13] = 0; m[14] = 0; m[15] = 1;

    /* R = uu' + cos(theta)*(I-uu') + sin(theta)*S
     *
     * S =  0  -z   y    u' = (x, y, z)
     *	    z   0  -x
     *	   -y   x   0
     */

     m[0] = x*x + ct*(1-x*x) + st*0;
     m[4] = x*y + ct*(0-x*y) + st*-z;
     m[8] = x*z + ct*(0-x*z) + st*y;

     m[1] = y*x + ct*(0-y*x) + st*z;
     m[5] = y*y + ct*(1-y*y) + st*0;
     m[9] = y*z + ct*(0-y*z) + st*-x;

     m[2] = z*x + ct*(0-z*x) + st*-y;
     m[6] = z*y + ct*(0-z*y) + st*x;
     m[10]= z*z + ct*(1-z*z) + st*0;
}

bool ddgTransform::draw( ddgContext *ctx )
{
	bool retVal;
	glMatrixMode(GL_MODELVIEW);
	if (fTranslate || fScale || fRotate || fSphericalBillBoard || fCylindricalBillBoard)
		glPushMatrix();
	if (fCylindricalBillBoard)
	{
		ddgMatrix4 *mmat;
		// Get the modelview matrix
		mmat = ctx->transformation();
		static float m[16] = {1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1};
		m[ 0] = (*mmat)[0];
		m[ 2] = -1*(*mmat)[2];
		m[ 8] = (*mmat)[2];
		m[10] = (*mmat)[0];

		glMultMatrixf(m);

	}
	else if (fSphericalBillBoard)
	{
		ddgMatrix4 *matinv;

		// Get the inverse matrix
		matinv = ctx->itransformation();
	
		glMultMatrixf(*matinv);
	}
	if (fTranslate)
		glTranslatef(_translate[0],_translate[1],_translate[2]);
	if (fScale)
		glScalef(_scale[0],_scale[1],_scale[2]);
	if (fRotate)
	{
		glRotatef(_rotate[0],1,0,0);
		glRotatef(_rotate[1],0,1,0);
		glRotatef(_rotate[2],0,0,1);
	}
	retVal = super::draw(ctx);
	if (fTranslate || fScale || fRotate || fSphericalBillBoard || fCylindricalBillBoard)
		glPopMatrix();
	return retVal;
}