/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
// 
#include "struct/ddgecos.h"
#include "render/ddgtree.h"
#include "render/ddgtexture.h"
#include "render/ddgcamera.h"

/// Image used for displayed texture.
const unsigned int ddgNoTreeType = 7;
const unsigned int ddgNoTreeImages = 7;
static char *g_treeImages[ddgNoTreeImages] =
	{ 
	"plants/oak.tga",
	"plants/pine.tga",
	"plants/fern.tga",
	"plants/birch1.tga",
	"plants/birch2.tga",
	"plants/bush.tga",
	"plants/palm.tga",
};
typedef struct ddgTreeStructType
{
	unsigned int img;
	float px;	// coord 1
	float py;	// range [0-1]
	float sx;	// coord 2
	float sy;	// rangge[0-1]
} ddgTreeStruct;
ddgTreeStruct g_treeStruct[ddgNoTreeType] = {
	{0,0,0,1,1},	// oak
	{1,0,0,1,1},	// pine
	{2,0,0,1,1},	// fern
	{3,0,0,1,1},	// birch
	{4,0,0,1,1},	// birch
	{5,0,0,1,1},	// bush
	{6,0,0,1,1},	// palm
};
ddgTexture	*g_treeTexture[ddgNoTreeType];
bool ddgTree::ddgTreeInit = false;

static bool loadTreeData(void)
{
	ddgStr		s;
	// Load the images & create textures
	int i = 0;
	while (i < ddgNoTreeImages)
	{
		ddgImage *cImg = new ddgImage();
		ddgMemorySet(ddgImage,1);
		s.assign(g_treeImages[i]);
		if (cImg->readFile(s))
		{
			g_treeTexture[i] = NULL;
			return ddgFailure;
		}
		else
		{
			// Convert 3 channel color Alpha to true 4 channel alpha.
			if (cImg->channels() == 3 && cImg->mapColorToAlpha() == ddgFailure)
				return ddgFailure;
			g_treeTexture[i] = new ddgTexture(cImg,true,true,ddgTexture::MODULATE);
			ddgMemorySet(ddgTexture,1);
			g_treeTexture[i]->init();
		}
		i++;
	}
	return ddgSuccess;
}

ddgTree::ddgTree(ddgTreeType t)
{
	_type = t;
}

bool ddgTree::init(ddgContext *ctx)
{
    if (super::init(ctx)== ddgFailure)
        return ddgFailure;

    if (!ddgTreeInit)
	{
		ddgTreeInit = true;
		if (loadTreeData() == ddgFailure)
			return ddgFailure;
	}
	return ddgSuccess;
}

bool ddgTree::draw(ddgContext *ctx)
{
	ddgTreeStructType *tst = &(g_treeStruct[_type]);

	glEnable(GL_ALPHA_TEST);
	glAlphaFunc(GL_GEQUAL,0.9);
	g_treeTexture[tst->img]->activate();
    glBegin(GL_QUADS);
	glNormal3f(0,0,-1);


	// Bottom right
	glTexCoord2f(tst->px+tst->sx,		tst->py);
	glVertex3f(0.5,0,0); 

	// Top right
	glTexCoord2f(tst->px+tst->sx,tst->py+tst->sy);
	glVertex3f(0.5,1,0); 

	// Top Left 1
	glTexCoord2f(tst->px,      tst->py+tst->sy);
	glVertex3f(-0.5,1,0); 

	// Bottom left 1
	glTexCoord2f(tst->px,		tst->py);
	glVertex3f(-0.5,0,0); 

	glEnd();
	g_treeTexture[_type]->deactivate();
    glDisable(GL_ALPHA_TEST);

	return ddgSuccess;
}
// Init
bool ddgForest::init(ddgContext *ctx)
{
	if (_ecoSystem->init() == ddgFailure)
		return ddgFailure;

	if (ddgGroup::size() == 0)
	{
		unsigned int b = 0;
		while (b < _ecoSystem->blockNum)
		{
			ddgEcoBlock *peb = _ecoSystem->ecoBlocks[b];
			add(new ddgForestBlock(_tree,peb,false));
			b++;
		}

		if ((_tree->init(ctx) == ddgFailure)
		  ||(super::init(ctx) == ddgFailure) )
			return ddgFailure;
	}
	return ddgSuccess;
}

bool ddgForest::draw(ddgContext *ctx)
{
	ddgTreeStructType *tst = &(g_treeStruct[_tree->_type]);

	// Check the visibility of the blocks:
	unsigned int b = 0;
	ddgVector2 campos(ctx->control()->position()->v[0], ctx->control()->position()->v[2]);
	if (ctx->quality() != ddgContext::DEBUG)
	{
		while (b < _ecoSystem->blockNum)
		{
			_ecoSystem->ecoBlocks[b]->vis(ctx->rootHull(),campos);
			b++;
		}
	}

	if (ctx->quality() != ddgContext::DEBUG)
	{
		glEnable(GL_ALPHA_TEST);
		glAlphaFunc(GL_GEQUAL,0.9);
		glDisable(GL_CULL_FACE);
		g_treeTexture[tst->img]->activate();
		glColor3f(1,1,1);
	}
	super::draw(ctx);
	if ( ctx->quality() != ddgContext::DEBUG)
	{
		g_treeTexture[_tree->_type]->deactivate();
		glEnable(GL_CULL_FACE);
		glDisable(GL_ALPHA_TEST);
	}
	return ddgSuccess;
}

bool ddgForestBlock::init(ddgContext *ctx)
{
	ddgTreeStructType *tst = &(g_treeStruct[_tree->_type]);

	unsigned int i;
	ddgVector3 *pos;
	ddgVector3 p;
	ddgVector2 t;
	ddgVector3 s = _eb->osize();

	// 8 points per tree.
	varray.size(_eb->size()*4,_eb->size()*4);
	varray.renderMode(true,false,false);
	if (super::init(ctx) == ddgFailure)
		return ddgFailure;
	super::reset();

	ddgVBIndex index[4];
	for (i = 0; i < _eb->size(); i++)
	{
		pos = &(_eb->pos[i]);
		// Bottom right
		t.set(tst->px+tst->sx,		tst->py);
		p = pos+ddgVector3(0,0,0.5*s[2]); 
		index[0] = varray.pushVT(&p,&t);
		// Top right
		t.set(tst->px+tst->sx,tst->py+tst->sy);
		p = pos+ddgVector3(0,s[1],0.5*s[2]); 
		index[1] = varray.pushVT(&p,&t);
		// Top Left 1
		t.set(tst->px,      tst->py+tst->sy);
		p = pos+ddgVector3(0,s[1],-0.5*s[2]); 
		index[2] = varray.pushVT(&p,&t);
		// Bottom left 1
		t.set(tst->px,		tst->py);
		p = pos+ddgVector3(0,0,-0.5*s[2]); 
		index[3] = varray.pushVT(&p,&t);

		varray.pushQuad(index[0],index[1],index[2],index[3]);
	}
	// Initialize the display lists.
	draw(ctx);
	return ddgSuccess;
}

bool ddgForestBlock::draw(ddgContext *ctx)
{
	if (_eb->vis() && ctx->quality() != ddgContext::DEBUG)
	{
		if (!listValid())
		{
			listStart();
			super::draw(ctx);
			listEnd();
		}


		//listRender();
		ddgTreeStructType *tst = &(g_treeStruct[_tree->_type]);
		ddgMatrix4 *mmat;
		// Get the modelview matrix
		mmat = ctx->transformation();
		static float m[16] = {1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1};
		m[ 0] = (*mmat)[0];
		m[ 2] = -1*(*mmat)[2];
		m[ 8] = (*mmat)[2];
		m[10] = (*mmat)[0];
		unsigned int i;
		for (i = 0; i < _eb->size(); i++)
		{
			glPushMatrix();
			glTranslatef(_eb->pos[i].v[0],_eb->pos[i].v[1],_eb->pos[i].v[2]);

			glMultMatrixf(m);
			// Bottom right
			glBegin(GL_QUADS);
			glTexCoord2f(tst->px+tst->sx,		tst->py);
			glVertex3f(0.5,0,0); 

			// Top right
			glTexCoord2f(tst->px+tst->sx,tst->py+tst->sy);
			glVertex3f(0.5,1,0); 

			// Top Left 1
			glTexCoord2f(tst->px,      tst->py+tst->sy);
			glVertex3f(-0.5,1,0); 

			// Bottom left 1
			glTexCoord2f(tst->px,		tst->py);
			glVertex3f(-0.5,0,0); 
			glEnd();

			glPopMatrix();
		}
	}

	if ( ctx->quality() == ddgContext::DEBUG)
	{
		ddgInside v = _eb->vis();
		// Draw the bounding box.
		ddgVector3 p(_eb->bbox->min);
		ddgVector3 q(_eb->bbox->max);
		if (v==ddgOUT)
			glColor3ub(255,0,0);
		if (v==ddgPART)
			glColor3ub(100,100,100);
		if (v==ddgIN)
			glColor3ub(0,255,0);
		if (v==ddgUNDEF)
			glColor3ub(0,255,255);
		glBegin(GL_LINE_LOOP);
		glVertex3f(p[0],p[1],p[2]);
		glVertex3f(p[0],p[1],q[2]);
		glVertex3f(q[0],p[1],q[2]);
		glVertex3f(q[0],p[1],p[2]);
		glEnd();

		glBegin(GL_LINE_LOOP);
		glVertex3f(p[0],q[1],p[2]);
		glVertex3f(p[0],q[1],q[2]);
		glVertex3f(q[0],q[1],q[2]);
		glVertex3f(q[0],q[1],p[2]);
		glEnd();
		
		glBegin(GL_LINES);
		glVertex3f(p[0],p[1],p[2]);	glVertex3f(p[0],q[1],p[2]);
		glVertex3f(p[0],p[1],q[2]); glVertex3f(p[0],q[1],q[2]);
		glVertex3f(q[0],p[1],q[2]); glVertex3f(q[0],q[1],q[2]);
		glVertex3f(q[0],p[1],p[2]); glVertex3f(q[0],q[1],p[2]);
		glEnd();
	}
	return ddgSuccess;
}

