/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "util/ddgerror.h"
#include "render/ddgvbuf.h"
#include "glrender/ddgoglxt.h"
// ----------------------------------------------------------------------
// VBuffer
// ----------------------------------------------------------------------

bool ddgVBuffer::init(ddgContext *ctx)
{

    if (super::init(ctx))
        return ddgFailure;

	return varray.init();
}

bool ddgVBuffer::draw(ddgContext *ctx)
{
	bool retval = ddgSuccess;

	if (varray.inum() == 0)
		return retval;

	if (_rstart == 0)
	{
		if (varray.textureOn())
		{
			glEnableClientState (GL_TEXTURE_COORD_ARRAY);
			if (varray.tbuf)
				glTexCoordPointer (2, GL_FLOAT, 0, varray.tbuf);
			else
				glTexCoordPointer (2, GL_FLOAT, 0, varray.tibuf);
		}

		if (varray.colorOn())
		{
			// 4 color components.
			glEnableClientState (GL_COLOR_ARRAY);
			glColorPointer( 4, GL_UNSIGNED_BYTE, 0, varray.cbuf );
		}

		if (varray.normalOn())
		{
			// Normal array for rendering.
			glEnableClientState (GL_NORMAL_ARRAY);
			glNormalPointer(GL_FLOAT,  12, varray.nbuf); // 3 floats = 16 bytes.
		}

		// Vertex array for rendering.
		glEnableClientState (GL_VERTEX_ARRAY);
		glVertexPointer ( 3, GL_FLOAT, 12, varray.vbuf); // 3 floats = 16 bytes.
		// Lock the arrays
		if (ddgOpenGLExt::lockedArrays())
			ddgOpenGLExt::lockArrays(0, varray.inum());
	}
	static int mode[5] = { 0, GL_POINTS, GL_LINES, GL_TRIANGLES, GL_QUADS};

	if (_rnum)
		glDrawElements(mode[varray.type()], _rnum*varray.type(), GL_UNSIGNED_INT, &varray.ibuf[_rstart*varray.type()]);
	else
		glDrawElements(mode[varray.type()], varray.inum(), GL_UNSIGNED_INT, varray.ibuf);

	if (((_rnum == 0)|| (_rnum + _rstart == varray.inum()/varray.type())))
	{
		if (ddgOpenGLExt::lockedArrays())
			ddgOpenGLExt::unlockArrays();

		glDisableClientState (GL_VERTEX_ARRAY);
		if (varray.normalOn())  glDisableClientState (GL_NORMAL_ARRAY);
		if (varray.colorOn())  glDisableClientState (GL_COLOR_ARRAY);
		if (varray.textureOn()) glDisableClientState (GL_TEXTURE_COORD_ARRAY);
		_rnum = 0;
		_rstart = 0;
	}

    return retval;
}

