/*
    Copyright (C) 1997, 1998, 1999, 2000 by Alex Pfaffe
	(Digital Dawn Graphics Inc)
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "gl/glut.h"
#include "render/ddgviewport.h"
#include "util/ddgerror.h"
#include "glrender/ddgoglxt.h"

// ----------------------------------------------------------------------

ddgViewportSet *ddgViewportSet::_viewportset = 0;

#define DDGMODEMAX 13

typedef struct ddgFSMode_Struct {
	unsigned int width;
	unsigned int height;
	unsigned int depth;
	unsigned int frequency;
} ddgFSMode;
ddgFSMode g_ddgFSModes[DDGMODEMAX] = { {0,0,0,0},
	{640,480,16,60},
	{800,600,16,60},
	{1024,768,16,60},
	{1152,870,16,60},
	{1280,1024,16,60},
	{1600,1200,16,60}, 
	{640,480,32,60},
	{800,600,32,60},
	{1024,768,32,60},
	{1152,870,32,60},
	{1280,1024,32,60},
	{1600,1200,32,60} };

ddgViewport::ddgViewport( ddgCamera *c, unsigned short x, unsigned short y, unsigned short w, unsigned short h)
	: _camera(c), _slave(false), _control(true), _px(x), _py(y), _dx(w), _dy(h)
{
	_enabled = _edge = true;
	_id = 0;
	_update = 0;
	_eventHandler = new ddgViewportEventHandler;
}

void ddgViewportSet::name( char *n )
{
	_name.assign(n);
	glutSetWindowTitle(n);
}

char * ddgViewportSet::mode( unsigned int m, char *buf, unsigned int bufsize)
{
#ifdef DDGSTREAM
	ostrstream modeDescriptor(buf,bufsize);
	modeDescriptor << g_ddgFSModes[m].width << "x" << g_ddgFSModes[m].height
		 << ":" << g_ddgFSModes[m].depth << "@" << g_ddgFSModes[m].frequency;
	return buf;
#else
	return NULL;
#endif
}

bool ddgViewportSet::checkForError(void)
{
	// Check for error state.
	static GLenum errorCode = 0;
	static GLenum lastErrorCode = 0;
	const GLubyte *errorString;
	if ((errorCode = glGetError()) != GL_NO_ERROR)
	{
		errorString = gluErrorString(errorCode);
		if (errorCode!=lastErrorCode)
		{
			ddgErrorSetStr(Render, "Code " << errorCode << errorString);
			ddgError::report();
			lastErrorCode = errorCode;
		}
		return ddgFailure;
	}
	return ddgSuccess;
}


int ddgViewportSet::init( int argc, char **argv)
{
	int id;
	ddgViewport *vp = (ddgViewport*)head();
	ddgViewport *root;
	ddgViewportSet::_viewportset = this;
	ddgAssert(vp);
	while (vp)
    {
		if (vp == head())
		{
			root = vp;
			glutInit(&argc, argv);
			if (_fullscreen && _fsmode < DDGMODEMAX)
			{
				vp->px(0);
				vp->py(0);
				vp->dx(g_ddgFSModes[_fsmode].width);
				vp->dy(g_ddgFSModes[_fsmode].height);
			}
			glutInitWindowPosition(vp->px(), vp->py());
			glutInitWindowSize(vp->dx(), vp->dy());
			// WARNING GLUT_RGBA is actuall RGB!  So DESTINATION_ALPHA does not work for blending.
			glutInitDisplayMode( GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA | GLUT_STENCIL );
            if (_fullscreen && _fsmode < DDGMODEMAX) {
				char * s = new char[32];
				mode(_fsmode,s,32);
				
				glutGameModeString(s);
				glutEnterGameMode();
				id = 1;
            }
            else
            {
    			id = glutCreateWindow( _name );
			    glutReshapeFunc(ddgViewportSet::reshape);
            }
			// Once we have a root window, initialize extentions.
#ifdef DDGGEFORCE
			ddgOpenGLExt::init();
#endif
		}
		else
		{
			id = glutCreateSubWindow( root->id(),
				root->dx()*vp->px()/100,root->dy()*vp->py()/100,
				root->dx()*vp->dx()/100,root->dy()*vp->dy()/100 );
		}
		ddgAssert(id);

		vp->id(id);
		if (vp->camera())
		{
			glutDisplayFunc(ddgViewportSet::draw);
			if (!vp->slave())
			{
				glutIdleFunc(ddgViewportSet::idle);
				glutVisibilityFunc(ddgViewportSet::visible);
				glutMouseFunc(ddgViewportSet::mouse);
				glutMotionFunc(ddgViewportSet::mouseMotion);
				glutKeyboardFunc(ddgViewportSet::key);
				glutSpecialFunc(ddgViewportSet::specialKey);
			}

			if ( vp->camera()->init())
			{
		        ddgErrorSet(Render,"Error initializing viewport");
		        ddgError::report();
				return ddgFailure;
			}
		}
		// Initialize the ui objects if any.
		if ( vp->uigroup()->init(vp->camera()->context()))
			return ddgFailure;

		vp = (ddgViewport*) vp->next();
    }

	// Check for error state.
	return checkForError();
}

ddgViewport *ddgViewportSet::hitTest(int x, int y)
{
	ddgViewport *vp = (ddgViewport*)ddgViewportSet::_viewportset->head();
	float px = 100 * x / vp->dx();
	float py = 100 * y / vp->dy();
	while (vp = (ddgViewport*) vp->next())
	{
		if (   px > vp->px()
			&& py > vp->py()
			&& px < vp->px() + vp->dx()
			&& py < vp->py() + vp->dy())
			break;
	}
	if (!vp) vp = (ddgViewport*)ddgViewportSet::_viewportset->head();

	// See if viewport is allowed to control.
	if (vp->control())
		glutSetWindow(vp->id());
	else
		vp = 0;
	return vp;
}

void ddgViewportSet::enable(void)
{
	glEnable(GL_SCISSOR_TEST);
	glutMainLoop();
}

void ddgViewportSet::draw( void )
{
	ddgViewport *root = (ddgViewport*)ddgViewportSet::_viewportset->head();
	ddgViewport *vp;
	vp = ddgViewportSet::_viewportset->find(glutGetWindow());
	ddgAssert(vp);

#ifdef _DEBUG
	// Check for error state.
	checkForError();
#endif
	glutSetWindow(vp->id());

	// Update the camera.
	if (vp->camera())
		vp->camera()->update();

	// Update the viewport.
	if (vp->update())
		vp->doUpdate();

	if (vp == root)
	{
		glScissor(0, 0, vp->dx(), vp->dy());
	}
	else
	{
		glScissor(0,0,root->dx()*vp->dx()/100,root->dy()*vp->dy()/100 );
	}
	vp->camera()->draw();

	if (vp->uigroup()->draw( vp->camera()->context() ))
	{
		ddgErrorSet(Render,"UI Rendering error");
		ddgError::report();
		exit(ddgError::Render);
	}

	if (vp != ddgViewportSet::_viewportset->head() && vp->edge())
	{
		// Render a thin border around the inset window.
		glMatrixMode(GL_PROJECTION);
		glPushMatrix();
		glLoadIdentity();
		glOrtho(-0.5,99.5,-0.5,99.5,-1.0,1.0);

		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();

		if (vp == ddgViewportSet::_viewportset->_current)
			glColor3ub(255,0,0);
		else
			glColor3ub(255,255,255);
		glBegin(GL_LINE_LOOP);
		glVertex3f(-0.4,-0.4,0.0);
		glVertex3f(-0.4,99.4,0.0);
		glVertex3f(99.4,99.4,0.0);
		glVertex3f(99.4,-0.4,0.0);
		glEnd();

		glMatrixMode(GL_PROJECTION);
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
	}
	glutSwapBuffers();

	// If we are the root, update any slaves.
	if (vp == (ddgViewport*)ddgViewportSet::_viewportset->head())
	{
		while (vp = (ddgViewport*) vp->next())
		{
			if (vp->slave())
			{
				glutSetWindow(vp->id());
				glutPostRedisplay();
			}
		}
	}
}

void ddgViewportSet::idle( void )
{
	ddgViewport *vp = ddgViewportSet::_viewportset->find(glutGetWindow());
	ddgAssert(vp);
	// If we are a slave update the root.
	if (vp->slave())
		vp = (ddgViewport*)ddgViewportSet::_viewportset->head();

	glutSetWindow(vp->id());
	glutPostRedisplay();
}

// If the viewport is obscured, disable the idle loop completely.
void ddgViewportSet::visible( int vis )
{
	if (vis != GLUT_VISIBLE)
	{ 
		glutIdleFunc(NULL);
	}
	else
	{ 
		glutIdleFunc(ddgViewportSet::idle);
	}
}

void ddgViewportSet::reshape( int w, int h )
{
	ddgViewport *vp;
	ddgViewport *root = (ddgViewport*)ddgViewportSet::_viewportset->head();
	ddgAssert(root);
	if (w == 0 ) w = root->dx();
	if (h == 0 ) h = root->dy();
	glViewport(0,0,w,h);
	glutSetWindow(root->id());
	glutPositionWindow(root->px(),root->py());
	glutReshapeWindow(root->dx(w),root->dy(h));
	vp = (ddgViewport*) root->next();
	while (vp)
	{
		glutSetWindow(vp->id());
		glutPositionWindow( root->dx()*vp->px()/100,root->dy()*vp->py()/100);
		glutReshapeWindow(  root->dx()*vp->dx()/100,root->dy()*vp->dy()/100 );
		vp = (ddgViewport*) vp->next();
	}
}

void ddgViewportSet::doShutdown(ddgError *e)
{
	// TODO: Break the wait loop.
	if (_fullscreen)
		glutLeaveGameMode();

	// Crashes in W2K
//	glutDestroyWindow(((ddgViewport *)head())->id());
	if (shutdown())
	{
		// If true if shutdown should be aborted.
		if (!_shutdown())
			exit(e ? e->report(): 0);
	}
	else
		exit(e ? e->report() :0);
}

void ddgViewportSet::key( unsigned char key, int x, int y )
{
	ddgViewportSet *vps = ddgViewportSet::_viewportset;
	ddgViewport *vp = vps->hitTest(x,y);
	if (!vp) return;

	if (vp && vp->eventHandler()->keyHandler()(vp->eventHandler()->handlerObject(),key,x,y))
		glutPostRedisplay();
}

void ddgViewportSet::specialKey( int key, int x, int y )
{
	ddgViewport *vp = ddgViewportSet::_viewportset->hitTest(x,y);
	if (!vp) return;
	if (vp->eventHandler()->specialKeyHandler()(vp->eventHandler()->handlerObject(),key,x,y))
		glutPostRedisplay();
}

void ddgViewportSet::mouse( int button, int state, int x, int y )
{
	ddgViewport *vp = ddgViewportSet::_viewportset->find(glutGetWindow());
	if (vp && vp->eventHandler()->mouseHandler()(vp->eventHandler()->handlerObject(),button,state,x,y))
	    glutPostRedisplay();
}
void ddgViewportSet::mouseMotion( int x, int y )
{
	ddgViewport *vp = ddgViewportSet::_viewportset->find(glutGetWindow());
	if (vp && vp->eventHandler()->mouseMotionHandler()(vp->eventHandler()->handlerObject(),x,y))
		glutPostRedisplay();
}


/// Event handler, returns true if event was handled.
bool ddgViewportSet::smouse(void *obj, int button, int state, int x, int y)
{
	return false;
}
/// Event handler, returns true if event was handled.
bool ddgViewportSet::smouseMotion(void *obj, int x, int y)
{
	return false;
}
/// Event handler, returns true if event was handled.
bool ddgViewportSet::sspecialKey(void *obj, int k, int x, int y)
{
	return false;
}
/// Event handler, returns true if event was handled.
bool ddgViewportSet::skey(void *obj, unsigned char k, int x, int y)
{
	ddgViewportSet *vps = ddgViewportSet::_viewportset;
	ddgViewport *vp = vps->hitTest(x,y);
	if (!vp) return false;

	switch (k) {
	case 27:
		// Escape to exit.
		vps->doShutdown(NULL);
		break;
	case '~':
		vp = (ddgViewport*)vps->head();
		if (vps->_current && vps->_current != vp)
		{
			ddgCamera *c = vp->camera();
			vp->camera(vps->_current->camera());
			vps->_current->camera(c);
		}
		break;
	case '+':
		if (vps->_current)
		{
			vps->_current = (ddgViewport*)vps->_current->next();
		}
		if (vps->_current == 0)
			vps->_current = (ddgViewport*)vps->head()->next();
		break;
	case '[':
		if (vps->_current)
		{
			int p = vps->_current->px() - 10;
			if (p < 0)
				p = 0;
			vps->_current->px(p);
			reshape(0,0);
		}
		break;
	case ']':
		if (vps->_current)
		{
			int p = vps->_current->px() + 10;
			if (p > 100 - vps->_current->dx())
				p = 100 - vps->_current->dx();
			vps->_current->px(p);
			reshape(0,0);
		}
		break;
	case '{':
		if (vps->_current)
		{
			int p = vps->_current->py() - 10;
			if (p < 0)
				p = 0;
			vps->_current->py(p);
			reshape(0,0);
		}
		break;
	case '}':
		if (vps->_current)
		{
			int p = vps->_current->py() + 10;
			if (p > 100 - vps->_current->dy())
				p = 100 - vps->_current->dy();
			vps->_current->py(p);
			reshape(0,0);
		}
		break;
	case '?':
#ifdef DDGSTREAM
		cerr << "Viewport Commands:\n"
		"ESC ................ Quit program.\n"
		"+ .................. Make next viewport active.\n"
		"~ .................. Swap primary and active viewport.\n"
		"[/] ................ Move active window left/right.\n"
		"{/} ................ Move active window up/down.\n"
		"# .................. Report settings.\n";
#endif
		break;
	case '#':
#ifdef DDGSTREAM
		cerr << "Current settings:\n";
#endif
		break;
	default:
		return false;
	}
	return true;
}

ddgViewportEventHandler::ddgViewportEventHandler(void)
{
	set(this, ddgViewportSet::skey, ddgViewportSet::sspecialKey, ddgViewportSet::smouse, ddgViewportSet::smouseMotion);
}